//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Cluster autodetection
//

register_command("auto", "Run automatic cluster detection");

function auto_help() {
    local command_help = {
	name = "auto"
	usage = "auto [-n <host count>] [-t <timeout>] [-d <delay>]"
	description = "Automatically scan computers executing parcompmark applications with broadcast messages."
	parameters = [
	    {
		parameter="-n <host count>"
		description="Wait for specified number of hosts. If not specified, the `auto\' command will scan only once."
	    }
	    {
		parameter="-t <timeout>"
		description="Timeout for seaching hosts. The default value is 3 seconds."
	    }
	    {
		parameter="-d <delay>"
		description="Delay between 2 search. The default value is 0.5 second."
	    }
	]
	see_also = ["lshosts"]
    };
    
    print_command_help(command_help);
}

function auto(args) {

    local hostCount = 1;
    local timeout = 3.0;
    local delay = 0.5;
    
    local hostFound = 0;

    // Process command line
    while(args.len() > 0) {
        switch(args[0]) {
	    case "-n":
		if(args.len() > 1) {
		    args = args.slice(1); hostCount = args[0].tointeger();
		}
		else {
		    auto_help();
		    return;
		}
	    break;
	    case "-t":
		if(args.len() > 1) {
		    args = args.slice(1); timeout = args[0].tofloat();
		}
		else {
		    auto_help();
		    return;
		}
	    break;
	    case "-d":
		if(args.len() > 1) {
		    args = args.slice(1); delay = args[0].tofloat();
		}
		else {
		    auto_help();
		    return;
		}
	    break;
	    default:
		auto_help();
		return;
	}

        if(args.len() > 1) args = args.slice(1);
	else break;
    }
			
    print("Scanning cluster...\n");
    local startTime = Timer.getApplicationTime();

    do {
	// Scan cluster
	Application.getInstance().autoDetection();
	
	// Number of found hosts
	hostFound = stringToObject(Application.getInstance().getHostListForSquirrel()).len();
	
	// If timeout then exit
	if(Timer.getApplicationTime() - startTime >= timeout) {
	    print("Searching time is over.\n");
	    break;
	}
	
	// If needed wait for a while
	if(hostFound < hostCount) {
	    Timer.sleep(delay);
	}
    } while(hostFound < hostCount)

    // Initialize dynamic script parameters in complement mode
    // This is needed to find out host related possible value and default value fields    
    if(this.rawin("DYNAMICSCRIPTPARAMETERS"))
	initializeDynamicScriptParameters(false);
    
    lshosts([]);
}
