//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Printing help
//

// Register command with name and short description
function register_command(nameStr, descriptionStr) {
    if(!this.rawin("COMMANDS")) COMMANDS<-[];
	COMMANDS.append({
	name=nameStr
	description=descriptionStr
    });
}

// Compare two commands for sorting
function compare_command(c1, c2) {
    if(c1.name > c2.name) return 1;
    else if(c1.name < c2.name) return -1;
    return 0;
}

// Find description for the selected commands
function find_description_for_command(nameStr) {
    foreach(command in COMMANDS)
	if(command.name == nameStr) 
	    return command.description;
	    
    throw ("Can not find command `" + nameStr + "\'");
}

register_command("help", "Show help");

function help(args) {

    // Help on command
    if(args.len() == 1) {
	compilestring(args[0]+"_help()")();
    }

    // General help
    else {
        print("This is the command line of ParCompMark (Parallel Compositing Benchmark Framework)\n");
	print(Application.getInstance().getProgramInfo() + "\n\n");

        // Sort the registered commands
	COMMANDS.sort(compare_command);

        // List available commands
	print("There are the following commands in this shell:\n");    
        foreach(command in COMMANDS) {
	    print("  " + command.name);
	    for(local i=command.name.len(); i<16; i++) print(" ");
	    print("  " + command.description + "\n");
	}
    
	print("\nFor more help, type `help <command>\'!\n");
    }
}

function print_command_help(command_help) {
    print(command_help.name + " - " + find_description_for_command(command_help.name) + "\n");

    print("\n Usage:\n")
    print("    " + command_help.usage + "\n");

    print("\n Description:\n")
    print("    " + stringReplace(command_help.description, "\n", " ") + "\n");

    local paramLength = 0;
    foreach(parameter in command_help.parameters)
	if(parameter.parameter.len() > paramLength)
	    paramLength = parameter.parameter.len();

    foreach(parameter in command_help.parameters) {
	print("    " + parameter.parameter);
	for(local i=0;i<paramLength-parameter.parameter.len()+3;i++) print(" ");
	print(parameter.description + "\n");
    }

    if(command_help.see_also.len() != 0) {
	print("\n See also:\n    ");
	foreach(see_also in command_help.see_also)
	    print(see_also + " ");
	print("\n");
    }
}
