//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Load script
//

register_command("load", "Load scenario, dynamic, or low-level script");

function load_help() {
    local command_help = {
	name = "load"
	usage = "load [-s|-d|-l] <script_file>"
	description = @"Load dynamic or low-level script. If the `script_file' does not have `.nut' extension it will be completed. 
If none of `-l\' and `-d\' flags are specified then the script will be considered as a scenario script."
	parameters = [
            {
	        parameter = "-s"
	        description = "The specified script file name will be considered as a scenario script"
	    }
            {
	        parameter = "-d"
	        description = "The specified script file name will be considered as a dynamic script"
	    }
            {
	        parameter = "-l"
	        description = "The specified script file name will be considered as a low-level script"
	    }
	]
	see_also = ["compile", "param"]
    };
    
    print_command_help(command_help);
}

function load(args) {

    local SCENARIO = 0, DYNAMIC = 1, LOWLEVEL = 2;

    if(!args.len())
	load_help();
    
    else {

	local scriptMode = SCENARIO;
	local scriptFile = "";

	// Handle possible flags    
	if(args[0] == "-s") {
	    if(args.len() < 2) {
		load_help();
		return;
	    }
	    else
		scriptFile = args[1];
	}    

	else if(args[0] == "-d") {
	    scriptMode = DYNAMIC;
	
	    if(args.len() < 2) {
		load_help();
		return;
	    }
	    else
		scriptFile = args[1];	    
	}    
	
	else if(args[0] == "-l") {
	    scriptMode = LOWLEVEL;

	    if(args.len() < 2) {
		load_help();
		return;
	    }
	    else
		scriptFile = args[1];
	}
	
	else
	    scriptFile = args[0];

	// Test externsion and complete if needed
	if(scriptFile.slice(-4) != ".nut")
	    scriptFile = scriptFile + ".nut";

	// Load script
	local scriptPath;
	switch(scriptMode) {
	    case SCENARIO:
		scriptPath = "scripts/scenario/" + scriptFile;
		if(FileSystemManager.getInstance().existsDataFile(scriptPath))
		    Application.getInstance().loadScenarioScript(scriptPath);
		else 
		    throw ("Scenario script `" + scriptPath + "\' is not found in the data directory!\n");
	    break;
	    case DYNAMIC:
		scriptPath = "scripts/dynamic/" + scriptFile;
		if(FileSystemManager.getInstance().existsDataFile(scriptPath))
		    Application.getInstance().loadDynamicScript(scriptPath);
		else 
		    throw ("Dynamic script `" + scriptPath + "\' is not found in the data directory!\n");
	    break;
	    case LOWLEVEL:
		scriptPath = "scripts/low-level/" + scriptFile;
		if(FileSystemManager.getInstance().existsDataFile(scriptPath))
		    Application.getInstance().loadLowLevelScript(scriptPath);
		else
		    throw ("Low-level script `" + scriptPath + "\' is not found in the data directory!\n");
	    break;
	}

	print("Script `" + scriptPath + "\' loaded.\n");

	// Clean up framework
	cleanup([]);
    }

}
