//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Dynamic script parameter handling
//

register_command("param", "Get and set dynamic script parameters");

function param_help() {
    local command_help = {
        name = "param"
        usage = "param [-l[d]] | -r | [-g <parameter>] | [-s <parameter> <value>]"
        description = ""
        parameters = [
	    {
		parameter = "-l[d]"
		description = "List all parameters of the current dynamic script. `-ld\' flag gives parameter details."
	    }
	    {
		parameter = "-r"
		description = "Reset all parameters. The parameters will be changed with the default values."
	    }
	    {
		parameter = "-g <parameter>"
		description = "Get value of the specified parameter"
	    }
	    {
		parameter = "-s <parameter> <value>"
		description = "Set value of the specified parameter"
	    }
	]
        see_also = ["compile", "load"]
    };

    print_command_help(command_help);
}

function param(args) {

    if(args.len() == 0) {
	param_help();
	return;
    }

    while(args.len() > 0) {
	switch(args[0]) {
	    case "-l":
	    case "-ld":
	    {
		local params = getDynamicScriptParameters();
		
		// Print parameters
		print("The dynamic script has the following parameters:\n");
		foreach(par in params) {
		
		    // Parameter name
		    print(par.type + stringFill(" ", 10-par.type.len()) + par.name + stringFill(" ", 20-par.name.len()));

		    // Parameter value
		    local parValue = DYNAMICSCRIPTPARAMETERS.rawin(par.name) ? DYNAMICSCRIPTPARAMETERS[par.name] : null ;
		    print((parValue!=null ? parValue : "[not defined]") + "\n");

		    // Parameter details
		    if(args[0] == "-ld") {
			print("  " + par.description + ". ");
			if(par.rawin("defaultValue") || par.rawin("possibleValues")) {
			    if(par.rawin("defaultValue")) print("Default value: " + par.defaultValue + ". ");
			    if(par.rawin("possibleValues")) { 
				print("Possible values: [");
				foreach(value in par.possibleValues)
				    print(value + (value != par.possibleValues[par.possibleValues.len()-1] ? ", " : "]."));
			    }
			}
			print("\n");
		    }
		}
	    }
	    break;
	    case "-r":
		initializeDynamicScriptParameters(true);
		print("Dynamic script parameters now have the default values.");
	    break;
	    case "-g":
	    {
		if(args.len() > 1) args = args.slice(1);
		else {
		    param_help();
		    return;
		}

		// Find parameter
		local par = findItemByName(getDynamicScriptParameters(), args[0]);
		if(!par) throw "Invalid parameter name `"+args[0]+"\'";

		// Parameter value
		local parValue = DYNAMICSCRIPTPARAMETERS.rawin(par.name) ? DYNAMICSCRIPTPARAMETERS[par.name] : null ;
		
		// Print its value
		print(par.name + (parValue!=null ? " = " + parValue : " is not defined") + "\n")
	    }
	    break;
	    case "-s":
	    {
		local name = "", value = "";

		if(args.len() > 2) {
		    args = args.slice(1); name = args[0];
		    args = args.slice(1); value = args[0];
		}
		else {
		    param_help();
		    return;
		}

		// Find parameter
		//if(!DYNAMICSCRIPTPARAMETERS.rawin(name)) throw "Invalid parameter name `"+name+"\'";
		local par = findItemByName(getDynamicScriptParameters(), name);
		if(!par) throw "Invalid parameter name `"+name+"\'";

		// Set parameter value
		switch(par.type) {
		    case "integer":
			DYNAMICSCRIPTPARAMETERS[name]<-value.tointeger();
		    break;
		    case "float":
			DYNAMICSCRIPTPARAMETERS[name]<-value.tofloat();
		    break;
		    case "boolean":
			DYNAMICSCRIPTPARAMETERS[name]<-value.tolower() == "true";
		    break;
		    default:
			DYNAMICSCRIPTPARAMETERS[name]<-value;
		}

		// Reset compilation flag
		Application.getInstance().setDynamicScriptCompiled(false);
		
		// Print its value
		print(name + " has been set.\n")
	    }
	    break;
	    default:
		param_help();
		return;
	}
	
	if(args.len() > 1) args = args.slice(1);
	else break;
    }
}
