//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Start benchmark
//

register_command("start", "Start benchmark");

function start_help() {
    local command_help = {
        name = "start"
        usage = "start [-s|-d]"
        description = @"Start the execution of the benchmark. If a dynamic script if specified (instead of low-level script), and the cluster
description retrieved by the autodetection is new than the compiled script, this command also executes dynamic script compilation. If a scenario
script file is specified then by default the scenario file is executed."
        parameters = [
            {
		parameter = "-s"
		description = "Start the execution of the loaded scenario"
	    }
	    {
	        parameter = "-d"
	        description = "Start the execution of the loaded dynamic script"
	    }
	    {
	        parameter = "-l"
	        description = "Start the execution of the loaded low-level script"
	    }
	]
        see_also = ["stop", "compile", "load"]
    };

    print_command_help(command_help);
}

function start(args) {

    local SCENARIO = 0, DYNAMIC = 1, LOWLEVEL = 2;

    if(Application.getInstance().getCompRun()) {
	print("Benchmark is already running.");
    }

    else {

        local scriptMode = Application.getInstance().hasScenarioScript() ? SCENARIO : 
	    Application.getInstance().hasDynamicScript() ? DYNAMIC : LOWLEVEL;

        // Handle possible flags    
        if(args.len() > 0 && args[0] == "-s")
            scriptMode = SCENARIO;
        else if(args.len() > 0 && args[0] == "-d")
            scriptMode = DYNAMIC;
        else if(args.len() > 0 && args[0] == "-l")
            scriptMode = LOWLEVEL;

	switch(scriptMode) {
	    case SCENARIO:
		print("Starting scenario...\n");

		// Get batch script and push to the command list
		local script = getScenarioBatchScript();
		foreach(line in script) {
		    Application.getInstance().pushSquirrelCommand(line);
		}
	    break;
	    case DYNAMIC:
		// If not compiled, compile now
		if(Application.getInstance().getDynamicScriptForSquirrel().len() > 0 && 
		 !Application.getInstance().getDynamicScriptCompiled())
		    compile([]);

		// (Continue with low-level case)

	    case LOWLEVEL:
		// Start the benchmark
		print("Starting benchmark...\n");
		Application.getInstance().start();
	    break;
	}

    }
}

