//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Common utility functions
//

function hasElement(a, e) {
    foreach(_e in a)
        if(_e == e) return true;

    return false;
}

function getIndex(a, e) {
    foreach(i, _e in a)
        if(_e == e) return i;

    return -1;
}

// Find an item in an array which has table items with `name' fields
function findItemByName(_array, name) {
    foreach(item in _array)
        if(item.name == name) return item;

    return null;
}

// Convert Squirrel object to code string

function objectToString(obj) {
    return objectToStringIndent(obj, 0, false);
}

function objectToStringIndent(obj, indent, firstLineNotIndent) {
    local str, indentStr = "";

    // Indent
    for(local i=0; i<indent; i++)
	indentStr+="  ";

    // Type-dependent string representation
    switch(typeof obj) {
        case "table":
            str = (firstLineNotIndent ? "" : indentStr) + "{\n";
            foreach(i,val in obj)
                str += indentStr + "  " + i + "=" + objectToStringIndent(val, indent+1, true);
            str+= indentStr + "}\n";
        break;
        case "array":
            str = (firstLineNotIndent ? "" : indentStr) + "[\n";
            foreach(val in obj)
                str+=objectToStringIndent(val, indent+1, false);
            str+= indentStr + "]\n";
        break;
        case "string":
            str = (firstLineNotIndent ? "" : indentStr) + (obj.find("\n") != null ?  "@" : "" ) + "\"" + obj + "\"\n";
        break;
        case "float":
            str = (firstLineNotIndent ? "" : indentStr) + obj + (obj.tostring().find(".") != null ?  "" : ".0" ) + "\n";
        break;
        default:
            str = (firstLineNotIndent ? "" : indentStr) + obj + "\n";
    }

    return str;
}

// Convert Squirrel object to xml

function objectToXML(obj) {
    return objectToXMLIndent(obj, 0, false);
}

function objectToXMLIndent(obj, indent, firstLineNotIndent) {
    local str, indentStr = "";

    // Indent
    for(local i=0; i<indent; i++)
	indentStr+="  ";

    // Type-dependent string representation
    switch(typeof obj) {
        case "table":
            str = (firstLineNotIndent ? "" : indentStr) + "<table>\n";
            foreach(i,val in obj)
                str += indentStr + "  " + "<slot name=\"" + i + "\">" + objectToXMLIndent(val, indent+1, true) + "</slot>\n";
            str+= indentStr + "</table>\n";
        break;
        case "array":
            str = (firstLineNotIndent ? "" : indentStr) + "<array>\n";
            foreach(val in obj)
                str+= indentStr + "  " + "<element>" + objectToXMLIndent(val, indent+1, false) + "</element>\n";
            str+= indentStr + "<array>\n";
        break;
        case "string":
            str = (firstLineNotIndent ? "" : indentStr) + obj;
        break;
        case "float":
            str = (firstLineNotIndent ? "" : indentStr) + obj + (obj.tostring().find(".") != null ?  "" : ".0" ) + "\n";
        break;
        default:
            str = (firstLineNotIndent ? "" : indentStr) + obj;
    }

    return str;
}

// Convert code string to Squirrel object
function stringToObject(str) {
    return compilestring("return " + str)();
}

// Replace 'from' char in 'str' to 'to' char
function stringReplace(str, from, to) {

    local _str = "";

    for(local i=0; i<str.len(); i++)
	if(str.slice(i, i+1) == from) 
	    _str += to; 
	else 
	    _str += str.slice(i, i+1);

    return _str;
}

// Creates a string with `count' number of `ch' characters
function stringFill(ch, count) {
    local str = "";
    for(local i=0;i<count;i++)
	str += ch;
    
    return str;
}
