//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Basic benchmark: Measuring rendering parameters such as light count, render mode, and
//   composite mode
//

function prepareScenario() {
    // Load dynamic script
    load(["-d", "trirenderer.nut"]);
    
    print("Scenario has been loaded.\n");
}

function getScenarioBatchScript() {

    local script = [];

    // Set default parameters
    script.append("param -s frameWidth 800");
    script.append("param -s frameHeight 600");
    script.append("param -s renderHostCount 1");

    // Triangle counts and frame counts
    local triangleCounts = [10000, 100000, 1000000];
    local frameCounts = [500, 200, 50];

    // Measure compositing type and triangle count
    foreach(compositeMode in ["de", "al"]) {
	script.append("param -s compositeMode " + compositeMode);

	for(local i=0; i<triangleCounts.len(); i++) {
	    script.append("param -s triangleCount " + triangleCounts[i]);
	    script.append("param -s frameCount " + frameCounts[i]);
	
    	    script.append("start -d");
	}
    }
    script.append("param -s compositeMode de");

    // Measure light count and triangle count
    for(local lcount=0; lcount<=6; lcount++) {
        script.append("param -s lightCount " + lcount);

	for(local i=0; i<triangleCounts.len(); i++) {
	    script.append("param -s triangleCount " + triangleCounts[i]);
	    script.append("param -s frameCount " + frameCounts[i]);
	
    	    script.append("start -d");
	}
    }
    script.append("param -s lightCount 1");

    // Measure render mode and triangle count
    foreach(renderMode in ["im", "va", "dl"]) {
	script.append("param -s renderMode " + renderMode);

	for(local i=0; i<triangleCounts.len(); i++) {
	    script.append("param -s triangleCount " + triangleCounts[i]);
	    script.append("param -s frameCount " + frameCounts[i]);
	
    	    script.append("start -d");
	}
    }
    script.append("param -s renderMode im");

    return script;
}
