
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMBuffer_impl.h
/// Contains header implementation of class Buffer

 //
 // Getters & setters
 //

inline Node *Buffer::getParent() const
{
  return mParent;
}

 /*-------------------------------------------------------------------------*/

inline const PCuint & Buffer::getLeft() const
{
  return mLeft;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setLeft(const PCuint left)
{
  Assert(!mInitialized, INVALID_OPERATION_ERROR, "Buffer::setLeft()");

  mLeft = left;
}

 /*-------------------------------------------------------------------------*/

inline const PCuint & Buffer::getTop() const
{
  return mTop;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setTop(const PCuint top)
{
  Assert(!mInitialized, INVALID_OPERATION_ERROR, "Buffer::setTop()");

  mTop = top;
}

 /*-------------------------------------------------------------------------*/

inline const PCuint & Buffer::getWidth() const
{
  return mWidth;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setWidth(const PCuint width)
{
  Assert(!mInitialized, INVALID_OPERATION_ERROR, "Buffer::setWidth()");

  mWidth = width;
}

 /*-------------------------------------------------------------------------*/

inline const PCuint & Buffer::getHeight() const
{
  return mHeight;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setHeight(const PCuint height)
{
  Assert(!mInitialized, INVALID_OPERATION_ERROR, "Buffer::setHeight()");

  mHeight = height;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Buffer::getOwnPointers() const
{
  return mOwnPointers;
}

 /*-------------------------------------------------------------------------*/

inline PCuint *Buffer::getColour()
{
  return mColour;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setColour(const PCuint * colour)
{
  Assert(mInitialized, INVALID_OPERATION_ERROR, "Buffer::setColour()");

  if(mInitialized)
	freeBuffers();
  mOwnPointers = false;
  mColour = (PCuint *) colour;
}

 /*-------------------------------------------------------------------------*/

inline void *Buffer::getDepth()
{
  return mDepth;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setDepth(const void *depth)
{
  Assert(mInitialized, INVALID_OPERATION_ERROR, "Buffer::setColour()");

  if(mInitialized)
	freeBuffers();
  mOwnPointers = false;
  mDepth = (void *) depth;
}

 /*-------------------------------------------------------------------------*/

inline const PCint & Buffer::getDepthFormat() const
{
  return mDepthFormat;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setDepthFormat(const PCint depthformat)
{
  Assert(!mInitialized, INVALID_OPERATION_ERROR, "Buffer::setDepthFormat()");

  mDepthFormat = depthformat;
}

 /*-------------------------------------------------------------------------*/

inline const PCint & Buffer::getOutputRowPixel() const
{
  return mOutputRowPixel;
}

 /*-------------------------------------------------------------------------*/

inline void Buffer::setOutputRowPixel(const PCint & outputrowpixel)
{
  Assert(mInitialized, INVALID_OPERATION_ERROR, "Buffer::setOutputRowPixel()");

  mOutputRowPixel = outputrowpixel;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Buffer::getInitialized() const
{
  return mInitialized;
}

 /*-------------------------------------------------------------------------*/
