
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMClient.h
/// Contains declaration of class Client

#ifndef __PCM_CLIENT_H__
#  define __PCM_CLIENT_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMNetwork.h"

#  include "PCMStringConverter.h"

#  include "PCMCluster.h"

#  include "PCMHostInfo.h"

#  include "PCMHost.h"

namespace ParCompMark
{

  /**
	Client functions.
   */

  class Client:public Network
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestClient;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer to this class. */
	typedef ParCompMark::Pointer < Client,
	DummyLock > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** IP address of the server host.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mServerIP;

	/** IP address of the server host.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mType;

	/** The parent network.

	 @remarks
	 This attribute references an attribute.
	*/
	Network *mParent;

	/** IP address of the server host.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mMessage;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create network class.
	 @param[in] name Name of the network.
	 */
	Client(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Client();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mServerIP. Returns value of mServerIP.

	 @return The value of mServerIP
	*/
	const std::string & getServerIP() const;

	/**
	 Setter of mServerIP. Sets value of mServerIP.

	 @param[in] serverip The value of mServerIP
	*/
	void setServerIP(const std::string & serverip);

	/**
	 Getter of mType. Returns value of mType.

	 @return The value of mType
	*/
	const std::string & getType() const;

	/**
	 Setter of mType. Sets value of mType.

	 @param[in] type The value of mType
	*/
	void setType(const std::string & type);

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Network *getParent() const;

	/**
	 Getter of mMessage. Returns value of mMessage.

	 @return The value of mMessage
	*/
	const std::string & getMessage() const;

	/**
	 Setter of mMessage. Sets value of mMessage.

	 @param[in] message The value of mMessage
	*/
	void setMessage(const std::string & message);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Open TCP/IP connection (mServerIP).
	 */
	virtual void openConnection();

	/** 
	 Close TCP/IP connection (mServerIP).
	 */
	virtual void closeConnection();

	/** 
	 Send message with TCP protocol.
	 @param[in] type Type of the message.
	 @param[in] message The message.
	 */
	virtual void sendMessage(const std::string & type, const std::string & message);

	/** 
	 Wait for TCP/IP message.
	 */
	virtual void recieveMessage();

	/** 
	 Handle TCP/IP message.
	 */
	virtual void handleMessage();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMClient_impl.h"

}

#endif
