
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMCluster.h
/// Contains declaration of class Cluster

#ifndef __PCM_CLUSTER_H__
#  define __PCM_CLUSTER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMHostInfo.h"

#  include "PCMContainer.h"

#  include "PCMPointer.h"

#  include "PCMSingleton.h"

//
// Outer includes
//

#  include <string>

namespace ParCompMark
{

  /**
	Class contain hosts. Description of the physical grid.
   */

  class Cluster:public Singleton < Cluster >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestCluster;
#  endif

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Hosts in the cluster.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < HostInfo, Mutex >::Pointer mHosts;

	/** Cluster pointer.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mClusterDescription;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Cluster();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Cluster();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mHosts. Returns value of mHosts.

	 @return The value of mHosts
	*/
	Container < HostInfo, Mutex >::Pointer & getHosts();

	/**
	 Getter of mClusterDescription. Returns value of mClusterDescription.

	 @return The value of mClusterDescription
	*/
	OutputNode::Pointer & getClusterDescription();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Creates a Cluster class instance based on xml
	 @param[in] strXml Description of system.
	 @return Returns the pointer of the Cluster class instance created based on the xml
	 */
	static Cluster *parseXML(const std::string & strXml);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Refresh output node data.
	 */
	virtual void refreshData();

	/** 
	 Serialize the cluster to Squirrel structure code snipplet.
	 @return Serialized Squirrel structure code snipplet.
	 */
	virtual std::string serialize2Squirrel();

	/** 
	 Serialize the cluster to XML. 
	 @return Serialized cluster.
	 */
	virtual std::string serialize2XML();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMCluster_impl.h"

}

#endif
