
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMConfig.h
/// Contains configuration options for ParCompMark

#ifndef __PCM_CONFIG_H__
#  define __PCM_CONFIG_H__

// GNU buildsystem configuration file
#  ifdef HAVE_CONFIG_H
#   include "config.h"
#  endif

// Particular header files for ANSI C headers
#  ifdef HAVE_MEMORY_H
#   include <memory.h>
#  endif

#  ifdef HAVE_STDINT_H
#   include <stdint.h>
#  endif

#  ifdef HAVE_STDLIB_H
#   include <stdlib.h>
#  endif

#  ifdef HAVE_STRINGS_H
#   include <strings.h>
#  endif

#  ifdef HAVE_STRING_H
#   include <string.h>
#  endif

#  ifdef HAVE_SYS_STAT_H
#   include <sys/stat.h>
#  endif

#  ifdef HAVE_SYS_TYPES_H
#   include <sys/types.h>
#  endif

#  ifdef HAVE_UNISTD_H
#   include <unistd.h>
#  endif

// C++ standard IO header
#  ifdef HAVE_IOSTREAM
#   include <iostream>
#  else
#   error Cannot find header 'iostream'
#  endif

// Standard C++ string header
#  ifdef HAVE_STRING
#   include <string>
#  else
#   error Cannot find header 'string'
#  endif

// ANSI C math header
#  ifdef HAVE_MATH_H
#   include <math.h>
#  else
#   error Cannot find header 'math.h'
#  endif

// Command line argument parser
#  ifdef HAVE_GETOPT_H

#  else
#   error Cannot find header 'getopt.h'
#  endif

// OpenGL main header
// [Optional]
#  ifdef HAVE_GL_GL_H

#  endif

// OpenGL utility header
// [Optional]
#  ifdef HAVE_GL_GLU_H

#  endif

// OpenGL utility toolkit header
// [Optional]
#  ifdef HAVE_GL_GLUT_H

#  endif

// Internet Protocol family header
#  ifdef HAVE_SYS_SOCKET_H

#  else
#   error Cannot find header 'sys/socket.h'
#  endif

// Definitions for Internet operations
#  ifdef HAVE_ARPA_INET_H

#  else
#   error Cannot find header 'arpa/inet.h'
#  endif

// Standard symbolic constants and types
#  ifdef HAVE_UNISTD_H
#   include <unistd.h>
#  else
#   error Cannot find header 'unistd.h'
#  endif

// Thread handling
#  ifdef HAVE_PTHREAD_H

#  else
#   error Cannot find header 'pthread.h'
#  endif

// Header for time types
#  ifdef HAVE_SYS_TIME_H

#  else
#   error Cannot find header 'sys/time.h'
#  endif

// Signal handling routines
#  ifdef HAVE_SIGNAL_H

#  else
#   error Cannot find header 'signal.h'
#  endif

// Squirrel header
#  ifdef HAVE_SQUIRREL_SQUIRREL_H

#  else
#   error Cannot find header 'squirrel/squirrel.h'
#  endif

// Squirrel to C++ binding header
#  ifdef HAVE_SQUIRREL_SQPLUS_H

#  else
#   error Cannot find header 'squirrel/sqplus.h'
#  endif

// GNU readline header
// [Optional]
#  ifdef HAVE_READLINE_READLINE_H

#  endif

// GNU readline (history) header
// [Optional]
#  ifdef HAVE_READLINE_HISTORY_H

#  endif

// OpenIL main header
// [Optional]
#  ifdef HAVE_IL_IL_H

#  endif

// OpenIL utility header
// [Optional]
#  ifdef HAVE_IL_ILU_H

#  endif

// OpenGL rendering support
#  if PCM_OPENGL == 1 && (0 || not defined(HAVE_GL_GL_H) || not defined(HAVE_GL_GLU_H) || not defined(HAVE_LIBGL) || not defined(HAVE_LIBGLU))
#   error Cannot enable option 'opengl' (OpenGL rendering support) since there are unincludable headers or unlinkable libraries
#  endif

// CEGUI user interface support
#  if PCM_GUI == 1 && (0 )
#   error Cannot enable option 'gui' (CEGUI user interface support) since there are unincludable headers or unlinkable libraries
#  endif

// Cg support
#  if PCM_CG == 1 && (0 || not defined(HAVE_LIBCG) || not defined(HAVE_LIBCGGL))
#   error Cannot enable option 'cg' (Cg support) since there are unincludable headers or unlinkable libraries
#  endif

#endif
