
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMContainer_impl.h
/// Contains header implementation of class Container

 //
 // Constructors & destructor
 //

template < class ElementType, class LockType > inline Container < ElementType, LockType >::Container()
  // You have to initialize the following attributes:
  // - mElements
{
  // STL auto-initialization
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > inline Container < ElementType, LockType >::~Container()
{
  // STL auto-finalization
}

 /*----------------------------------------------------------------------*/

 //
 // Methods
 //

template < class ElementType, class LockType > void Container < ElementType,
 LockType >::add(std::string name, ElementPointer element)
{
  Assert(!has(name), INVALID_NAME_ERROR, "Container::add");
  mElements[name] = element;
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > typename ElementType::Pointer Container < ElementType,
  LockType >::get(const std::string & name)
{
  Assert(has(name), INVALID_NAME_ERROR, "Container::get");
  return mElements[name];
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > bool Container < ElementType,
  LockType >::has(const std::string & name)
{
  return mElements.find(name) != mElements.end();
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > void Container < ElementType,
LockType >::remove(const std::string & name)
{
  Assert(has(name), INVALID_NAME_ERROR, "Container::remove");
  mElements.erase(name);
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > u32 Container < ElementType, LockType >::getSize()
{
  return mElements.size();
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > bool Container < ElementType, LockType >::isEmpty()
{
  return !getSize();
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > typename std::map < std::string,
  typename ElementType::Pointer >::iterator Container < ElementType, LockType >::begin()
{
  return mElements.begin();
}

 /*----------------------------------------------------------------------*/

template < class ElementType, class LockType > typename std::map < std::string,
  typename ElementType::Pointer >::iterator Container < ElementType, LockType >::end()
{
  return mElements.end();
}

 /*----------------------------------------------------------------------*/
