
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMContext.h
/// Contains declaration of class Context

#ifndef __PCM_CONTEXT_H__
#  define __PCM_CONTEXT_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <pcapi.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Class containing PC context information.
   */

  class Context
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestContext;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Context > (_T("Context")).func(&Context::setProcesses,
								_T("setProcesses")).func(&Context::setContextTypeSq,
											 _T("setContextTypeSq")).
		func(&Context::getContextTypeSq, _T("getContextTypeSq")).func(&Context::getUseGLFrameletEXT,
									  _T("getUseGLFrameletEXT")).
		func(&Context::setUseGLFrameletEXT, _T("setUseGLFrameletEXT")).func(&Context::getContextType,
										 _T("getContextType")).
		func(&Context::setContextType, _T("setContextType")).func(&Context::getFrameWidth,
									_T("getFrameWidth")).
		func(&Context::setFrameWidth, _T("setFrameWidth")).func(&Context::getFrameHeight,
									_T("getFrameHeight")).
		func(&Context::setFrameHeight, _T("setFrameHeight")).func(&Context::getColourFormat,
									_T("getColourFormat")).
		func(&Context::setColourFormat, _T("setColourFormat")).func(&Context::getDepthFormat,
									 _T("getDepthFormat")).
		func(&Context::setDepthFormat, _T("setDepthFormat")).func(&Context::getCompositeType,
									_T("getCompositeType")).
		func(&Context::setCompositeType, _T("setCompositeType")).func(&Context::getCompressionHint,
									  _T("getCompressionHint")).
		func(&Context::setCompressionHint, _T("setCompressionHint")).func(&Context::getRetainOutputCount,
										_T("getRetainOutputCount")).
		func(&Context::setRetainOutputCount,
		  _T("setRetainOutputCount")).func(&Context::getVolatileFrameletCount,
						  _T("getVolatileFrameletCount")).
		func(&Context::setVolatileFrameletCount, _T("setVolatileFrameletCount")).func(&Context::getOutputDepth,
											  _T("getOutputDepth")).
		func(&Context::setOutputDepth, _T("setOutputDepth")).func(&Context::getProcessIndex,
									_T("getProcessIndex")).
		func(&Context::setProcessIndex, _T("setProcessIndex")).func(&Context::getNetworkID,
									 _T("getNetworkID")).
		func(&Context::setNetworkID, _T("setNetworkID")).enumInt(Context::MASTER,
									 _T("MASTER")).enumInt(Context::SLAVE,
											   _T("SLAVE"));
	}

	//@}  

	//
	// Enumerations
	//

   public:

  /** The control type of PC context (local, global). */
	enum ContextType
	{

	/** Master context. */
	 MASTER = 0,

	/** Slave context. */
	 SLAVE = 1
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < Context,
	DummyLock > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** ID for sorting contexts. One process can render for several contexts, so avoid deadlock, 
				sorting contexts is useful.

	 @remarks
	 This is own attribute of this class.
	*/
	int mID;

	/** Use OpenGL framelet PC library extension.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mUseGLFrameletEXT;

	/** Type of the context.

	 @remarks
	 This is own attribute of this class.
	*/
	ContextType mContextType;

	/** The current frame id. It sets by PC.

	 @remarks
	 This is own attribute of this class.
	*/
	PCid mFrameID;

	/** Width of rendered frame by the context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mFrameWidth;

	/** Height of rendered frame by the context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mFrameHeight;

	/** Colour format of rendered frame by the context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mColourFormat;

	/** Depth format of rendered frame by the context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mDepthFormat;

	/** The pixel format. Or link between depth and colour format.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mPixelFormat;

	/** Type of composition.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mCompositeType;

	/** Compression hint.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mCompressionHint;

	/** The output of a frame is available between a pcFrameEnd and the following "mRetainOutput"th pcFrameBegin.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mRetainOutputCount;

	/** How many frames in the future the application will resuse the buffer.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mVolatileFrameletCount;

	/** Needs depth information of the composition.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mOutputDepth;

	/** Processes in the context. The format of the strings is "hostname:localid".

	 @remarks
	 This is own attribute of this class.
	*/
	PCstring *mProcesses;

	/** Number of processes (PC nomenclature nodes) in this context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mProcessCount;

	/** Position of the parent process in mProcesses array.

	 @remarks
	 This is own attribute of this class.
	*/
	int mProcessIndex;

	/** The own host index.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mHostIndex;

	/** The network ID.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mNetworkID;

	/** The PC context.

	 @remarks
	 This is own attribute of this class.
	*/
	PCcontext mContext;

	/** Parent Process of this Context.

	 @remarks
	 This attribute references an attribute.
	*/
	Process *mParent;

	/** The context is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	Context();

	/** 
	 Create Context. Normally Process calls this constructor.
	 @param[in] parent Parent host
	 */
	Context(Process * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Context();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mID. Returns value of mID.

	 @return The value of mID
	*/
	const int &getID() const;

	/**
	 Setter of mID. Sets value of mID.

	 @param[in] id The value of mID
	*/
	void setID(const int &id);

	/**
	 Getter of mUseGLFrameletEXT. Returns value of mUseGLFrameletEXT.

	 @return The value of mUseGLFrameletEXT
	*/
	bool & getUseGLFrameletEXT();

	/**
	 Setter of mUseGLFrameletEXT. Sets value of mUseGLFrameletEXT.

	 @param[in] useglframeletext The value of mUseGLFrameletEXT
	*/
	void setUseGLFrameletEXT(const bool useglframeletext);

	/**
	 Getter of mContextType. Returns value of mContextType.

	 @return The value of mContextType
	*/
	Context::ContextType & getContextType();

	/**
	 Setter of mContextType. Sets value of mContextType.

	 @param[in] contexttype The value of mContextType
	*/
	void setContextType(const Context::ContextType & contexttype);

	/**
	 Getter of mFrameID. Returns value of mFrameID.

	 @return The value of mFrameID
	*/
	PCid & getFrameID();

	/**
	 Getter of mFrameWidth. Returns value of mFrameWidth.

	 @return The value of mFrameWidth
	*/
	PCint & getFrameWidth();

	/**
	 Setter of mFrameWidth. Sets value of mFrameWidth.

	 @param[in] framewidth The value of mFrameWidth
	*/
	void setFrameWidth(const PCint framewidth);

	/**
	 Getter of mFrameHeight. Returns value of mFrameHeight.

	 @return The value of mFrameHeight
	*/
	PCint & getFrameHeight();

	/**
	 Setter of mFrameHeight. Sets value of mFrameHeight.

	 @param[in] frameheight The value of mFrameHeight
	*/
	void setFrameHeight(const PCint frameheight);

	/**
	 Getter of mColourFormat. Returns value of mColourFormat.

	 @return The value of mColourFormat
	*/
	PCint & getColourFormat();

	/**
	 Setter of mColourFormat. Sets value of mColourFormat.

	 @param[in] colourformat The value of mColourFormat
	*/
	void setColourFormat(const PCint colourformat);

	/**
	 Getter of mDepthFormat. Returns value of mDepthFormat.

	 @return The value of mDepthFormat
	*/
	PCint & getDepthFormat();

	/**
	 Setter of mDepthFormat. Sets value of mDepthFormat.

	 @param[in] depthformat The value of mDepthFormat
	*/
	void setDepthFormat(const PCint depthformat);

	/**
	 Getter of mPixelFormat. Returns value of mPixelFormat.

	 @return The value of mPixelFormat
	*/
	const PCint & getPixelFormat() const;

	/**
	 Getter of mCompositeType. Returns value of mCompositeType.

	 @return The value of mCompositeType
	*/
	PCint & getCompositeType();

	/**
	 Setter of mCompositeType. Sets value of mCompositeType.

	 @param[in] compositetype The value of mCompositeType
	*/
	void setCompositeType(const PCint compositetype);

	/**
	 Getter of mCompressionHint. Returns value of mCompressionHint.

	 @return The value of mCompressionHint
	*/
	PCint & getCompressionHint();

	/**
	 Setter of mCompressionHint. Sets value of mCompressionHint.

	 @param[in] compressionhint The value of mCompressionHint
	*/
	void setCompressionHint(const PCint compressionhint);

	/**
	 Getter of mRetainOutputCount. Returns value of mRetainOutputCount.

	 @return The value of mRetainOutputCount
	*/
	PCint & getRetainOutputCount();

	/**
	 Setter of mRetainOutputCount. Sets value of mRetainOutputCount.

	 @param[in] retainoutputcount The value of mRetainOutputCount
	*/
	void setRetainOutputCount(const PCint retainoutputcount);

	/**
	 Getter of mVolatileFrameletCount. Returns value of mVolatileFrameletCount.

	 @return The value of mVolatileFrameletCount
	*/
	PCint & getVolatileFrameletCount();

	/**
	 Setter of mVolatileFrameletCount. Sets value of mVolatileFrameletCount.

	 @param[in] volatileframeletcount The value of mVolatileFrameletCount
	*/
	void setVolatileFrameletCount(const PCint volatileframeletcount);

	/**
	 Getter of mOutputDepth. Returns value of mOutputDepth.

	 @return The value of mOutputDepth
	*/
	bool & getOutputDepth();

	/**
	 Setter of mOutputDepth. Sets value of mOutputDepth.

	 @param[in] outputdepth The value of mOutputDepth
	*/
	void setOutputDepth(const bool outputdepth);

	/**
	 Getter of mProcesses. Returns value of mProcesses.

	 @return The value of mProcesses
	*/
	PCstring *getProcesses() const;

	/**
	 Getter of mProcessCount. Returns value of mProcessCount.

	 @return The value of mProcessCount
	*/
	const PCint & getProcessCount() const;

	/**
	 Getter of mProcessIndex. Returns value of mProcessIndex.

	 @return The value of mProcessIndex
	*/
	int &getProcessIndex();

	/**
	 Setter of mProcessIndex. Sets value of mProcessIndex.

	 @param[in] processindex The value of mProcessIndex
	*/
	void setProcessIndex(const int processindex);

	/**
	 Getter of mHostIndex. Returns value of mHostIndex.

	 @return The value of mHostIndex
	*/
	const PCint & getHostIndex() const;

	/**
	 Getter of mNetworkID. Returns value of mNetworkID.

	 @return The value of mNetworkID
	*/
	PCint & getNetworkID();

	/**
	 Setter of mNetworkID. Sets value of mNetworkID.

	 @param[in] networkid The value of mNetworkID
	*/
	void setNetworkID(const PCint networkid);

	/**
	 Getter of mContext. Returns value of mContext.

	 @return The value of mContext
	*/
	const PCcontext & getContext() const;

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Process *getParent();

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Create process array from string separated by semicolons.
	 @param[in] processList Process list separated by semicolons (C string for squirrel compatibility).
	 */
	virtual void setProcesses(const char *processList);

	/** 
	 Init the PC context.
	 */
	virtual void initialize();

	/** 
	 Finalize the PC context.
	 */
	virtual void finalize();

	/** 
	 Alternate squirrel setter of mContextType.
	 @param[in] contextType Context type int value (0 means MASTER, 1 means SLAVE).
	 */
	virtual void setContextTypeSq(const int contextType);

	/** 
	 Alternate squirrel setter of mContextType.
	 @return Context type int value (0 means MASTER, 1 means SLAVE).
	 */
	virtual int getContextTypeSq();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMContext_impl.h"

}

// Declare Context class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Context, Context);

DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Context::ContextType, Context::ContextType);

#endif
