
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMDummyLock.h
/// Contains declaration of class DummyLock

#ifndef __PCM_DUMMYLOCK_H__
#  define __PCM_DUMMYLOCK_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLock.h"

namespace ParCompMark
{

  /**
	Dummy lock implementing the Lock interface. Does not do anything.
		The DummyLock can always be locked.

   */

  class DummyLock:public Lock
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestDummyLock;
#  endif

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ DummyLock();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Lock the lock. Does not do anything.
	 */
	virtual void lock();

	/** 
	 Try locking the lock. Does not do anything. Always return true.
	 @return True if the locking was successful. Always true.
	 */
	virtual bool trylock();

	/** 
	 Unlock the lock. Does not do anything.
	 */
	virtual void unlock();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMDummyLock_impl.h"

}

#endif
