
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMDynLoad.h
/// Contains declaration of class DynLoad

#ifndef __PCM_DYNLOAD_H__
#  define __PCM_DYNLOAD_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

namespace ParCompMark
{

  /**
	Dynamic load library.
   */

  class DynLoad
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestDynLoad;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < DynLoad,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** The dynamic library handler.

	 @remarks
	 This is own attribute of this class.
	*/
	void *mHandle;

	/** The dynamic library name.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mLibraryName;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create a Dynamic load class, and load the libName named library.
	 @param[in] libName Name of the loaded library
	 */
	DynLoad(const std::string & libName);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ DynLoad();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mHandle. Returns value of mHandle.

	 @return The value of mHandle
	*/
	void *getHandle() const;

	/**
	 Getter of mLibraryName. Returns value of mLibraryName.

	 @return The value of mLibraryName
	*/
	const std::string & getLibraryName() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Return true when the dynamic library has a callable function with the specified name.
	 @param[in] funcName Name of function
	 @return True when the dynamic library has a callable function with the specified name
	 */
	virtual bool hasFunction(const std::string & funcName) const;

	/** 
	 Get a function pointer from dynamic library by name (symbol).
	 @param[in] funcName Name of function
	 @return Pointer to the function
	 */
	virtual void *getFunction(const std::string & funcName) const;

   protected:

	/** 
	 Load a library. Called by constructor.
	 */
	virtual void load();

	/** 
	 Load a library. Called by constructor.
	 */
	virtual void unload();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMDynLoad_impl.h"

}

#endif
