
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMGPU.h
/// Contains declaration of class GPU

#ifndef __PCM_GPU_H__
#  define __PCM_GPU_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMOutputNode.h"

#  include "PCMPointer.h"

#  include "PCMOutputNode.h"

#  include "PCMFileSystemManager.h"

#  include "PCMXDisplay.h"

//
// Outer includes
//

#  include <string>

#  include <tinyxml/tinyxml.h>

namespace ParCompMark
{

  /**
	Class contain GPU information.
   */

  class GPU:public OutputNode
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestGPU;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < GPU,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Name of the vendor of the GPU.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mVendor;

	/** GPU model name.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mModel;

	/** GPU memory size.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mMemorySize;

	/** Triangles/second or texels/second. GPU load=rendering 
					 capacity can be only get per second.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mLoadPerSecond;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates GPU node.
	 @param[in] name Name of the node.
	 @param[in] type Type of the node.
	 */
	GPU(const std::string & name, const OutputNode::NodeType & type);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ GPU();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mVendor. Returns value of mVendor.

	 @return The value of mVendor
	*/
	const std::string & getVendor() const;

	/**
	 Getter of mModel. Returns value of mModel.

	 @return The value of mModel
	*/
	const std::string & getModel() const;

	/**
	 Getter of mMemorySize. Returns value of mMemorySize.

	 @return The value of mMemorySize
	*/
	const u32 & getMemorySize() const;

	/**
	 Getter of mLoadPerSecond. Returns value of mLoadPerSecond.

	 @return The value of mLoadPerSecond
	*/
	const Real & getLoadPerSecond() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Creates a GPU class instance based on xml
	 @param[out] tiGPU Description of GPU.
	 @return Returns the pointer of the GPU class instance created based on the xml
	 */
	static GPU::Pointer parseXML(TiXmlElement * &tiGPU);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Refresh GPU datas.
	 */
	virtual void refreshData();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMGPU_impl.h"

}

#endif
