
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMHandleClient.h
/// Contains declaration of class HandleClient

#ifndef __PCM_HANDLECLIENT_H__
#  define __PCM_HANDLECLIENT_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMClient.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMCluster.h"

#  include "PCMPointer.h"

#  include "PCMApplication.h"

#  include "PCMHostInfo.h"

namespace ParCompMark
{

  /**
	Class for handle client messages.
   */

  class HandleClient:public Client
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestHandleClient;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer or this class. */
	typedef ParCompMark::Pointer < HandleClient,
	DummyLock > Pointer;

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create HandleClient class.
	 @param[in] net Parent network.
	 */
	HandleClient(const NetServer * net);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ HandleClient();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the class (broadcast recieve thread).
	 */
	virtual void initialize();

	/** 
	 Initialize the class (broadcast recieve thread).
	 */
	virtual void finalize();

	/** 
	 Open TCP/IP connection (do nothing).
	 */
	virtual void openConnection();

   protected:

	/** 
	 Handle a client.
	 */
	virtual void task();

	//@}  

  };

}

#endif
