
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMHost.h
/// Contains declaration of class Host

#ifndef __PCM_HOST_H__
#  define __PCM_HOST_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMContainer.h"

#  include "PCMXDisplay.h"

#  include "PCMNode.h"

#  include "PCMTimer.h"

#  include "PCMSqVM.h"

//
// Outer includes
//

#  include <string>

#  include <pcapi.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Class for describe a hosts.
   */

  class Host:public Singleton < Host >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestHost;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Host > (_T("Host")).staticFunc(&Host::getInstance,
								_T("getInstance")).func(&Host::createNode,
											_T("createNode"));
	}

	//@}  

	//
	// Typedefs
	//

   public:

  /** Type for pointer to an int. */
	typedef ParCompMark::Pointer < int,
	Mutex > IntPointer;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Host initializer Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string HOSTINITNUT;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Node wait?.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mWait;

	/** Name of the host (IP address).

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mName;

	/** Squirrel virtual machine.

	 @remarks
	 This is own attribute of this class.
	*/
	SqVM::Pointer mSqVM;

	/** Squirrel low level script for initialization.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mLowLevelScript;

	/** X Displays on this host.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < XDisplay, Mutex >::Pointer mXDisplays;

	/** Nodes on this host.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < Node, Mutex >::Pointer mNodes;

	/** The host is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Root of the output document on this host.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mOutputDocument;

	/** PC session ID.

	 @remarks
	 This is own attribute of this class.
	*/
	PCid mSessionID;

	/** Pointer to the netclietn class.

	 @remarks
	 This attribute references an attribute.
	*/
	NetClient *mNetClient;

	/** Count how many process end the rendering/compositing.

	 @remarks
	 This is own attribute of this class.
	*/
	IntPointer mEndProcessCount;

	/** Count how many processes are on the host.

	 @remarks
	 This is own attribute of this class.
	*/
	int mProcessCount;

	/** ID for the host.

	 @remarks
	 This is own attribute of this class.
	*/
	int mID;

	/** Network message sendign time.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mMessageSendingTime;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates a host with a specified name.
	 @param[in] name Name of the host.
	 @param[in] netClient Pointer to the network handler class.
	 */
	Host(const std::string & name = "(unknown host)", NetClient * netClient = NULL);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Host();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mWait. Returns value of mWait.

	 @return The value of mWait
	*/
	const bool & getWait() const;

	/**
	 Getter of mName. Returns value of mName.

	 @return The value of mName
	*/
	const std::string & getName() const;

	/**
	 Setter of mName. Sets value of mName.

	 @param[in] name The value of mName
	*/
	void setName(const std::string & name);

	/**
	 Getter of mLowLevelScript. Returns value of mLowLevelScript.

	 @return The value of mLowLevelScript
	*/
	const std::string & getLowLevelScript() const;

	/**
	 Setter of mLowLevelScript. Sets value of mLowLevelScript.

	 @param[in] lowlevelscript The value of mLowLevelScript
	*/
	void setLowLevelScript(const std::string & lowlevelscript);

	/**
	 Getter of mNodes. Returns value of mNodes.

	 @return The value of mNodes
	*/
	Container < Node, Mutex >::Pointer & getNodes();

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mOutputDocument. Returns value of mOutputDocument.

	 @return The value of mOutputDocument
	*/
	OutputNode::Pointer & getOutputDocument();

	/**
	 Getter of mSessionID. Returns value of mSessionID.

	 @return The value of mSessionID
	*/
	const PCid & getSessionID() const;

	/**
	 Getter of mNetClient. Returns value of mNetClient.

	 @return The value of mNetClient
	*/
	NetClient *getNetClient() const;

	/**
	 Setter of mNetClient. Sets value of mNetClient.

	 @param[in] netclient The value of mNetClient
	*/
	void setNetClient(const NetClient * netclient);

	/**
	 Getter of mEndProcessCount. Returns value of mEndProcessCount.

	 @return The value of mEndProcessCount
	*/
	const Host::IntPointer & getEndProcessCount() const;

	/**
	 Setter of mEndProcessCount. Sets value of mEndProcessCount.

	 @param[in] endprocesscount The value of mEndProcessCount
	*/
	void setEndProcessCount(const Host::IntPointer & endprocesscount);

	/**
	 Getter of mProcessCount. Returns value of mProcessCount.

	 @return The value of mProcessCount
	*/
	const int &getProcessCount() const;

	/**
	 Getter of mID. Returns value of mID.

	 @return The value of mID
	*/
	const int &getID() const;

	/**
	 Setter of mID. Sets value of mID.

	 @param[in] id The value of mID
	*/
	void setID(const int &id);

	/**
	 Getter of mMessageSendingTime. Returns value of mMessageSendingTime.

	 @return The value of mMessageSendingTime
	*/
	const Real & getMessageSendingTime() const;

	/**
	 Setter of mMessageSendingTime. Sets value of mMessageSendingTime.

	 @param[in] messagesendingtime The value of mMessageSendingTime
	*/
	void setMessageSendingTime(const Real & messagesendingtime);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Gets the instance of the singleton class. Reimplemented for squirrel usage.
	 @return Instance of the class
	 */
	static Host *getInstance();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Open an X display. An X display can be opened 
			several times, and does not have to be closed. The host
			 will close it when it is destructed. When no parameter is set,
			the default display will be opened.

	 @param[in] displayName X display name
	 @return Opened X Display
	 */
	virtual XDisplay::Pointer openXDisplay(const std::string & displayName = "");

	/** 
	 Return the first X Display on this host.
	 @return The first display on this host, on null if there are no opened displays.
	 */
	virtual XDisplay::Pointer getFirstXDisplay();

	/** 
	 Automatically scan available X displays on this host and open them.
	 */
	virtual void openXDisplays();

	/** 
	 Close all opened X displays.
	 */
	virtual void closeXDisplays();

	/** 
	 Initialize host from low level script.
	 */
	virtual void initialize();

	/** 
	 Finalize the host.
	 */
	virtual void finalize();

	/** 
	 Create node on this host.
	 @param[in] nodeName Name of the node (C string for squirrel compatibility).
	 @return Pointer to the created node.
	 */
	virtual Node *createNode(const char *nodeName);

	/** 
	 Start the nodes.
	 */
	virtual void start();

	/** 
	 Stop the nodes.
	 @return frameID
	 */
	virtual u32 stop();

	/** 
	 Collect data from nodes.
	 */
	virtual void collectData();

	/** 
	 Set the ending frameID of processes.
	 @param[in] frameID StopID.
	 */
	virtual void setFrameID(const u32 & frameID);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMHost_impl.h"

}

// Declare Host class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Host, Host);

#endif
