
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMLock.h
/// Contains declaration of class Lock

#ifndef __PCM_LOCK_H__
#  define __PCM_LOCK_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

namespace ParCompMark
{

  /**
	Lock interface. Provides lock, trylock and unlock methods.
   */

  class Lock
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestLock;
#  endif

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Indicates that the lock is locked.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mLocked;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Lock();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Lock();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mLocked. Returns value of mLocked.

	 @return The value of mLocked
	*/
	const bool & getLocked() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Lock the lock.
	 */
	virtual void lock() = 0;

	/** 
	 Try locking the lock.
	 @return True if the locking was successful.
	 */
	virtual bool trylock() = 0;

	/** 
	 Unlock the lock.
	 */
	virtual void unlock() = 0;

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMLock_impl.h"

}

#endif
