
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMLogger_impl.h
/// Contains header implementation of class Logger

 //
 // Getters & setters
 //

inline const u8 & Logger::getLogMode() const
{
  return mLogMode;
}

 /*-------------------------------------------------------------------------*/

inline const Logger::LogLevel & Logger::getConsoleLogLevel() const
{
  return mConsoleLogLevel;
}

 /*-------------------------------------------------------------------------*/

inline void Logger::setConsoleLogLevel(const Logger::LogLevel & consoleloglevel)
{
  mConsoleLogLevel = consoleloglevel;
}

 /*-------------------------------------------------------------------------*/

inline const Logger::LogLevel & Logger::getFileLogLevel() const
{
  return mFileLogLevel;
}

 /*-------------------------------------------------------------------------*/

inline void Logger::setFileLogLevel(const Logger::LogLevel & fileloglevel)
{
  mFileLogLevel = fileloglevel;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Logger::getInitialized() const
{
  return mInitialized;
}

 /*-------------------------------------------------------------------------*/

inline const std::string & Logger::getLogFileName() const
{
  return mLogFileName;
}

 /*-------------------------------------------------------------------------*/

inline void Logger::setLogFileName(const std::string & logfilename)
{
  mLogFileName = logfilename;
}

 /*-------------------------------------------------------------------------*/

 //
 // Class methods
 //

inline std::string Logger::translateLogLevel(const LogLevel & loglevel)
{

  switch (loglevel)
  {
	case Logger::FATAL:
	 return "[FATAL]----:";
	 break;
	case Logger::ERROR:
	 return "[ERROR]----:";
	 break;
	case Logger::WARNING:
	 return "[WARNING]--:";
	 break;
	case Logger::NOTICE:
	 return "[NOTICE]---:";
	 break;
	case Logger::DEBUG:
	 return "[DEBUG]----:";
	 break;
  }

  return "[unknown]--:";

}

 /*----------------------------------------------------------------------*/

 //
 // Methods
 //

inline void Logger::log(const LogLevel & loglevel, const std::string & message)
{
  Assert(mInitialized, NULL_POINTER_ERROR, "Logger::log()");
  // File pointer locks the console too

  mFp.lock();

  // Log to console
  if(mLogMode & LOGTOCONSOLE)
  {
	if(loglevel <= mConsoleLogLevel)
	{

	 if(loglevel > ERROR)
		std::cout << "PCM: " << translateLogLevel(loglevel) << " " << message << std::endl;
	 else
		std::cerr << "PCM: " << translateLogLevel(loglevel) << " " << message << std::endl;
	}
  }
  // Log to file
  if(mLogMode & LOGTOFILE)
  {
	if(loglevel <= mFileLogLevel)
	{
	 fprintf(mFp.getPtr(), "[%s] %s %s\n", Timer::getTimeString().c_str(), translateLogLevel(loglevel).c_str(),
		 message.c_str());

	 // Flush the buffer
	 fflush(mFp.getPtr());
	}
  }

  mFp.unlock();

}

 /*----------------------------------------------------------------------*/

inline void Logger::log(const Exception & exception)
{
  Assert(mInitialized, NULL_POINTER_ERROR, "Logger::log()");

  log(FATAL, translateException(exception));
}

 /*----------------------------------------------------------------------*/
