
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMMutex.h
/// Contains declaration of class Mutex

#ifndef __PCM_MUTEX_H__
#  define __PCM_MUTEX_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLock.h"

#  include "PCMException.h"

//
// Outer includes
//

#  include <pthread.h>

namespace ParCompMark
{

  /**
	Mutual exception lock.
   */

  class Mutex:public Lock
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestMutex;
#  endif

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Guard mutex.

	 @remarks
	 This is own attribute of this class.
	*/
	pthread_mutex_t mMutex;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Mutex();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Mutex();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Lock the mutex.
	 */
	virtual void lock();

	/** 
	 Try locking the mutex.
	 @return True if the locking was successful.
	 */
	virtual bool trylock();

	/** 
	 Unlock the mutex.
	 */
	virtual void unlock();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMMutex_impl.h"

}

#endif
