
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMName.h
/// Contains declaration of class Name

#ifndef __PCM_NAME_H__
#  define __PCM_NAME_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

//
// Outer includes
//

#  include <string>

namespace ParCompMark
{

  /**
	This is a dummy class for inheritance. Contain grid data.
   */

  class Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestName;
#  endif

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Name.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mName;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Name();

	/** 
	 Default constructor. 
	 @param[in] name Name of Name:)
	 */
	Name(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Name();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mName. Returns value of mName.

	 @return The value of mName
	*/
	const std::string & getName() const;

	/**
	 Setter of mName. Sets value of mName.

	 @param[in] name The value of mName
	*/
	void setName(const std::string & name);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMName_impl.h"

}

#endif
