
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMNetServer.h
/// Contains declaration of class NetServer

#ifndef __PCM_NETSERVER_H__
#  define __PCM_NETSERVER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMNetwork.h"

#  include "PCMHandleClient.h"

namespace ParCompMark
{

  /**
	Network server class.
   */

  class NetServer:public Network
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestNetServer;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer to this class. */
	typedef ParCompMark::Pointer < NetServer,
	DummyLock > Pointer;

  /** Type for pointer to an int. */
	typedef ParCompMark::Pointer < int,
	Mutex > IntPointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** The broadcast address structure for send later time.

	 @remarks
	 This is own attribute of this class.
	*/
	struct sockaddr_in mBroadcastAddress;

	/** .

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mMaxConnection;

	/** Number if the host in the cluster.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mHostNumber;

	/** Number of the recieved message for a broadcast.

	 @remarks
	 This is own attribute of this class.
	*/
	IntPointer mRecievedNumber;

	/** Frame id for stop PC.

	 @remarks
	 This is own attribute of this class.
	*/
	IntPointer mStopFrameID;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create network class.
	 @param[in] name Name of the network.
	 */
	NetServer(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ NetServer();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mBroadcastAddress. Returns value of mBroadcastAddress.

	 @return The value of mBroadcastAddress
	*/
	const struct sockaddr_in &getBroadcastAddress() const;

	/**
	 Getter of mMaxConnection. Returns value of mMaxConnection.

	 @return The value of mMaxConnection
	*/
	const u32 & getMaxConnection() const;

	/**
	 Getter of mHostNumber. Returns value of mHostNumber.

	 @return The value of mHostNumber
	*/
	const u32 & getHostNumber() const;

	/**
	 Setter of mHostNumber. Sets value of mHostNumber.

	 @param[in] hostnumber The value of mHostNumber
	*/
	void setHostNumber(const u32 & hostnumber);

	/**
	 Getter of mRecievedNumber. Returns value of mRecievedNumber.

	 @return The value of mRecievedNumber
	*/
	const NetServer::IntPointer & getRecievedNumber() const;

	/**
	 Getter of mStopFrameID. Returns value of mStopFrameID.

	 @return The value of mStopFrameID
	*/
	const NetServer::IntPointer & getStopFrameID() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the class (server thread, broadcast send).
	 */
	virtual void initialize();

	/** 
	 Initialize the class (broadcast recieve thread).
	 */
	virtual void finalize();

	/** 
	 Send a broadcast message.
	 @param[in] type Type of the message.
	 @param[in] message The message.
	 */
	virtual void sendBroadcastMessage(const std::string & type, const std::string & message);

	/** 
	 Build cluster information.
	 */
	virtual void buildCluster();

   protected:

	/** 
	 Thread for accept TCP/IP connection.
	 */
	virtual void task();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMNetServer_impl.h"

}

#endif
