
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMOpenGLRenderingEngine.h
/// Contains declaration of class OpenGLRenderingEngine

#ifndef __PCM_OPENGLRENDERINGENGINE_H__
#  define __PCM_OPENGLRENDERINGENGINE_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <list>

#  include <set>

#  include <vector>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Collection of rendering methods implemented using OpenGL API.
   */

  class OpenGLRenderingEngine
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestOpenGLRenderingEngine;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < OpenGLRenderingEngine >
		(_T("OpenGLRenderingEngine")).staticFunc(&OpenGLRenderingEngine::create,
							 _T("create")).
		func(&OpenGLRenderingEngine::createCustomRenderer,
		  _T("createCustomRenderer")).func(&OpenGLRenderingEngine::perspective,
						  _T("perspective")).func(&OpenGLRenderingEngine::ortho2D,
									  _T("ortho2D")).
		func(&OpenGLRenderingEngine::viewport, _T("viewport")).func(&OpenGLRenderingEngine::pushModelView,
									 _T("pushModelView")).
		func(&OpenGLRenderingEngine::popModelView, _T("popModelView")).func(&OpenGLRenderingEngine::translate,
										 _T("translate")).
		func(&OpenGLRenderingEngine::rotate, _T("rotate")).func(&OpenGLRenderingEngine::scale,
									_T("scale")).
		func(&OpenGLRenderingEngine::setCameraPosition,
		  _T("setCameraPosition")).func(&OpenGLRenderingEngine::setCameraTarget,
						 _T("setCameraTarget")).
		func(&OpenGLRenderingEngine::setCameraUpVector,
		  _T("setCameraUpVector")).func(&OpenGLRenderingEngine::setColor,
						 _T("setColor")).func(&OpenGLRenderingEngine::setAmbientMaterial,
									_T("setAmbientMaterial")).
		func(&OpenGLRenderingEngine::setDiffuseMaterial,
		  _T("setDiffuseMaterial")).func(&OpenGLRenderingEngine::setSpecularMaterial,
						 _T("setSpecularMaterial")).
		func(&OpenGLRenderingEngine::setDrawStyle,
		  _T("setDrawStyle")).func(&OpenGLRenderingEngine::setBackCulling,
					  _T("setBackCulling")).func(&OpenGLRenderingEngine::setBlending,
									 _T("setBlending")).
		func(&OpenGLRenderingEngine::drawTriangle,
		  _T("drawTriangle")).func(&OpenGLRenderingEngine::generateRandomTriangles,
					  _T("generateRandomTriangles")).func(&OpenGLRenderingEngine::renderObject,
										_T("renderObject")).
		func(&OpenGLRenderingEngine::renderObjectTriangles,
		  _T("renderObjectTriangles")).func(&OpenGLRenderingEngine::setAmbientLight,
						   _T("setAmbientLight")).
		func(&OpenGLRenderingEngine::removeLightSources,
		  _T("removeLightSources")).func(&OpenGLRenderingEngine::addLightSource,
						 _T("addLightSource")).
		func(&OpenGLRenderingEngine::setLightSourcePosition,
		  _T("setLightSourcePosition")).func(&OpenGLRenderingEngine::setLightSourceDiffuse,
							_T("setLightSourceDiffuse")).
		func(&OpenGLRenderingEngine::setLightSourceSpecular,
		  _T("setLightSourceSpecular")).func(&OpenGLRenderingEngine::createDisplayList,
							_T("createDisplayList")).
		func(&OpenGLRenderingEngine::finishDisplayList,
		  _T("finishDisplayList")).func(&OpenGLRenderingEngine::executeDisplayList,
						 _T("executeDisplayList")).func(&OpenGLRenderingEngine::drawSphere,
										_T("drawSphere")).
		func(&OpenGLRenderingEngine::drawCylinder, _T("drawCylinder")).func(&OpenGLRenderingEngine::drawDisk,
										 _T("drawDisk")).
		func(&OpenGLRenderingEngine::drawTeapot, _T("drawTeapot")).func(&OpenGLRenderingEngine::drawCube,
										_T("drawCube")).
		func(&OpenGLRenderingEngine::drawTorus, _T("drawTorus")).func(&OpenGLRenderingEngine::drawDodecahedron,
									  _T("drawDodecahedron")).
		func(&OpenGLRenderingEngine::drawOctahedron,
		  _T("drawOctahedron")).func(&OpenGLRenderingEngine::drawTetrahedron,
						_T("drawTetrahedron")).func(&OpenGLRenderingEngine::drawIcosahedron,
									 _T("drawIcosahedron"));
	}

	//@}  

	//
	// Enumerations
	//

   public:

  /** Drawing styles. */
	enum DrawStyle
	{

	/** The primitive will not be drawn */
	 NONE = 0,

	/** Only the vertices will be drawn */
	 POINT = 1,

	/** Only the edges will be drawn */
	 WIRE = 2,

	/** Filled polygon primitives will be drawn */
	 FILL = 3
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < OpenGLRenderingEngine,
	Mutex > Pointer;

   protected:

  /** Camera type. */
	typedef struct Camera
	{
	 Real position[3];
	 Real target[3];

	 Real upVector[3];								 /** Set default values */
	  Camera()
	 {
		position[0] = position[1] = position[2] = 1.0;
		target[0] = target[1] = target[2] = 0.0;
		upVector[0] = 0.0;
		upVector[1] = 1.0;
		upVector[2] = 0.0;
	 };
	} Camera;

  /** Camera type. */
	typedef struct ObjectData

	{			/** Number of vertices */

	 int vertexCount;							/** Specifies the number of coordinates per vertex; must be 2 or 3. */

	 int vertexSize;																		  /** Specifies the number of texture coordinates per vertex; must be 0, 2, or 3. */

	 int texSize;																															 /** Array of vertex coordinates */

	 float *vertexCoordinates;																																						  /** Array of vertex texture coordinates */

	 float *texCoordinates;																																															 /** Array of normals */

	 float *normals;																																																						/** Set default values */
	  ObjectData()
	 {
		vertexSize = 3;
		texSize = 0;
		vertexCoordinates = texCoordinates = 0;
	 };
	} ObjectData;

  /** OpenGL display list type. */
	typedef struct DisplayList

	{			 /** Display list handle */

	 u32 handle;							   /** Triangle count for this display list */

	 u32 triangleCount;															   /** Set default values */
	  DisplayList()
	 {
		handle = 0;
		triangleCount = 0;
	 };
	} DisplayList;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Parent process of the rendering engine.

	 @remarks
	 This attribute references an attribute.
	*/
	Process * mParent;

	/** Camera of this rendering engine.

	 @remarks
	 This is own attribute of this class.
	*/
	Camera mCamera;

	/** Drawing style for primitives.

	 @remarks
	 This is own attribute of this class.
	*/
	DrawStyle mDrawStyle;

	/** Custom renderers used with this rendering engine.

	 @remarks
	 This is own attribute of this class.
	*/
	std::set < Renderer * >mRenderers;

	/** Automatic rendering queue for custom renderers.

	 @remarks
	 This is own attribute of this class.
	*/
	std::vector < Renderer * >mAutoRenderingQueue;

	/** Data of objects can be rendered on this rendering engine.

	 @remarks
	 This is own attribute of this class.
	*/
	std::vector < ObjectData > mObjectData;

	/** Display lists on this rendering engine.

	 @remarks
	 This is own attribute of this class.
	*/
	std::vector < DisplayList * >mDisplayLists;

	/** Pointer to the current display list.

	 @remarks
	 This attribute references an attribute.
	*/
	DisplayList *mCurrentDisplayList;

	/** Number of lights.

	 @remarks
	 This attribute references an attribute.
	*/
	u32 mLightCount;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	OpenGLRenderingEngine();

	/** 
	 Create rendering engine for the specified process.
	 @param[in] parent Parent process
	 */
	OpenGLRenderingEngine(Process * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ OpenGLRenderingEngine();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Process * getParent()const;

	/**
	 Getter of mCamera. Returns value of mCamera.

	 @return The value of mCamera
	*/
	const OpenGLRenderingEngine::Camera & getCamera() const;

	/**
	 Getter of mDrawStyle. Returns value of mDrawStyle.

	 @return The value of mDrawStyle
	*/
	const OpenGLRenderingEngine::DrawStyle & getDrawStyle() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Create (cast) rendering engine from a pointer passed in a string in '0x000000' format.
	 @param[in] stringPtr A pointer passed in a string in '0x000000' format.
	 @return Cast rendering engine object.
	 */
	static OpenGLRenderingEngine *create(const char *stringPtr);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Create custom renderer from a renderer plugin.
	 @param[in] rendererName Name of the renderer plugin.
	 @return The created renderer.
	 */
	virtual Renderer * createCustomRenderer(const char *rendererName);

	/** 
	 Resize the canvas of all renderers belongs to this rendering engine. 
			The <tt>pcmOnResize</tt> event handler method of the renderers will be called.

	 @param[in] width New canvas width.
	 @param[in] height New canvas height.
	 */
	virtual void resizeRenderers(const u32 width, const u32 height);

	/** 
	 Get alpha sorting order from renderers on this rendering engine. If the corresponding renderer plugin
			has <tt>pcmOnResize</tt> function, it will be called otherwise -1 is returned. Note: -1 is also returned
			when the sorting order can not be specified by the plugin.

	 @return Sort order or -1 if it can not be calculated.
	 */
	virtual s32 getSortOrderFromRenderers();

	/** 

			Set automatic rendering order for the specified renderer. If Renderer::NOAUTORENDERING is specified as order, the
			renderer will be removed from the queue.

	 @param[in] renderer The renderer to place into the queue (or remove).
	 @param[in] order Order in the rendering queue.
	 */
	virtual void setAutoRenderOrder(const Renderer * renderer, const s32 & order);

	/** 

			Automatically call render() methods of the custom renderers placed in the queue.

	 */
	virtual void doAutoRendering();

	/** 
	 Set perspective projection.
	 @param[in] fovy Specifies the field of view angle, in degrees, in the y direction.
	 @param[in] zNear Specifies the distance from the viewer to the near clipping plane (always positive).
	 @param[in] zFar Specifies the distance from the viewer to the far clipping plane (always positive).
	 */
	virtual void perspective(const double fovy, const double zNear, const double zFar);

	/** 
	 Define a 2D orthographic projection.
	 @param[in] left Specifies the coordinate for the left vertical clipping plane.
	 @param[in] right Specifies the coordinate for the right vertical clipping plane.
	 @param[in] bottom Specifies the coordinate for the bottom horizontal clipping plane.
	 @param[in] top Specifies the coordinate for the top horizontal clipping plane.
	 */
	virtual void ortho2D(const double left, const double right, const double bottom,
			  const double top);

	/** 
	 Set viewport with window relative coordinates (0.0 .. 1.0).
	 @param[in] left Left coordinate of the viewport.
	 @param[in] top Top coordinate of the viewport.
	 @param[in] width Width of the viewport.
	 @param[in] height Height of the viewport.
	 */
	virtual void viewport(const double left, const double top, const double width,
			  const double height);

	/** 
	 Push the modelview matrix on the matrix stack.
	 */
	virtual void pushModelView();

	/** 
	 Pop the modelview matrix from the matrix stack.
	 */
	virtual void popModelView();

	/** 
	 Translate the modelview matrix.
	 @param[in] x X coordinate of the translation vector.
	 @param[in] y Y coordinate of the translation vector.
	 @param[in] z Z coordinate of the translation vector.
	 */
	virtual void translate(const double x, const double y, const double z);

	/** 
	 Rotate the modelview matrix.
	 @param[in] angle Rotation angle in degrees.
	 @param[in] x X coordinate of the rotation vector.
	 @param[in] y Y coordinate of the rotation vector.
	 @param[in] z Z coordinate of the rotation vector.
	 */
	virtual void rotate(const double angle, const double x, const double y, const double z);

	/** 
	 Scale the modelview matrix.
	 @param[in] x X coordinate of the scale vector.
	 @param[in] y Y coordinate of the scale vector.
	 @param[in] z Z coordinate of the scale vector.
	 */
	virtual void scale(const double x, const double y, const double z);

	/** 
	 Set camera position.
	 @param[in] eyeX Specifies the X coordinate of position of the eye point.
	 @param[in] eyeY Specifies the Y coordinate of position of the eye point.
	 @param[in] eyeZ Specifies the Z coordinate of position of the eye point.
	 */
	virtual void setCameraPosition(const double eyeX, const double eyeY, const double eyeZ);

	/** 
	 Set position of camera target.
	 @param[in] centerX Specifies the X coordinate of position of the reference point.
	 @param[in] centerY Specifies the Y coordinate of position of the reference point.
	 @param[in] centerZ Specifies the Z coordinate of position of the reference point.
	 */
	virtual void setCameraTarget(const double centerX, const double centerY,
				  const double centerZ);

	/** 
	 Set up vector of the camera.
	 @param[in] upX Specifies the X coordinate of the direction of the up vector.
	 @param[in] upY Specifies the Y coordinate of the direction of the up vector.
	 @param[in] upZ Specifies the Z coordinate of the direction of the up vector.
	 */
	virtual void setCameraUpVector(const double upX, const double upY, const double upZ);

	/** 
	 Set drawing color.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setColor(const double red, const double green, const double blue,
			  const double alpha);

	/** 
	 Set ambient material color.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setAmbientMaterial(const double red, const double green, const double blue,
					const double alpha);

	/** 
	 Set diffuse material color.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setDiffuseMaterial(const double red, const double green, const double blue,
					const double alpha);

	/** 
	 Set specular material color.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 @param[in] shininess Shininess value.
	 */
	virtual void setSpecularMaterial(const double red, const double green, const double blue,
					 const double alpha, const int shininess);

	/** 
	 Set drawing style.
	 @param[in] drawStyle Draw style code.
	 */
	virtual void setDrawStyle(const unsigned drawStyle);

	/** 
	 Specify whether back-facing facets can be culled.
	 @param[in] isBackCulling Back culling is on.
	 */
	virtual void setBackCulling(const bool isBackCulling);

	/** 
	 Turn on/off blending.
	 @param[in] isOn Blending is on.
	 */
	virtual void setBlending(const bool isOn);

	/** 
	 Draw a triangle width (0,1,0), (0,0,0), (1,0,0) vertices.
	 */
	virtual void drawTriangle();

	/** 

			Generate certain number of 2D or 3D triangles with random coordinates. The vertex coordinates are in the [0..1] interval. Use translate(), rotate(),
			and scale() methods to modify this square (2D) and cube (3D).

	 @param[in] dimension 2 or 3 means 2D and 3D triangles.
	 @param[in] count Triangle count.
	 @return Handle of the generated triangle set. This handle is needed for rendering.
	 */
	virtual int generateRandomTriangles(const int dimension, const int count);

	/** 
	 Render the object with the specified handle.
	 @param[in] handle Handle of object to render.
	 @param[in] useVertexArrays Use vertex arrays for rendering.
	 */
	virtual void renderObject(const u32 handle, const bool useVertexArrays);

	/** 
	 Render specified number of triangles from an object with the given handle.
	 @param[in] handle Handle of object to render.
	 @param[in] useVertexArrays Use vertex arrays for rendering.
	 @param[in] triangleCount Number of triangles.
	 */
	virtual void renderObjectTriangles(const u32 handle, const bool useVertexArrays,
					 const u32 triangleCount);

	/** 
	 Set global ambient light parameters.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setAmbientLight(const double red, const double green, const double blue,
				  const double alpha);

	/** 
	 Remove all light sources.
	 */
	virtual void removeLightSources();

	/** 
	 Add light source.
	 @return Handle of the light. This handle is needed for parameter setting.
	 */
	virtual int addLightSource();

	/** 
	 Set position of the specified light source.
	 @param[in] light Handle of the light.
	 @param[in] x X coordinate.
	 @param[in] y Y coordinate.
	 @param[in] z Z coordinate.
	 */
	virtual void setLightSourcePosition(const int light, const double x, const double y,
					 const double z);

	/** 
	 Set diffuse light parameters for the specified light source.
	 @param[in] light Handle of the light.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setLightSourceDiffuse(const int light, const double red, const double green,
					 const double blue, const double alpha);

	/** 
	 Set specular light parameters for the specified light source.
	 @param[in] light Handle of the light.
	 @param[in] red Red component.
	 @param[in] green Green component.
	 @param[in] blue Blue component.
	 @param[in] alpha Alpha component.
	 */
	virtual void setLightSourceSpecular(const int light, const double red, const double green,
					 const double blue, const double alpha);

	/** 
	 Create a new display list. This will be the active list.
	 @return Handle of the generated display list. This handle is needed for executing the generated list.
	 */
	virtual u32 createDisplayList();

	/** 
	 Finish the active display list.
	 */
	virtual void finishDisplayList();

	/** 
	 Execute the specified display list.
	 @param[in] displayList Handle of display list to execute.
	 */
	virtual void executeDisplayList(const u32 displayList);

	/** 
	 Draw a sphere.
	 @param[in] radius Specifies the radius of the sphere.
	 @param[in] slices Specifies the number of subdivisions around the z axis (similar to lines of longitude).
	 @param[in] stacks Specifies the number of subdivisions along the z axis (similar to lines of latitude).
	 */
	virtual void drawSphere(const double radius, const int slices, const int stacks);

	/** 
	 Draw a cylinder.
	 @param[in] baseRadius Specifies the radius of the cylinder at z = 0.
	 @param[in] topRadius Specifies the radius of the cylinder at z = height.
	 @param[in] height Specifies the height of the cylinder.
	 @param[in] slices Specifies the number of subdivisions around the z axis.
	 @param[in] stacks Specifies the number of subdivisions along the z axis.
	 */
	virtual void drawCylinder(const double baseRadius, const double topRadius,
				const double height, const int slices, const int stacks);

	/** 
	 Draw a disk.
	 @param[in] innerRadius Specifies the inner radius of the disk (may be 0).
	 @param[in] outerRadius Specifies the outer radius of the disk.
	 @param[in] slices Specifies the number of subdivisions around the z axis.
	 @param[in] loops Specifies the number of concentric rings about the origin into which the disk is subdivided.
	 */
	virtual void drawDisk(const double innerRadius, const double outerRadius, const int slices,
			  const int loops);

	/** 
	 Draw a teapot.
	 @param[in] size Specifies the size of the teapot.
	 */
	virtual void drawTeapot(const double size);

	/** 
	 Draw a cube.
	 @param[in] size Specifies the size of the cube.
	 */
	virtual void drawCube(const double size);

	/** 
	 Draw a torus.
	 @param[in] innerRadius Specifies the inner radius of the torus.
	 @param[in] outerRadius Specifies the outer radius of the torus.
	 @param[in] nsides Number of sides for each radial section.
	 @param[in] rings Number of radial divisions for the torus.
	 */
	virtual void drawTorus(const double innerRadius, const double outerRadius, const int nsides,
			   const int rings);

	/** 
	 Draw a dodecahedron.
	 @param[in] size Specifies the size of the dodecahedron.
	 */
	virtual void drawDodecahedron(const double size);

	/** 
	 Draw an octahedron.
	 @param[in] size Specifies the size of the octahedron.
	 */
	virtual void drawOctahedron(const double size);

	/** 
	 Draw a tetrahedron.
	 @param[in] size Specifies the size of the tetrahedron.
	 */
	virtual void drawTetrahedron(const double size);

	/** 
	 Draw an icosahedron.
	 @param[in] size Specifies the size of the icosahedron.
	 */
	virtual void drawIcosahedron(const double size);

   protected:

	/** 
	 Register the specified renderer.
	 @param[in] renderer Register renderer.
	 */
	virtual void _registerRenderer(Renderer * renderer);

	/** 
	 Register the specified object.
	 @param[in] objectData Object to register.
	 @return Handle of the generated triangle set. This handle is needed for rendering.
	 */
	virtual u32 _registerObject(ObjectData objectData);

	/** 
	 Render specified number of vertices from an object with the given handle.
	 @param[in] handle Handle of object to render.
	 @param[in] useVertexArrays Use vertex arrays for rendering.
	 @param[in] vertexCount Number of vertices.
	 */
	virtual void _renderObject(const u32 handle, const bool useVertexArrays,
				 const u32 vertexCount);

	/** 
	 Refresh camera setting in OpenGL.
	 */
	virtual void _refreshCamera();

	/** 
	 Map current draw style to draw style of GLU quadratics.
	 @param[in] quadratic Quadratic to set the draw style.
	 @return True if the quadratic has to be rendered, false otherwise
	 */
	virtual bool _setGLUDrawStyle(const void *quadratic);

	/** 
	 Report certain number of triangles being rendered to the corresponding render window. This is needed
			for window statistics.
	 @param[in] triangleCount Number of rendered triangles.
	 */
	virtual void _reportTriangles(const u32 & triangleCount);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMOpenGLRenderingEngine_impl.h"

}

// Declare OpenGLRenderingEngine class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::OpenGLRenderingEngine, OpenGLRenderingEngine);

#endif
