
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMPCWrapper.h
/// Contains declaration of class PCWrapper

#ifndef __PCM_PCWRAPPER_H__
#  define __PCM_PCWRAPPER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

//
// Outer includes
//

#  include <pcapi.h>

namespace ParCompMark
{

  /**
	Place the class description here!
   */

  class PCWrapper
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestPCWrapper;
#  endif

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Wraps a PC function.
	 @param[in] netId 
	 @param[in] thisHost 
	 @param[in] ctx 
	 @return 
	 */
	static PCerr contextCreate(PCid netId, PCstring thisHost, PCcontext * ctx);

	/** 
	 Wraps a PC function.
	 @param[in] properties 
	 @param[in] hosts 
	 @param[in] thisHost 
	 @param[in] ctx 
	 @return 
	 */
	static PCerr contextCreateMaster(const PCint * properties, PCstring * hosts, PCstring thisHost,
					 PCcontext * ctx);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @return 
	 */
	static PCerr contextDestroy(PCcontext ctx);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] property 
	 @param[in] hostIndex 
	 @param[in] value 
	 @return 
	 */
	static PCerr contextGetInteger(PCcontext ctx, PCenum property, PCint hostIndex, PCint * value);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] property 
	 @param[in] hostIndex 
	 @param[in] value 
	 @return 
	 */
	static PCerr contextSetInteger(PCcontext ctx, PCenum property, PCint hostIndex, PCint value);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @return 
	 */
	static PCerr contextSync(PCcontext ctx);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] frameId 
	 @param[in] color 
	 @param[in] depth 
	 @return 
	 */
	static PCerr frameAddFramelet(PCcontext ctx, PCid frameId, PCvoidp color, PCvoidp depth);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] frameId 
	 @param[in] xOffset 
	 @param[in] yOffset 
	 @return 
	 */
	static PCerr frameAddGLFrameletEXT(PCcontext ctx, PCid frameId, PCint xOffset, PCint yOffset);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] frameId 
	 @param[in] numFramelets 
	 @param[in] xOffset 
	 @param[in] yOffset 
	 @param[in] width 
	 @param[in] height 
	 @param[in] sortorder 
	 @return 
	 */
	static PCerr frameBegin(PCcontext ctx, PCid * frameId, PCuint numFramelets, PCuint * xOffset, PCuint * yOffset,
				PCuint * width, PCuint * height, PCuint sortorder);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] frameId 
	 @return 
	 */
	static PCerr frameEnd(const PCcontext ctx, const PCid frameId);

	/** 
	 Wraps a PC function.
	 @param[in] ctx 
	 @param[in] frameId 
	 @param[in] xOffset 
	 @param[in] yOffset 
	 @param[in] width 
	 @param[in] height 
	 @param[in] channelType 
	 @param[in] channel 
	 @return 
	 */
	static PCerr frameResultChannel(PCcontext ctx, PCid frameId, PCuint xOffset, PCuint yOffset, PCuint width,
					PCuint height, PCuint channelType, PCchannel channel);

	/** 
	 Wraps a PC function.
	 @param[in] error 
	 @return 
	 */
	static PCstring getErrorString(PCerr error);

	/** 
	 Wraps a PC function.
	 @param[in] extensionName 
	 @return 
	 */
	static PCint queryExtension(PCstring extensionName);

	/** 
	 Wraps a PC function.
	 @param[in] sessionId 
	 @return 
	 */
	static PCerr sessionCreate(PCid sessionId);

	/** 
	 Wraps a PC function.
	 @return 
	 */
	static PCerr sessionDestroy();

	/** 
	 Wraps a PC function.
	 @param[in] property 
	 @param[in] qualifier 
	 @param[in] value 
	 @return 
	 */
	static PCerr systemGetInteger(PCenum property, PCint qualifier, PCint * value);

	/** 
	 Wraps a PC function.
	 @return 
	 */
	static PCerr systemFinalize();

	/** 
	 Wraps a PC function.
	 @param[in] property 
	 @param[in] qualifier 
	 @param[in] value 
	 @return 
	 */
	static PCerr systemGetString(PCenum property, PCint qualifier, PCstring * value);

	/** 
	 Wraps a PC function.
	 @param[in] searchPath 
	 @return 
	 */
	static PCerr systemInitialize(const PCstring searchPath);

	//@}  

  };

}

#endif
