
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMProcess.h
/// Contains declaration of class Process

#ifndef __PCM_PROCESS_H__
#  define __PCM_PROCESS_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMThread.h"

#  include "PCMName.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMOpenGLRenderingEngine.h"

#  include "PCMContext.h"

#  include "PCMThread.h"

#  include "PCMName.h"

#  include "PCMBuffer.h"

#  include "PCMGLXRenderWindow.h"

#  include "PCMSqVM.h"

#  include "PCMOutputNode.h"

#  include "PCMStringConverter.h"

//
// Outer includes
//

#  include <pcapi.h>

#  include <string>

#  include <sstream>

#  include <list>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Entity that composite or render something.
   */

  class Process:public Thread,
   public Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestProcess;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Process > (_T("Process")).func(&Process::addContext,
								_T("addContext")).func(&Process::setProcessTypeSq,
											 _T("setProcessTypeSq")).
		func(&Process::getProcessTypeSq, _T("getProcessTypeSq")).func(&Process::setBufferByName,
									  _T("setBufferByName")).
		func(&Process::getInitProcCode, _T("getInitProcCode")).func(&Process::setInitProcCode,
									 _T("setInitProcCode")).
		func(&Process::getRunningProcCode, _T("getRunningProcCode")).func(&Process::setRunningProcCode,
										_T("setRunningProcCode")).
		func(&Process::getSortOrder, _T("getSortOrder")).func(&Process::setSortOrder,
								  _T("setSortOrder")).func(&Process::getStopID,
											   _T("getStopID")).
		func(&Process::setStopID, _T("setStopID")).func(&Process::getOperate,
								_T("getOperate")).func(&Process::setOperate,
										   _T("setOperate")).
		func(&Process::getDisplayOutput, _T("getDisplayOutput")).func(&Process::setDisplayOutput,
									  _T("setDisplayOutput")).
		func(&Process::getGatherStatistics, _T("getGatherStatistics")).func(&Process::setGatherStatistics,
										 _T("setGatherStatistics")).
		func(&Process::getExportFrameStep, _T("getExportFrameStep")).func(&Process::setExportFrameStep,
										_T("setExportFrameStep")).
		func(&Process::getFrameFilenamePattern,
		  _T("getFrameFilenamePattern")).func(&Process::setFrameFilenamePattern,
							 _T("setFrameFilenamePattern")).enumInt(Process::COMPOSITE,
												_T("COMPOSITE")).
		enumInt(Process::RENDER, _T("RENDER"));
	}

	//@}  

	//
	// Enumerations
	//

   public:

  /** The control type of PC context (Local, Global). */
	enum ProcessType
	{

	/** Process composite and provides frame and requires framelets. */
	 COMPOSITE = 0,

	/** Process provides framelets. */
	 RENDER = 1
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < Process,
	Mutex > Pointer;

  /** Type for context list. One process can render in several context. */
	typedef std::list < Context::Pointer > ContextList;

  /** Type for context list iterator. */
	typedef ContextList::iterator ContextListIterator;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Process initializer Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string PROCESSINITNUT;

	/** Default filename pattern for exporting frames.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string DEFAULTEXPORTPATTERN;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** List of contexts.

	 @remarks
	 This is own attribute of this class.
	*/
	ContextList mContexts;

	/** Composite or render process.

	 @remarks
	 This is own attribute of this class.
	*/
	ProcessType mProcessType;

	/** Parent node of the process. (Parent reference is standard pointer to avoid circular reference)

	 @remarks
	 This attribute references an attribute.
	*/
	Node *mParent;

	/** Renderwindow of this process. Each process can have zero or one renderwindow.

	 @remarks
	 This is own attribute of this class.
	*/
	GLXRenderWindow::Pointer mRenderWindow;

	/** Rendering engine of this process.

	 @remarks
	 This is own attribute of this class.
	*/
	OpenGLRenderingEngine::Pointer mRenderingEngine;

	/** Squirrel virtual machine.

	 @remarks
	 This is own attribute of this class.
	*/
	SqVM::Pointer mSqVM;

	/** Squirrel initialization code (C string for squirrel compatibility).

	 @remarks
	 This is own attribute of this class.
	*/
	char *mInitProcCode;

	/** Squirrel running code (C string for squirrel compatibility).

	 @remarks
	 This is own attribute of this class.
	*/
	char *mRunningProcCode;

	/** Graphics memory buffer on which the process operates. The buffer belongs the to parent node.

	 @remarks
	 This is own attribute of this class.
	*/
	Buffer::Pointer mBuffer;

	/** The process is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Value for alpha sorting.

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint mSortOrder;

	/** Stop id where composite is ended.

	 @remarks
	 This is own attribute of this class.
	*/
	PCid mStopID;

	/** 
			Flag indicates that the process operates, ie. renders the framelet or composites the specified output.
			This flag is useful for dynamic rendering defined by Squirrel task script or plugins.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mOperate;

	/** 
			Flag indicates that the output of the compositing process is rendered on the screen. This flag is dontcare
			on rendering processes.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mDisplayOutput;

	/** This flag indicates that the process has to gather rendering statistics

	 @remarks
	 This is own attribute of this class.
	*/
	bool mGatherStatistics;

	/** Number of current frame.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mFrameNumber;

	/** Time elapsed between the start of the benchmark and the start of the current frame.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mFrameTime;

	/** Local (uncorrected) time at the start of the benchmark for this process.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mStartTime;

	/** Indicates that process must be stopped.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mStop;

	/** Indicates that process can be stopped during compositing.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mStopAble;

	/** Indicates that process can call application that the compositing is ended.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mCountedStop;

	/** Process output document.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mOutputDocument;

	/** Show little framelet icon on rendering windows.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mShowFrameletIcon;

	/** The process will export any (mExportFrameStep)th frame in mFrameFilenamePattern format. Zero value disables frame exporing.

	 @remarks
	 This is own attribute of this class.
	*/
	int mExportFrameStep;

	/** Filename pattern for exporting frames (C string for squirrel compatibility). The filename pattern can contain special formatting characters to 
			include the frame number. For details see the man page of the 

	 @remarks
	 This is own attribute of this class.
	*/
	char *mFrameFilenamePattern;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	Process();

	/** 
	 Creates a process with a specified name.
	 @param[in] name Name of the host.
	 @param[in] parent Parent node
	 */
	Process(const std::string & name, Node * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Process();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mContexts. Returns value of mContexts.

	 @return The value of mContexts
	*/
	Process::ContextList & getContexts();

	/**
	 Getter of mProcessType. Returns value of mProcessType.

	 @return The value of mProcessType
	*/
	Process::ProcessType & getProcessType();

	/**
	 Setter of mProcessType. Sets value of mProcessType.

	 @param[in] processtype The value of mProcessType
	*/
	void setProcessType(const Process::ProcessType & processtype);

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Node *getParent() const;

	/**
	 Getter of mRenderWindow. Returns value of mRenderWindow.

	 @return The value of mRenderWindow
	*/
	GLXRenderWindow::Pointer & getRenderWindow();

	/**
	 Getter of mRenderingEngine. Returns value of mRenderingEngine.

	 @return The value of mRenderingEngine
	*/
	OpenGLRenderingEngine::Pointer & getRenderingEngine();

	/**
	 Getter of mInitProcCode. Returns value of mInitProcCode.

	 @return The value of mInitProcCode
	*/
	char *getInitProcCode();

	/**
	 Setter of mInitProcCode. Sets value of mInitProcCode.

	 @param[in] initproccode The value of mInitProcCode
	*/
	void setInitProcCode(const char *initproccode);

	/**
	 Getter of mRunningProcCode. Returns value of mRunningProcCode.

	 @return The value of mRunningProcCode
	*/
	char *getRunningProcCode();

	/**
	 Setter of mRunningProcCode. Sets value of mRunningProcCode.

	 @param[in] runningproccode The value of mRunningProcCode
	*/
	void setRunningProcCode(const char *runningproccode);

	/**
	 Getter of mBuffer. Returns value of mBuffer.

	 @return The value of mBuffer
	*/
	Buffer::Pointer & getBuffer();

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mSortOrder. Returns value of mSortOrder.

	 @return The value of mSortOrder
	*/
	PCuint & getSortOrder();

	/**
	 Setter of mSortOrder. Sets value of mSortOrder.

	 @param[in] sortorder The value of mSortOrder
	*/
	void setSortOrder(const PCuint sortorder);

	/**
	 Getter of mStopID. Returns value of mStopID.

	 @return The value of mStopID
	*/
	PCid & getStopID();

	/**
	 Setter of mStopID. Sets value of mStopID.

	 @param[in] stopid The value of mStopID
	*/
	void setStopID(const PCid stopid);

	/**
	 Getter of mOperate. Returns value of mOperate.

	 @return The value of mOperate
	*/
	bool & getOperate();

	/**
	 Setter of mOperate. Sets value of mOperate.

	 @param[in] operate The value of mOperate
	*/
	void setOperate(const bool operate);

	/**
	 Getter of mDisplayOutput. Returns value of mDisplayOutput.

	 @return The value of mDisplayOutput
	*/
	bool & getDisplayOutput();

	/**
	 Setter of mDisplayOutput. Sets value of mDisplayOutput.

	 @param[in] displayoutput The value of mDisplayOutput
	*/
	void setDisplayOutput(const bool displayoutput);

	/**
	 Getter of mGatherStatistics. Returns value of mGatherStatistics.

	 @return The value of mGatherStatistics
	*/
	bool & getGatherStatistics();

	/**
	 Setter of mGatherStatistics. Sets value of mGatherStatistics.

	 @param[in] gatherstatistics The value of mGatherStatistics
	*/
	void setGatherStatistics(const bool gatherstatistics);

	/**
	 Getter of mFrameNumber. Returns value of mFrameNumber.

	 @return The value of mFrameNumber
	*/
	const u32 & getFrameNumber() const;

	/**
	 Getter of mFrameTime. Returns value of mFrameTime.

	 @return The value of mFrameTime
	*/
	const Real & getFrameTime() const;

	/**
	 Getter of mStartTime. Returns value of mStartTime.

	 @return The value of mStartTime
	*/
	const Real & getStartTime() const;

	/**
	 Setter of mStartTime. Sets value of mStartTime.

	 @param[in] starttime The value of mStartTime
	*/
	void setStartTime(const Real & starttime);

	/**
	 Getter of mStop. Returns value of mStop.

	 @return The value of mStop
	*/
	const bool & getStop() const;

	/**
	 Setter of mStop. Sets value of mStop.

	 @param[in] stop The value of mStop
	*/
	void setStop(const bool & stop);

	/**
	 Getter of mStopAble. Returns value of mStopAble.

	 @return The value of mStopAble
	*/
	const bool & getStopAble() const;

	/**
	 Setter of mStopAble. Sets value of mStopAble.

	 @param[in] stopable The value of mStopAble
	*/
	void setStopAble(const bool & stopable);

	/**
	 Getter of mCountedStop. Returns value of mCountedStop.

	 @return The value of mCountedStop
	*/
	const bool & getCountedStop() const;

	/**
	 Setter of mCountedStop. Sets value of mCountedStop.

	 @param[in] countedstop The value of mCountedStop
	*/
	void setCountedStop(const bool & countedstop);

	/**
	 Getter of mOutputDocument. Returns value of mOutputDocument.

	 @return The value of mOutputDocument
	*/
	OutputNode::Pointer & getOutputDocument();

	/**
	 Getter of mShowFrameletIcon. Returns value of mShowFrameletIcon.

	 @return The value of mShowFrameletIcon
	*/
	const bool & getShowFrameletIcon() const;

	/**
	 Getter of mExportFrameStep. Returns value of mExportFrameStep.

	 @return The value of mExportFrameStep
	*/
	int &getExportFrameStep();

	/**
	 Setter of mExportFrameStep. Sets value of mExportFrameStep.

	 @param[in] exportframestep The value of mExportFrameStep
	*/
	void setExportFrameStep(const int exportframestep);

	/**
	 Getter of mFrameFilenamePattern. Returns value of mFrameFilenamePattern.

	 @return The value of mFrameFilenamePattern
	*/
	char *getFrameFilenamePattern();

	/**
	 Setter of mFrameFilenamePattern. Sets value of mFrameFilenamePattern.

	 @param[in] framefilenamepattern The value of mFrameFilenamePattern
	*/
	void setFrameFilenamePattern(const char *framefilenamepattern);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Some init step as thread.
	 */
	virtual void threadInitialize();

	/** 
	 Some finitiate step as thread.
	 */
	virtual void threadFinalize();

	/** 
	 Start the process.
	 */
	virtual void start();

	/** 
	 Stop the process.
	 @return frameID
	 */
	virtual u32 stop();

	/** 
	 Create and add a context to the context list.
	 @return The new context
	 */
	virtual Context *addContext();

	/** 
	 Intialize PC functionality.
	 */
	virtual void initPC();

	/** 
	 Open GLX render window for rendering.
	 @param[in] caption Window caption
	 @param[in] displayName X display name
	 @param[in] fullScreen The window appears in full screen mode
	 @param[in] colourDepth Colour depth of the window
	 @param[in] width Horizontal size
	 @param[in] height Vertical size
	 @param[in] left Horizontal position
	 @param[in] top Vertical position
	 @param[in] fsaaSamples Number of fullscreen antialasing samples (Do not use FSAA samples other than 0 now with nVidia cards!)
	 */
	virtual void openRenderWindow(const std::string caption =
				  "PCM Framework", const std::string & displayName =
				  "", const bool & fullScreen = true, const u32 & colourDepth =
				  0, const u32 & width =
				  GLXRenderWindow::MAXIMALSIZE, const u32 & height =
				  GLXRenderWindow::MAXIMALSIZE, const s32 & left =
				  GLXRenderWindow::CENTERED, const s32 & top =
				  GLXRenderWindow::CENTERED, const u32 & fsaaSamples = 0);

	/** 
	 Actualize GLX render window on PC information (frame sizes etc).
	 */
	virtual void actualizeRenderWindow();

	/** 
	 Setup viewport for the rendering process based on the context properties.
	 */
	virtual void setViewportForRendering();

	/** 
	 Display small icon on to top-left corner of the GLX window indicating 
			the frame/framelet relationship for this process.
	 */
	virtual void displayFrameletIcon();

	/** 
	 This is an initializing method. 
			Called at the start of the working process.
			Starts a Squirrel VM and executes the initialization code.

	 */
	virtual void initProcess();

	/** 
	 This method achieves the "real functionality" of 
			the process. Called at every frame.
			Starts a Squirrel VM and executes the initialization code.

	 */
	virtual void runningProcess();

	/** 
	 Alternate squirrel setter of mProcessType.
	 @param[in] processType Process type int value (0 means COMPOSITE, 1 means RENDER).
	 */
	virtual void setProcessTypeSq(const int processType);

	/** 
	 Alternate squirrel setter of mProcessType.
	 @return Process type int value (0 means COMPOSITE, 1 means RENDER).
	 */
	virtual int getProcessTypeSq();

	/** 
	 Set mBuffer by name. mBuffer will be retrieved from the buffer container of the parent node by the name of the buffer.
	 @param[in] name Name of the buffer (C string for squirrel compatibility).
	 */
	virtual void setBufferByName(const char *name);

	/** 
	 Set parameters to finite pc.
	 @param[in] frameID StopID.
	 */
	virtual void setFinito(const u32 & frameID);

   protected:

	/** 
	 What we do during rendering/compositing.
	 */
	virtual void task();

	/** 
	 Refresh the sort order of the framelet produced by this process.
	 */
	virtual void refreshSortOrder();

	/** 
	 Gather rendering statistics for the current frame.
	 */
	virtual void gatherStatistics();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMProcess_impl.h"

}

// Declare Process class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Process, Process);

DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Process::ProcessType, Process::ProcessType);

#endif
