
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMRendererPlugin_impl.h
/// Contains header implementation of class RendererPlugin

 //
 // Getters & setters
 //

inline const std::list < std::string > &RendererPlugin::getNeededOpenGLExts() const
{
  return mNeededOpenGLExts;
}

 /*-------------------------------------------------------------------------*/

 //
 // Methods
 //

inline void RendererPlugin::resize(void *rendererHandle, const u32 & width, const u32 & height)
{
  mLastError = mPcmOnResize(rendererHandle, width, height);
  _checkError();
}

 /*----------------------------------------------------------------------*/

inline s32 RendererPlugin::getSortOrder(void *rendererHandle)
{
  // Call pcmGetSortOrder function if the plugin has it
  return mPcmGetSortOrder ? mPcmGetSortOrder(rendererHandle) : -1;
}

 /*----------------------------------------------------------------------*/

inline void RendererPlugin::render(void *rendererHandle, const double &time, const unsigned &frame)
{
  mLastError = mPcmOnRender(rendererHandle, time, frame);
  _checkError();
}

 /*-------------------------------------------------------------------------*/

 //
 // Methods
 //

inline void RendererPlugin::setMiscParam(void *rendererHandle, const char *&name, const char *&value)
{
  Assert(hasFunction("pcmSetMiscParam"), INVALID_NAME_ERROR, "RendererPlugin::setMiscParam()");

  pcmSetMiscParamType pcmSetMiscParam = (pcmSetMiscParamType) getFunction("pcmSetMiscParam");

  mLastError = pcmSetMiscParam(rendererHandle, name, value);
  _checkError();

  Logger::getInstance()->log(Logger::DEBUG,
			   std::string("Misc parameter (") + name + "=" + value + ") is set for renderer plugin `" +
			   mName + "\'.");
}

 /*----------------------------------------------------------------------*/

inline void RendererPlugin::setObjectSpaceBoundingBox(void *rendererHandle, const double &x0,
						  const double &y0, const double &z0,
						  const double &x1, const double &y1,
						  const double &z1)
{
  Assert(hasFunction("pcmSetObjectSpaceBoundingBox"), INVALID_NAME_ERROR,
	 "RendererPlugin::setObjectSpaceBoundingBox()");

  pcmSetObjectSpaceBoundingBoxType pcmSetObjectSpaceBoundingBox =
	(pcmSetObjectSpaceBoundingBoxType) getFunction("pcmSetObjectSpaceBoundingBox");

  mLastError = pcmSetObjectSpaceBoundingBox(rendererHandle, x0, y0, z0, x1, y1, z1);
  _checkError();

  Logger::getInstance()->log(Logger::DEBUG,
			   "Object space bounding box (" + StringConverter::toString(x0) + "," +
			   StringConverter::toString(y0) + "," + StringConverter::toString(z0) + ")-(" +
			   StringConverter::toString(x1) + "," + StringConverter::toString(y1) + "," +
			   StringConverter::toString(z1) + ") is set for renderer plugin `" + mName + "\'.");
}

 /*----------------------------------------------------------------------*/

inline void RendererPlugin::setObjectId(void *rendererHandle, const unsigned &objectId)
{
  Assert(hasFunction("pcmSetObjectId"), INVALID_NAME_ERROR, "RendererPlugin::setObjectId()");

  pcmSetObjectIdType pcmSetObjectId = (pcmSetObjectIdType) getFunction("pcmSetMiscParam");

  mLastError = pcmSetObjectId(rendererHandle, objectId);
  _checkError();

  Logger::getInstance()->log(Logger::DEBUG,
			   "Object id #" + StringConverter::toString(objectId) + " is set for renderer plugin `" +
			   mName + "\'.");
}

 /*----------------------------------------------------------------------*/

inline void RendererPlugin::setScreenSpaceFramelet(void *rendererHandle, const double &u0, const double &v0,
						 const double &u1, const double &v1)
{
  Assert(hasFunction("pcmSetScreenSpaceFramelet"), INVALID_NAME_ERROR, "RendererPlugin::setScreenSpaceFramelet()");

  pcmSetScreenSpaceFrameletType pcmSetScreenSpaceFramelet =
	(pcmSetScreenSpaceFrameletType) getFunction("pcmSetScreenSpaceFramelet");

  mLastError = pcmSetScreenSpaceFramelet(rendererHandle, u0, v0, u1, v1);
  _checkError();

  Logger::getInstance()->log(Logger::DEBUG,
			   "Screen space framelet (" + StringConverter::toString(u0) + "," +
			   StringConverter::toString(v0) + ")-(" + StringConverter::toString(u1) + "," +
			   StringConverter::toString(v1) + ") is set for renderer plugin `" + mName + "\'.");
}

 /*----------------------------------------------------------------------*/

inline void RendererPlugin::registerObjectSpaceBoundingBox(void *rendererHandle, const double &x0,
							 const double &y0, const double &z0,
							 const double &x1, const double &y1,
							 const double &z1)
{
  Assert(hasFunction("pcmRegisterObjectSpaceBoundingBox"), INVALID_NAME_ERROR,
	 "RendererPlugin::registerObjectSpaceBoundingBox()");

  pcmSetObjectSpaceBoundingBoxType pcmSetObjectSpaceBoundingBox =
	(pcmSetObjectSpaceBoundingBoxType) getFunction("pcmRegisterObjectSpaceBoundingBox");

  mLastError = pcmSetObjectSpaceBoundingBox(rendererHandle, x0, y0, z0, x1, y1, z1);
  _checkError();

  Logger::getInstance()->log(Logger::DEBUG,
			   "Register object space bounding box (" + StringConverter::toString(x0) + "," +
			   StringConverter::toString(y0) + "," + StringConverter::toString(z0) + ")-(" +
			   StringConverter::toString(x1) + "," + StringConverter::toString(y1) + "," +
			   StringConverter::toString(z1) + ") for renderer plugin `" + mName + "\'.");
}

 /*----------------------------------------------------------------------*/
