
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMRenderer_impl.h
/// Contains header implementation of class Renderer

 //
 // Getters & setters
 //

inline void *Renderer::getRendererHandle() const
{
  return mRendererHandle;
}

 /*-------------------------------------------------------------------------*/

inline void Renderer::setRendererHandle(const void *rendererhandle)
{
  mRendererHandle = (void *) rendererhandle;
}

 /*-------------------------------------------------------------------------*/

inline OpenGLRenderingEngine *Renderer::getParent() const
{
  return mParent;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Renderer::getInitialized() const
{
  return mInitialized;
}

 /*-------------------------------------------------------------------------*/

inline s32 & Renderer::getAutoRenderOrder()
{
  return mAutoRenderOrder;
}

 /*-------------------------------------------------------------------------*/

inline void Renderer::setAutoRenderOrder(const s32 autorenderorder)
{
  mParent->setAutoRenderOrder(this, mAutoRenderOrder = autorenderorder);
}

 /*-------------------------------------------------------------------------*/

 //
 // Methods
 //

inline void Renderer::resize(const u32 width, const u32 height)
{
  mRendererPlugin->resize(mRendererHandle, (const unsigned) width, (const unsigned) height);
}

 /*----------------------------------------------------------------------*/

inline s32 Renderer::getSortOrder()
{
  return mRendererPlugin->getSortOrder(mRendererHandle);
}

 /*----------------------------------------------------------------------*/

inline void Renderer::render()
{
  mRendererPlugin->render(mRendererHandle, mParent->getParent()->getFrameTime(),
			 mParent->getParent()->getFrameNumber());
}

 /*----------------------------------------------------------------------*/

inline void Renderer::setMiscParam(const char *name, const char *value)
{
  mRendererPlugin->setMiscParam(mRendererHandle, name, value);
}

 /*----------------------------------------------------------------------*/

inline void Renderer::setObjectSpaceBoundingBox(const double x0, const double y0, const double z0,
						const double x1, const double y1, const double z1)
{
  mRendererPlugin->setObjectSpaceBoundingBox(mRendererHandle, x0, y0, z0, x1, y1, z1);
}

 /*----------------------------------------------------------------------*/

inline void Renderer::setObjectId(const unsigned objectId)
{
  mRendererPlugin->setObjectId(mRendererHandle, objectId);
}

 /*----------------------------------------------------------------------*/

inline void Renderer::setScreenSpaceFramelet(const double u0, const double v0, const double u1,
					  const double v1)
{
  mRendererPlugin->setScreenSpaceFramelet(mRendererHandle, u0, v0, u1, v1);
}

 /*----------------------------------------------------------------------*/

inline void Renderer::registerObjectSpaceBoundingBox(const double x0, const double y0,
						  const double z0, const double x1,
						  const double y1, const double z1)
{
  mRendererPlugin->registerObjectSpaceBoundingBox(mRendererHandle, x0, y0, z0, x1, y1, z1);
}

 /*----------------------------------------------------------------------*/
