
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMSingleton.h
/// Contains declaration of class Singleton

#ifndef __PCM_SINGLETON_H__
#  define __PCM_SINGLETON_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

namespace ParCompMark
{

  /**
	Template class for creating single-instance global classes.
   */
  template < typename T > class Singleton
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestSingleton;
#  endif

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Class level instance. 

	 @remarks
	 This is own attribute of this class.
	*/
	static T *mInstance;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Singleton();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Singleton();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Gets the instance of the singleton class. You have to construct singleton object (createInstance) before calling this method.
	 @return Instance of the class
	 */
	static T *getInstance();

	/** 
	 Create singleton instance.
	 */
	static void createInstance();

	/** 
	 Destroy singleton instance.
	 */
	static void destroyInstance();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMSingleton_impl.h"

}

#endif
