
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMThread_impl.h
/// Contains header implementation of class Thread

 //
 // Getters & setters
 //

inline const std::string & Thread::getThreadName() const
{
  return mThreadName;
}

 /*-------------------------------------------------------------------------*/

inline const Thread::BoolPointer & Thread::getWait() const
{
  return mWait;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Thread::getStopRequested() const
{
  return mStopRequested;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Thread::getJoinable() const
{
  return mJoinable;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Thread::getWaitThread() const
{
  return mWaitThread;
}

 /*-------------------------------------------------------------------------*/

inline const bool & Thread::getRunning() const
{
  return mRunning;
}

 /*-------------------------------------------------------------------------*/

inline const u32 & Thread::getCurrentFPS() const
{
  return mCurrentFPS;
}

 /*-------------------------------------------------------------------------*/

inline const u32 & Thread::getExpectedFPS() const
{
  return mExpectedFPS;
}

 /*-------------------------------------------------------------------------*/

inline const u32 & Thread::getIterationNumber() const
{
  return mIterationNumber;
}

 /*-------------------------------------------------------------------------*/

 //
 // Class methods
 //

inline void Thread::yield()
{
  sched_yield();
}

 /*----------------------------------------------------------------------*/

inline Real Thread::getUSTime()
{
  struct timeval tv;
  struct timezone tz;

  gettimeofday(&tv, &tz);
  return tv.tv_sec * 1000.0 * 1000.0 + tv.tv_usec;
}

 /*----------------------------------------------------------------------*/

 //
 // Methods
 //

inline bool Thread::iteration()
{
  double usStart,
	usEnd;

  usStart = Thread::getUSTime();
  //usleep(sleepTime);

  // Execute task
  task();

  // Calculate fps
  usEnd = Thread::getUSTime();
  mCurrentFPS = (u32) (1000.0 * 1000.0 / (usEnd - usStart));

  //differenceFPS = mExpectedFPS - mCurrentFPS;

		/**if(differenceFPS > 0 && mExpectedFPS != 0)
		{

		 sleepTime = (u32) ((1000.0 * 1000.0) / differenceFPS);

		}*/

  // Yield the thread
  yield();

  // Test stop flag
  if(mStopRequested)
  {
	mStopRequested = false;
	return true;
  }

  return false;
}

 /*----------------------------------------------------------------------*/
