
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMTimer_impl.h
/// Contains header implementation of class Timer

 //
 // Getters & setters
 //

inline const Real & Timer::getTimeCorrection()
{
  return mTimeCorrection;
}

 /*-------------------------------------------------------------------------*/

inline void Timer::setTimeCorrection(const Real & timecorrection)
{
  mTimeCorrection = timecorrection;
  mStartTime = getUncorrectedSystemTime();
}

 /*-------------------------------------------------------------------------*/

inline const Real & Timer::getStartTime()
{
  return mStartTime;
}

 /*-------------------------------------------------------------------------*/

 //
 // Class methods
 //

inline Real Timer::getSystemTime()
{
  return getUncorrectedSystemTime() + mTimeCorrection;
}

 /*----------------------------------------------------------------------*/

inline Real Timer::getApplicationTime()
{
  return getUncorrectedSystemTime() - mStartTime;
}

 /*----------------------------------------------------------------------*/

inline Real Timer::getUncorrectedSystemTime()
{
  struct timeval tv;
  struct timezone tz;

  gettimeofday(&tv, &tz);
  return tv.tv_sec * 1.0 + tv.tv_usec * 0.000001;
}

 /*----------------------------------------------------------------------*/

inline void Timer::sleep(const Real time)
{
  usleep((__useconds_t) (time * 1.0e6));
}

 /*----------------------------------------------------------------------*/
