
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMDynLoad.h"

//
// Outer includes
//

#include <dlfcn.h>

namespace ParCompMark
{

  //
  // Constructors & destructor
  //

  DynLoad::DynLoad(const std::string & libName)
	// You have to initialize the following attributes:
	// - mHandle
	// - mLibraryName
  {
	mLibraryName = libName;
	load();
  }

 /*----------------------------------------------------------------------*/

  DynLoad::~DynLoad()
  {
	if(mHandle)
	{
	 unload();
	 mHandle = 0;
	}
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  bool DynLoad::hasFunction(const std::string & funcName) const
  {
	Assert(mHandle, INVALID_OPERATION_ERROR, "DynLoad::hasFunction()");
	void *f = dlsym(mHandle, funcName.c_str());

	return f != 0;
  }

 /*----------------------------------------------------------------------*/

  void *DynLoad::getFunction(const std::string & funcName) const
  {
	Assert(mHandle, INVALID_OPERATION_ERROR, "DynLoad::getFunction()");

	char *m;

	void *f = dlsym(mHandle, funcName.c_str());

	if(!f)
	{
	 m = dlerror();
	 if(m)
		Except(INTERNAL_ERROR, "DynLoad::getFunction()", m);
	}

	return f;

  }

 /*----------------------------------------------------------------------*/

  void DynLoad::load()
  {
	if((mHandle = dlopen(mLibraryName.c_str(), RTLD_LAZY)) == NULL)
	{
	 Except(INTERNAL_ERROR, "DynLoad::load()", dlerror());
	}

	Logger::getInstance()->log(Logger::NOTICE, mLibraryName + " loaded.");
  }

 /*----------------------------------------------------------------------*/

  void DynLoad::unload()
  {

	if(dlclose(mHandle) != 0)
	{
	 Except(INTERNAL_ERROR, "DynLoad::unload()", dlerror());
	}

	Logger::getInstance()->log(Logger::NOTICE, mLibraryName + " unloaded.");
  }

 /*----------------------------------------------------------------------*/

}
