
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMLogger.h"

#include "../include/PCMFileSystemManager.h"

#include "../include/PCMNetwork.h"

namespace ParCompMark
{

  // Inherited static attribute initialization
  template <> Logger * Singleton < Logger >::mInstance = 0;

  //
  // Class constants
  //

  const u8 Logger::LOGTOCONSOLE = 1;
  const u8 Logger::LOGTOFILE = 2;

  //
  // Constructors & destructor
  //

   Logger::Logger(const std::string & logFileName)
	// You have to initialize the following attributes:
	// - mLogMode
	// - mConsoleLogLevel
	// - mFileLogLevel
	// - mFp
	// - mInitialized
	// - mLogFileName
  {
	mLogMode = LOGTOCONSOLE | LOGTOFILE;
	mConsoleLogLevel = WARNING;
	mFileLogLevel = DEBUG;
	mFp = (FILE *) 0;
	mInitialized = false;
	Network::getIP();
	mLogFileName = Network::getIfConfs()->get("eth0")->IP + '-' + logFileName;
  }

 /*----------------------------------------------------------------------*/

  Logger::~Logger()
  {
	if(mInitialized)
	{
	 log(NOTICE, "Logger has been shut down.");
	 mInitialized = false;

	 // Close file
	 FileSystemManager::closeFile(mFp);
	}
  }

 /*----------------------------------------------------------------------*/

  //
  // Class methods
  //

  std::string Logger::translateException(const Exception & exception)
  {
	std::ostringstream line;
	line << Exception::translateType(exception.getType()) << ' ' << exception.
	 getDescription() << " in file `" << exception.getFileName() << '\'' << " in method `" << exception.
	 getFunctionName() << '\'' << " at line " << exception.
	 getLineNumber() << " on host `" << Network::getIfConfs()->get("eth0")->IP << "\'";

	return line.str();
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  void Logger::initialize()
  {
	if(mLogMode & LOGTOFILE)
	{

	 // Create file manager if not exist
	 if(!FileSystemManager::getInstance())
	 {
		FileSystemManager::createInstance();
		FileSystemManager::getInstance()->initialize();
	 }

	 mFp = FileSystemManager::getInstance()->openAppFileC(mLogFileName, FileSystemManager::WRITE);
	 Assert(mFp.getPtr(), INVALID_NAME_ERROR, "Logger::init()");

	 mInitialized = true;
	 log(NOTICE, "Logger has been initialized at " + Timer::getTimeDateString() + ".");
	}
  }

 /*----------------------------------------------------------------------*/

  void Logger::logMultiLine(const LogLevel & loglevel, const std::string & message)
  {
	// Tokenize
	// and log as single lines
	std::string::size_type lineStart = 0, lineEnd = 0;
	std::string delim = "\n";

	while((lineEnd = message.find(delim, lineStart)) != std::string::npos)
	{
	 log(loglevel, message.substr(lineStart, lineEnd - lineStart));
	 lineStart = lineEnd + 1;
	}

	log(loglevel, message.substr(lineStart, lineEnd - lineStart));
  }

 /*----------------------------------------------------------------------*/

}
