
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMNetClient.h"

namespace ParCompMark
{

  //
  // Constructors & destructor
  //

  NetClient::NetClient(const std::string & name):
	// Parent initializer 
  Client(name			/* Name of the class */
	)
  {
	getIP();
	mOwnIP = mIfConfs->get("eth0")->IP.c_str();
	initialize();
  }

 /*----------------------------------------------------------------------*/

  NetClient::~NetClient()
  {
	if(mInitialized)
	{
	 finalize();
	}
  }

 /*----------------------------------------------------------------------*/

  //

  //

  void NetClient::initialize()
  {

	Assert(!mInitialized, INVALID_OPERATION_ERROR, "NetClient::initialize()");

	//broadcast recieve initialize

	struct sockaddr_in addr;

	if((mBroadcastSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
	 Except(INTERNAL_ERROR, "Network::initBroadcastRecieve", "Unable to create socket");
	}

	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	addr.sin_port = htons(mBroadcastPort);
	memset(&(addr.sin_zero), '\0', 8);

	if(bind(mBroadcastSocket, (struct sockaddr *) &addr, sizeof(struct sockaddr)) < 0)
	{
	 Except(INTERNAL_ERROR, "NetClient::initialize()", "Unable to bind socket");
	}

	initThread(0, 0, true);

	mInitialized = true;

	Logger::getInstance()->log(Logger::NOTICE, "NetClient initialized.");
  }

 /*----------------------------------------------------------------------*/

  void NetClient::finalize()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "NetClient::finalize()");

	if(mBroadcastSocket != -1)
	{
	 close(mBroadcastSocket);
	 mBroadcastSocket = -1;
	}

	if(mStreamSocket != -1)
	{
	 close(mStreamSocket);
	 mStreamSocket = -1;
	}

	mInitialized = false;

  }

 /*----------------------------------------------------------------------*/

  void NetClient::task()
  {

	//recieve broadcast messages

	struct sockaddr_in addr;

	unsigned int addrLen = sizeof(struct sockaddr);

	int max = 100;

	char recvString[max + 1];

	int recvStringLen = -1;

	if((recvStringLen = recvfrom(mBroadcastSocket, recvString, max, 0, (struct sockaddr *) &addr, &addrLen)) < 0)
	{
	 Except(INTERNAL_ERROR, "NetClient::task()", "Unable to recieve broadcast message");
	}
	//server IP, we have to answer for this IP address

	mServerIP = inet_ntoa(addr.sin_addr);

	recvString[recvStringLen] = '\0';

	std::string message = recvString;

	int index = message.find("|");

	std::string type = message.substr(0, index);

	std::string m = message.substr(index + 1);
	m = m.substr(0, m.length());

	mType = type;
	mMessage = m;

	handleMessage();

  }

 /*----------------------------------------------------------------------*/

}
