
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMNetwork.h"

namespace ParCompMark
{

  //
  // Class variables
  //

  Container < Network::IfConf, DummyLock >::Pointer Network::mIfConfs;
  unsigned short Network::mCommunicationPort = 1234;
  unsigned short Network::mBroadcastPort = 1235;

  //
  // Constructors & destructor
  //

   Network::Network(const std::string & name):
	// Parent initializer 
   Thread(name		/* Name of the thread */
	)
	// You have to initialize the following attributes:
	// - mIfConfs
	// - mInitialized
	// - mCommunicationPort
	// - mBroadcastPort
	// - mStreamSocket
	// - mBroadcastSocket
	// - mOwnIP
  {

	mInitialized = false;
	mStreamSocket = -1;
	mBroadcastSocket = -1;
	mOwnIP = "";

	Logger::getInstance()->log(Logger::NOTICE, "Network created.");

  }

 /*----------------------------------------------------------------------*/

  Network::~Network()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Class methods
  //

  void Network::getIP()
  {

	if(mIfConfs.isNull())
	{

	 mIfConfs = new Container < IfConf, DummyLock >;

	} else
	{
	 mIfConfs.kill();
	 mIfConfs = new Container < IfConf, DummyLock >;
	}

	int n,
	sock,
	max;

	max = 4;
	int buffSize = max * sizeof(struct ifreq);
	struct ifreq *ifr;
	struct ifconf ifc;

	//TODO: ha nagyon sok akkor novelni kell!!!
	char *buff = new char[buffSize],
	data[16];

	bool enough = false;

	sock = socket(AF_INET, SOCK_STREAM, 0);

	unsigned char *d;

	do
	{

	 ifc.ifc_buf = buff;
	 ifc.ifc_len = buffSize;
	 ioctl(sock, SIOCGIFCONF, &ifc);
	 n = ifc.ifc_len / sizeof(struct ifreq);

	 if(n >= max)
	 {
		delete buff;

		max = max + 2;
		buffSize = max * sizeof(struct ifreq);
		buff = new char[buffSize];
	 } else
	 {
		enough = true;
	 }

	} while(!enough);

	for(ifr = ifc.ifc_req; n > 0; n--, ifr++)
	{
	 if(ifr->ifr_addr.sa_family == AF_INET)
	 {
		std::string name = ifr->ifr_name;
		IfConf::Pointer c(new IfConf);

		d = (unsigned char *) ifr->ifr_ifru.ifru_addr.sa_data + 2;
		snprintf(data, 16, "%i.%i.%i.%i", d[0], d[1], d[2], d[3]);
		c->IP = data;

		ioctl(sock, SIOCGIFBRDADDR, ifr);
		d = (unsigned char *) ifr->ifr_ifru.ifru_broadaddr.sa_data + 2;
		snprintf(data, 16, "%i.%i.%i.%i", d[0], d[1], d[2], d[3]);
		c->BroadcastIP = data;

		ioctl(sock, SIOCGIFNETMASK, ifr);
		d = (unsigned char *) ifr->ifr_ifru.ifru_broadaddr.sa_data + 2;
		snprintf(data, 16, "%i.%i.%i.%i", d[0], d[1], d[2], d[3]);
		c->Netmask = data;

		mIfConfs->add(name, c);

	 }
	}

	close(sock);

  }

 /*----------------------------------------------------------------------*/

  std::string Network::getHostName()
  {
	char name[32];

	gethostname(name, 32);

	std::string result;

	result = name;

	return result;
  }

 /*----------------------------------------------------------------------*/

}
