
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMOpenGLExtensionLoader.h"

//
// Outer includes
//

#include <GL/glew.h>
#include <GL/glx.h>
#include <GL/glu.h>

namespace ParCompMark
{

  //
  // Class variables
  //

  OpenGLExtensionLoader::LoadingMode OpenGLExtensionLoader::mLoadingMode = DELAYED;
  std::list < std::string > OpenGLExtensionLoader::mExtensionsToLoad;

  //
  // Class methods
  //

  void OpenGLExtensionLoader::load(const std::list < std::string > &extensions)
  {
	switch (mLoadingMode)
	{
	 case IMMEDIATE:
		_init();
		_load(extensions);
		break;
		case DELAYED:std::list < std::string > _extensions = extensions;
		mExtensionsToLoad.splice(mExtensionsToLoad.end(), _extensions);
		break;
	}
  }

 /*----------------------------------------------------------------------*/

  void OpenGLExtensionLoader::loadDelayed()
  {
	_init();
	_load(mExtensionsToLoad);
  }

 /*----------------------------------------------------------------------*/

  void OpenGLExtensionLoader::_load(const std::list < std::string > &extensions)
  {
	//loadOpenGLExtension should be called inside an OpenGL context 

	std::string currentExt;

	std::list < std::string > ext = extensions;

	for(; !ext.empty(); ext.pop_back())
	{

	 //only for safety reasons
	 currentExt = "";

	 currentExt = ext.back();

	 //check requested extensions
	 if(!glewIsSupported(currentExt.c_str()))
		Except(INTERNAL_ERROR, "OpenGLExtensionLoader::load()",
		   "OpenGL extension `" + currentExt + "\'is not supported.");

	}
  }

 /*----------------------------------------------------------------------*/

  void OpenGLExtensionLoader::_init()
  {
	//ensuring that all extensions with valid entry points will be exposed
	glewExperimental = GL_TRUE;

	// TODO: maybe one call is good
	GLenum err = glewInit();

	//check if everything is OK
	if(GLEW_OK != err)
	 Except(INTERNAL_ERROR, "OpenGLExtensionLoader::load()",
		 std::string("Error during OpenGL extentension loading: ") +
		 std::string((const char *) glewGetErrorString(err)));
  }

 /*----------------------------------------------------------------------*/

}
