
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMRenderer.h"

namespace ParCompMark
{

  //
  // Class constants
  //

  const s32 Renderer::NOAUTORENDERING = -1;

  //
  // Constructors & destructor
  //

   Renderer::Renderer()
	// You have to initialize the following attributes:
	// - mRendererHandle
	// - mRendererPlugin
	// - mParent
	// - mInitialized
	// - mAutoRenderOrder
  {
	Except(INVALID_OPERATION_ERROR, "Renderer::Renderer()",
	   "Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.");
  }

 /*----------------------------------------------------------------------*/

  Renderer::Renderer(RendererPlugin * rendererPlugin, void *rendererHandle, OpenGLRenderingEngine * parent):
	// Renderer instance initializer 
   mRendererHandle(rendererHandle	/* Pointer to the renderer instance in the plugin code. */
  ),
	// Renderer plugin initializer 
   mRendererPlugin(rendererPlugin	/* Corresponding renderer plugin. */
  ),
	// Rendering engine initializer 
   mParent(parent		/* Parent rendering engine of the renderer. */
	)
	// You have to initialize the following attributes:
	// - mRendererHandle
	// - mRendererPlugin
	// - mParent
	// - mInitialized
	// - mAutoRenderOrder
  {
	mInitialized = false;
	mAutoRenderOrder = Renderer::NOAUTORENDERING;

	Logger::getInstance()->log(Logger::NOTICE,
				 "A new custom renderer object has been created from renderer plugin `" +
				 mRendererPlugin->getName() + "'.");
  }

 /*----------------------------------------------------------------------*/

  Renderer::~Renderer()
  {
	if(mInitialized)
	 finalize();

	Logger::getInstance()->log(Logger::NOTICE,
				 "A renderer object from renderer plugin `" + mRendererPlugin->getName() +
				 "' has been destoryed .");
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  void Renderer::initialize()
  {
	Assert(!mInitialized, INVALID_OPERATION_ERROR, "Renderer::initialize()");

	mInitialized = true;
	Logger::getInstance()->log(Logger::DEBUG,
				 "A new custom renderer object has been initialized from renderer plugin `" +
				 mRendererPlugin->getName() + "'.");
  }

 /*----------------------------------------------------------------------*/

  void Renderer::finalize()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "Renderer::initialize()");

	mInitialized = false;
	Logger::getInstance()->log(Logger::DEBUG,
				 "A new custom renderer object has been finalized from renderer plugin `" +
				 mRendererPlugin->getName() + "'.");
  }

 /*----------------------------------------------------------------------*/

}
