
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMStringConverter.h"

namespace ParCompMark
{

  //
  // Class constants
  //

  const s32 StringConverter::DEFAULTPRECISION = -1;
  const s32 StringConverter::DEFAULTFIELDWIDTH = -1;

  //
  // Class methods
  //

   std::string StringConverter::toString(const u8 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const u16 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const u32 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const s8 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const s16 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const s32 & value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const Real & value, const s32 & fieldWidth,
					const s32 & precision)
  {
	std::ostringstream osstr;
	if(precision == DEFAULTPRECISION)
	 osstr << value;
	else
	{
	 osstr.precision(precision);
	 osstr << std::fixed << value;
	}
	return _fitFieldWidth(osstr.str(), fieldWidth);
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const bool value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::toString(const void *value)
  {
	std::ostringstream osstr;
	osstr << value;
	return osstr.str();
  }

 /*----------------------------------------------------------------------*/

  u32 StringConverter::toU32(const std::string value)
  {
	u32 result;

	std::istringstream isstr(value);

	isstr >> result;
	return result;
  }

 /*----------------------------------------------------------------------*/

  Real StringConverter::toReal(const std::string value)
  {
	Real result;

	std::istringstream isstr(value);

	isstr >> result;
	return result;
  }

 /*----------------------------------------------------------------------*/

  void *StringConverter::parsePointer(const std::string value)
  {
	std::istringstream isstr(value, std::istringstream::in);
	void *p;

	isstr >> p;
	return p;
  }

 /*----------------------------------------------------------------------*/

  void StringConverter::trim(std::string & str)
  {
	std::string::size_type pos = str.find_last_not_of(" \t");
	if(pos != std::string::npos)
	{
	 str.erase(pos + 1);
	 pos = str.find_first_not_of(" \t");
	 if(pos != std::string::npos)
		str.erase(0, pos);
	} else
	 str.erase(str.begin(), str.end());
  }

 /*----------------------------------------------------------------------*/

  std::vector < std::string > StringConverter::tokenize(const std::string & str, const char *delimiters)
  {
	std::vector < std::string > tokens;

	std::string::size_type lastPos = 0, pos = 0;
	int count = 0;

	if(str.length() < 1)
	 return tokens;

	// skip delimiters at beginning. 
	lastPos = str.find_first_not_of(delimiters, 0);

	if((str.substr(0, lastPos - pos).length()) > 0)
	{
	 count = str.substr(0, lastPos - pos).length();

	 for(int i = 0; i < count; i++)
		tokens.push_back("");

	 if(std::string::npos == lastPos)
		tokens.push_back("");
	}
	// find first "non-delimiter".
	pos = str.find_first_of(delimiters, lastPos);

	while(std::string::npos != pos || std::string::npos != lastPos)
	{
	 // found a token, add it to the vector.
	 tokens.push_back(str.substr(lastPos, pos - lastPos));

	 // skip delimiters. Note the "not_of"
	 lastPos = str.find_first_not_of(delimiters, pos);

	 if((std::string::npos != pos) && (str.substr(pos, lastPos - pos).length() > 1))
	 {
		count = str.substr(pos, lastPos - pos).length();

		for(int i = 0; i < count; i++)
		 tokens.push_back("");
	 }

	 pos = str.find_first_of(delimiters, lastPos);
	}

	return tokens;
  }

 /*----------------------------------------------------------------------*/

  std::string StringConverter::_fitFieldWidth(const std::string & str, const s32 & fieldWidth)
  {
	s32 padding = fieldWidth - str.size();

	if(fieldWidth == DEFAULTFIELDWIDTH || padding <= 0)
	 return str;
	char fitStr[padding + 1];

	memset(fitStr, ' ', padding);
	fitStr[padding] = '\0';

	return fitStr + str;
  }

 /*----------------------------------------------------------------------*/

}
