
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMTimer.h"

#include "../include/PCMStringConverter.h"

namespace ParCompMark
{

  //
  // Class constants
  //

  const Real Timer::EPSILONDELAY = 0.000001;

  //
  // Class variables
  //

  Real Timer::mTimeCorrection = 0.0;
  Real Timer::mStartTime = 0.0;

  //
  // Class methods
  //

   std::string Timer::getTimeString(const bool & microseconds, const bool & seconds,
				   const bool & minutes, const bool & hours,
				   const Real & correction)
  {
	return getTimeDateString(microseconds, seconds, minutes, hours, false, false, false, correction);
  }

 /*----------------------------------------------------------------------*/

  std::string Timer::getDateString(const bool & days, const bool & months, const bool & years,
				  const Real & correction)
  {
	return getTimeDateString(false, false, false, false, days, months, years, correction);
  }

 /*----------------------------------------------------------------------*/

  std::string Timer::getTimeDateString(const bool & microseconds, const bool & seconds,
					 const bool & minutes, const bool & hours, const bool & days,
					 const bool & months, const bool & years,
					 const Real & correction)
  {
	time_t tloc;
	struct tm result;

	// Get time in seconds
	time(&tloc);

	// Get time in microseconds
	Real tloc_ms = getSystemTime();

	// Correct time
	tloc += (time_t) correction;
	tloc_ms += correction - floor(correction);

	localtime_r(&tloc, &result);

	return (hours ? StringConverter::toString(result.tm_hour) : "") + (minutes ? ":" +
									 StringConverter::toString(result.
												  tm_min) : "") +
	 (seconds ? ":" + StringConverter::toString(result.tm_min) : "") + (microseconds ? "." +
									   StringConverter::
									   toString((tloc_ms -
											 tloc) * 1000000) : "") +
	 (years ? " " + StringConverter::toString(result.tm_year + 1900) : "") + (months ? "-" +
										  StringConverter::toString(result.
													   tm_mon +
													   1) : "")
	 + (days ? "-" + StringConverter::toString(result.tm_mday) : "");
  }

 /*----------------------------------------------------------------------*/

}
