
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestApplication.h
/// Test case declaration for class Application.

#ifndef __TEST_APPLICATION_H__
#  define __TEST_APPLICATION_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMApplication.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Application. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Application of project ParCompMark.
   */

  class TestApplication:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestApplication);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_complement_int_int);
	CPPUNIT_TEST(test_parseCommandLine_cu32_cchar_pp);
	CPPUNIT_TEST(test_showHelp_cstd__string);
	CPPUNIT_TEST(test_showVersion_cstd__string);
	CPPUNIT_TEST(test_setCommanderOn_cstd__string);
	CPPUNIT_TEST(test_setGUIOn_cstd__string);
	CPPUNIT_TEST(test_setLowLevelOn_cstd__string);
	CPPUNIT_TEST(test_setLowLevelScriptFile_cstd__string);
	CPPUNIT_TEST(test_setDynamicScriptFile_cstd__string);
	CPPUNIT_TEST(test_setScenarioScriptFile_cstd__string);
	CPPUNIT_TEST(test_setCluster_cstd__string);
	CPPUNIT_TEST(test_setParameters_cstd__string);
	CPPUNIT_TEST(test_setInput_cstd__string);
	CPPUNIT_TEST(test_setOutput_cstd__string);
	CPPUNIT_TEST(test_setExpectedHostCount_cstd__string);
	CPPUNIT_TEST(test_setUserInterface_cstd__string);
	CPPUNIT_TEST(test_hasEnvironmentVariable_cstd__string);
	CPPUNIT_TEST(test_getEnvironmentVariable_cstd__string);
	CPPUNIT_TEST(test_terminateHandler);
	CPPUNIT_TEST(test_terminateHandlerNOP);
	CPPUNIT_TEST(test_unexpectedHandler);
	CPPUNIT_TEST(test_segfaultHandler_int);
	CPPUNIT_TEST(test_interruptHandler_int);
	CPPUNIT_TEST(test_setupHandlers);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_finalizeCommander);
	CPPUNIT_TEST(test_finalizeSoldier);
	CPPUNIT_TEST(test_setFromConfigOptions);
	CPPUNIT_TEST(test_createVirtualMachines);
	CPPUNIT_TEST(test_startOperation);
	CPPUNIT_TEST(test_commanderOperation);
	CPPUNIT_TEST(test_soldierOperation);
	CPPUNIT_TEST(test_textUserInterface);
	CPPUNIT_TEST(test_noUserInterface);
	CPPUNIT_TEST(test_executeUserCommand_cstd__string);
	CPPUNIT_TEST(test_processSquirrelCommandList);
	CPPUNIT_TEST(test_doPostCommand);
	CPPUNIT_TEST(test_loadLowLevelScript_cchar_p);
	CPPUNIT_TEST(test_loadDynamicScript_cchar_p);
	CPPUNIT_TEST(test_loadScenarioScript_cchar_p);
	CPPUNIT_TEST(test_pushSquirrelCommand_cchar_p);
	CPPUNIT_TEST(test_createLowLevelScript_cstd__string);
	CPPUNIT_TEST(test_processScenarioScript);
	CPPUNIT_TEST(test_setEnvironmentVariablesToSquirrel);
	CPPUNIT_TEST(test_initializeDynamicScriptParameters_cbool);
	CPPUNIT_TEST(test_retrieveDynamicScriptParameters);
	CPPUNIT_TEST(test_writeOutput);
	CPPUNIT_TEST(test_waitForOutput);
	CPPUNIT_TEST(test_getInstance);
	CPPUNIT_TEST(test_quit);
	CPPUNIT_TEST(test_compileDynamic);
	CPPUNIT_TEST(test_start);
	CPPUNIT_TEST(test_stop);
	CPPUNIT_TEST(test_autoDetection);
	CPPUNIT_TEST(test_cleanup);
	CPPUNIT_TEST(test_getHostListForSquirrel);
	CPPUNIT_TEST(test_getLowLevelScriptForSquirrel);
	CPPUNIT_TEST(test_getDynamicScriptForSquirrel);
	CPPUNIT_TEST(test_hasDynamicScript);
	CPPUNIT_TEST(test_hasScenarioScript);
	CPPUNIT_TEST(test_getDynamicScriptParametersForSquirrel);
	CPPUNIT_TEST(test_getProgramInfo);
	CPPUNIT_TEST(test_calculateMessageSendTime);
	CPPUNIT_TEST(test__loadScenario);
	CPPUNIT_TEST(test__loadDynamic);
	CPPUNIT_TEST(test__compileDynamic);
	CPPUNIT_TEST(test__start);
	CPPUNIT_TEST(test__stop);
	CPPUNIT_TEST(test__pseudoStop);
	CPPUNIT_TEST(test__quit);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Application::Application().
	 */
	void test_constructor();

	/** 
	 Tests destructor Application::~Application().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method Application::complement(int first, int second).
	 */
	void test_complement_int_int();

	/** 
	 Tests method Application::parseCommandLine(const u32 &argc, const char ** &argv).
	 */
	void test_parseCommandLine_cu32_cchar_pp();

	/** 
	 Tests method Application::showHelp(const std::string &strarg).
	 */
	void test_showHelp_cstd__string();

	/** 
	 Tests method Application::showVersion(const std::string &strarg).
	 */
	void test_showVersion_cstd__string();

	/** 
	 Tests method Application::setCommanderOn(const std::string &strarg).
	 */
	void test_setCommanderOn_cstd__string();

	/** 
	 Tests method Application::setGUIOn(const std::string &strarg).
	 */
	void test_setGUIOn_cstd__string();

	/** 
	 Tests method Application::setLowLevelOn(const std::string &strarg).
	 */
	void test_setLowLevelOn_cstd__string();

	/** 
	 Tests method Application::setLowLevelScriptFile(const std::string &strarg).
	 */
	void test_setLowLevelScriptFile_cstd__string();

	/** 
	 Tests method Application::setDynamicScriptFile(const std::string &strarg).
	 */
	void test_setDynamicScriptFile_cstd__string();

	/** 
	 Tests method Application::setScenarioScriptFile(const std::string &strarg).
	 */
	void test_setScenarioScriptFile_cstd__string();

	/** 
	 Tests method Application::setCluster(const std::string &strarg).
	 */
	void test_setCluster_cstd__string();

	/** 
	 Tests method Application::setParameters(const std::string &strarg).
	 */
	void test_setParameters_cstd__string();

	/** 
	 Tests method Application::setInput(const std::string &strarg).
	 */
	void test_setInput_cstd__string();

	/** 
	 Tests method Application::setOutput(const std::string &strarg).
	 */
	void test_setOutput_cstd__string();

	/** 
	 Tests method Application::setExpectedHostCount(const std::string &hostCount).
	 */
	void test_setExpectedHostCount_cstd__string();

	/** 
	 Tests method Application::setUserInterface(const std::string &strUI).
	 */
	void test_setUserInterface_cstd__string();

	/** 
	 Tests method Application::hasEnvironmentVariable(const std::string &name).
	 */
	void test_hasEnvironmentVariable_cstd__string();

	/** 
	 Tests method Application::getEnvironmentVariable(const std::string &name).
	 */
	void test_getEnvironmentVariable_cstd__string();

	/** 
	 Tests method Application::terminateHandler().
	 */
	void test_terminateHandler();

	/** 
	 Tests method Application::terminateHandlerNOP().
	 */
	void test_terminateHandlerNOP();

	/** 
	 Tests method Application::unexpectedHandler().
	 */
	void test_unexpectedHandler();

	/** 
	 Tests method Application::segfaultHandler(int value).
	 */
	void test_segfaultHandler_int();

	/** 
	 Tests method Application::interruptHandler(int value).
	 */
	void test_interruptHandler_int();

	/** 
	 Tests method Application::getInstance().
	 */
	void test_getInstance();

	//
	// Method tests
	//

	/** 
	 Tests method Application::setupHandlers().
	 */
	void test_setupHandlers();

	/** 
	 Tests method Application::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Application::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Application::finalizeCommander().
	 */
	void test_finalizeCommander();

	/** 
	 Tests method Application::finalizeSoldier().
	 */
	void test_finalizeSoldier();

	/** 
	 Tests method Application::setFromConfigOptions().
	 */
	void test_setFromConfigOptions();

	/** 
	 Tests method Application::createVirtualMachines().
	 */
	void test_createVirtualMachines();

	/** 
	 Tests method Application::startOperation().
	 */
	void test_startOperation();

	/** 
	 Tests method Application::writeOutput().
	 */
	void test_writeOutput();

	/** 
	 Tests method Application::waitForOutput().
	 */
	void test_waitForOutput();

	/** 
	 Tests method Application::quit().
	 */
	void test_quit();

	/** 
	 Tests method Application::compileDynamic().
	 */
	void test_compileDynamic();

	/** 
	 Tests method Application::start().
	 */
	void test_start();

	/** 
	 Tests method Application::stop().
	 */
	void test_stop();

	/** 
	 Tests method Application::autoDetection().
	 */
	void test_autoDetection();

	/** 
	 Tests method Application::cleanup().
	 */
	void test_cleanup();

	/** 
	 Tests method Application::getHostListForSquirrel().
	 */
	void test_getHostListForSquirrel();

	/** 
	 Tests method Application::getLowLevelScriptForSquirrel().
	 */
	void test_getLowLevelScriptForSquirrel();

	/** 
	 Tests method Application::getDynamicScriptForSquirrel().
	 */
	void test_getDynamicScriptForSquirrel();

	/** 
	 Tests method Application::hasDynamicScript().
	 */
	void test_hasDynamicScript();

	/** 
	 Tests method Application::hasScenarioScript().
	 */
	void test_hasScenarioScript();

	/** 
	 Tests method Application::getDynamicScriptParametersForSquirrel().
	 */
	void test_getDynamicScriptParametersForSquirrel();

	/** 
	 Tests method Application::getProgramInfo().
	 */
	void test_getProgramInfo();

	/** 
	 Tests method Application::calculateMessageSendTime().
	 */
	void test_calculateMessageSendTime();

	/** 
	 Tests method Application::_pseudoStop().
	 */
	void test__pseudoStop();

	/** 
	 Tests method Application::commanderOperation().
	 */
	void test_commanderOperation();

	/** 
	 Tests method Application::soldierOperation().
	 */
	void test_soldierOperation();

	/** 
	 Tests method Application::textUserInterface().
	 */
	void test_textUserInterface();

	/** 
	 Tests method Application::noUserInterface().
	 */
	void test_noUserInterface();

	/** 
	 Tests method Application::executeUserCommand(const std::string &command).
	 */
	void test_executeUserCommand_cstd__string();

	/** 
	 Tests method Application::processSquirrelCommandList().
	 */
	void test_processSquirrelCommandList();

	/** 
	 Tests method Application::doPostCommand().
	 */
	void test_doPostCommand();

	/** 
	 Tests method Application::loadLowLevelScript(const char * filename).
	 */
	void test_loadLowLevelScript_cchar_p();

	/** 
	 Tests method Application::loadDynamicScript(const char * filename).
	 */
	void test_loadDynamicScript_cchar_p();

	/** 
	 Tests method Application::loadScenarioScript(const char * filename).
	 */
	void test_loadScenarioScript_cchar_p();

	/** 
	 Tests method Application::pushSquirrelCommand(const char * command).
	 */
	void test_pushSquirrelCommand_cchar_p();

	/** 
	 Tests method Application::createLowLevelScript(const std::string &clusterDescription).
	 */
	void test_createLowLevelScript_cstd__string();

	/** 
	 Tests method Application::processScenarioScript().
	 */
	void test_processScenarioScript();

	/** 
	 Tests method Application::setEnvironmentVariablesToSquirrel().
	 */
	void test_setEnvironmentVariablesToSquirrel();

	/** 
	 Tests method Application::initializeDynamicScriptParameters(const bool &reset).
	 */
	void test_initializeDynamicScriptParameters_cbool();

	/** 
	 Tests method Application::retrieveDynamicScriptParameters().
	 */
	void test_retrieveDynamicScriptParameters();

	/** 
	 Tests method Application::_loadScenario().
	 */
	void test__loadScenario();

	/** 
	 Tests method Application::_loadDynamic().
	 */
	void test__loadDynamic();

	/** 
	 Tests method Application::_compileDynamic().
	 */
	void test__compileDynamic();

	/** 
	 Tests method Application::_start().
	 */
	void test__start();

	/** 
	 Tests method Application::_stop().
	 */
	void test__stop();

	/** 
	 Tests method Application::_quit().
	 */
	void test__quit();

  };

}

#endif
