
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestConfigOptions.h
/// Test case declaration for class ConfigOptions.

#ifndef __TEST_CONFIGOPTIONS_H__
#  define __TEST_CONFIGOPTIONS_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMConfigOptions.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::ConfigOptions. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::ConfigOptions of project ParCompMark.
   */

  class TestConfigOptions:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestConfigOptions);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_hasOption_cstd__string);
	CPPUNIT_TEST(test_getString_cstd__string);
	CPPUNIT_TEST(test_getBool_cstd__string);
	CPPUNIT_TEST(test_getInteger_cstd__string);
	CPPUNIT_TEST(test_getReal_cstd__string);
	CPPUNIT_TEST(test_testString_cstd__string_cstd__string);
	CPPUNIT_TEST(test_setString_cstd__string_cstd__string_cstd__string);
	CPPUNIT_TEST(test_setBool_cstd__string_cbool_cstd__string);
	CPPUNIT_TEST(test_setInteger_cstd__string_cs32_cstd__string);
	CPPUNIT_TEST(test_setReal_cstd__string_cReal_cstd__string);
	CPPUNIT_TEST(test_removeOption_cstd__string);
	CPPUNIT_TEST(test_saveToIniFile_cstd__string);
	CPPUNIT_TEST(test_loadFromIniFile_cstd__string);
	CPPUNIT_TEST(test_getValue_cstd__string_cConfigOptions__OptionType);
	CPPUNIT_TEST(test_setValue_cstd__string_cvoid_p_cConfigOptions__OptionType_cstd__string);
	CPPUNIT_TEST(test_getOptionTypeSize_cConfigOptions__OptionType);
	CPPUNIT_TEST(test_optionToString_ConfigOptions__Option__Pointer);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor ConfigOptions::ConfigOptions().
	 */
	void test_constructor();

	/** 
	 Tests destructor ConfigOptions::~ConfigOptions().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method ConfigOptions::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method ConfigOptions::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method ConfigOptions::hasOption(const std::string &name).
	 */
	void test_hasOption_cstd__string();

	/** 
	 Tests method ConfigOptions::getString(const std::string &name).
	 */
	void test_getString_cstd__string();

	/** 
	 Tests method ConfigOptions::getBool(const std::string &name).
	 */
	void test_getBool_cstd__string();

	/** 
	 Tests method ConfigOptions::getInteger(const std::string &name).
	 */
	void test_getInteger_cstd__string();

	/** 
	 Tests method ConfigOptions::getReal(const std::string &name).
	 */
	void test_getReal_cstd__string();

	/** 
	 Tests method ConfigOptions::testString(const std::string &name, const std::string &value).
	 */
	void test_testString_cstd__string_cstd__string();

	/** 
	 Tests method ConfigOptions::setString(const std::string &name, const std::string &value, const std::string &description = ).
	 */
	void test_setString_cstd__string_cstd__string_cstd__string();

	/** 
	 Tests method ConfigOptions::setBool(const std::string &name, const bool &value, const std::string &description = ).
	 */
	void test_setBool_cstd__string_cbool_cstd__string();

	/** 
	 Tests method ConfigOptions::setInteger(const std::string &name, const s32 &value, const std::string &description = ).
	 */
	void test_setInteger_cstd__string_cs32_cstd__string();

	/** 
	 Tests method ConfigOptions::setReal(const std::string &name, const Real &value, const std::string &description = ).
	 */
	void test_setReal_cstd__string_cReal_cstd__string();

	/** 
	 Tests method ConfigOptions::removeOption(const std::string &name).
	 */
	void test_removeOption_cstd__string();

	/** 
	 Tests method ConfigOptions::saveToIniFile(const std::string &filename).
	 */
	void test_saveToIniFile_cstd__string();

	/** 
	 Tests method ConfigOptions::loadFromIniFile(const std::string &filename).
	 */
	void test_loadFromIniFile_cstd__string();

	/** 
	 Tests method ConfigOptions::getValue(const std::string &name, const ConfigOptions::OptionType &type).
	 */
	void test_getValue_cstd__string_cConfigOptions__OptionType();

	/** 
	 Tests method ConfigOptions::setValue(const std::string &name, const void * value, const ConfigOptions::OptionType &type, const std::string &description).
	 */
	void test_setValue_cstd__string_cvoid_p_cConfigOptions__OptionType_cstd__string();

	/** 
	 Tests method ConfigOptions::getOptionTypeSize(const ConfigOptions::OptionType &type).
	 */
	void test_getOptionTypeSize_cConfigOptions__OptionType();

	/** 
	 Tests method ConfigOptions::optionToString(ConfigOptions::Option::Pointer &option).
	 */
	void test_optionToString_ConfigOptions__Option__Pointer();

  };

}

#endif
