
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestContainer.h
/// Test case declaration for class Container.

#ifndef __TEST_CONTAINER_H__
#  define __TEST_CONTAINER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMContainer.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Container. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Container of project ParCompMark.
   */

  class TestContainer:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestContainer);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_add_std__string_ElementPointer);
	CPPUNIT_TEST(test_get_cstd__string);
	CPPUNIT_TEST(test_has_cstd__string);
	CPPUNIT_TEST(test_remove_cstd__string);
	CPPUNIT_TEST(test_getSize);
	CPPUNIT_TEST(test_isEmpty);
	CPPUNIT_TEST(test_begin);
	CPPUNIT_TEST(test_end);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Container::Container().
	 */
	void test_constructor();

	/** 
	 Tests destructor Container::~Container().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method Container::add(std::string name, ElementPointer element).
	 */
	void test_add_std__string_ElementPointer();

	/** 
	 Tests method Container::get(const std::string &name).
	 */
	void test_get_cstd__string();

	/** 
	 Tests method Container::has(const std::string &name).
	 */
	void test_has_cstd__string();

	/** 
	 Tests method Container::remove(const std::string &name).
	 */
	void test_remove_cstd__string();

	/** 
	 Tests method Container::getSize().
	 */
	void test_getSize();

	/** 
	 Tests method Container::isEmpty().
	 */
	void test_isEmpty();

	/** 
	 Tests method Container::begin().
	 */
	void test_begin();

	/** 
	 Tests method Container::end().
	 */
	void test_end();

  };

}

#endif
