
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestContext.h
/// Test case declaration for class Context.

#ifndef __TEST_CONTEXT_H__
#  define __TEST_CONTEXT_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMContext.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Context. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Context of project ParCompMark.
   */

  class TestContext:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestContext);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_Process_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_setProcesses_cchar_p);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_setContextTypeSq_cint);
	CPPUNIT_TEST(test_getContextTypeSq);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Context::Context().
	 */
	void test_constructor();

	/** 
	 Tests constructor Context::Context(Process * parent).
	 */
	void test_constructor_Process_p();

	/** 
	 Tests destructor Context::~Context().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method Context::setProcesses(const char * processList).
	 */
	void test_setProcesses_cchar_p();

	/** 
	 Tests method Context::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Context::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Context::setContextTypeSq(const int contextType).
	 */
	void test_setContextTypeSq_cint();

	/** 
	 Tests method Context::getContextTypeSq().
	 */
	void test_getContextTypeSq();

  };

}

#endif
