
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestFileSystemManager.h
/// Test case declaration for class FileSystemManager.

#ifndef __TEST_FILESYSTEMMANAGER_H__
#  define __TEST_FILESYSTEMMANAGER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMFileSystemManager.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::FileSystemManager. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::FileSystemManager of project ParCompMark.
   */

  class TestFileSystemManager:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestFileSystemManager);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_getInstance);
	CPPUNIT_TEST(test_existsDirectory_cstd__string);
	CPPUNIT_TEST(test_createDirectory_cstd__string);
	CPPUNIT_TEST(test_findDataDirectory);
	CPPUNIT_TEST(test_existsLibrary_cstd__string_cstd__vector___std__string__);
	CPPUNIT_TEST(test_findLibraryPath_cstd__string_cstd__vector___std__string__);
	CPPUNIT_TEST(test_listDirectory_cstd__string);
	CPPUNIT_TEST(test_listDataDirectory_cstd__string);
	CPPUNIT_TEST(test_existsFile_cstd__string);
	CPPUNIT_TEST(test_existsAppFile_cstd__string);
	CPPUNIT_TEST(test_existsDataFile_cchar_p);
	CPPUNIT_TEST(test_openFileC_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_openAppFileC_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_openDataFileC_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_openFileCpp_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_openAppFileCpp_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_openDataFileCpp_cstd__string_cFileOperation);
	CPPUNIT_TEST(test_getPathDataFile_cstd__string);
	CPPUNIT_TEST(test_readTextFile_cstd__string);
	CPPUNIT_TEST(test_readAppTextFile_cstd__string);
	CPPUNIT_TEST(test_readDataTextFile_cstd__string);
	CPPUNIT_TEST(test_closeFile_FileSystemManager__CppFilePointer);
	CPPUNIT_TEST(test_closeFile_FileSystemManager__CFilePointer);
	CPPUNIT_TEST(test__findHomeDirectory);
	CPPUNIT_TEST(test__replaceHomeChar_cstd__string);
	CPPUNIT_TEST(test__translateToAbsolutePath_cstd__string);
	CPPUNIT_TEST(test__createAppDirectory);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor FileSystemManager::FileSystemManager(const std::string &appDirectory = ~/.ParCompMark/, const std::string &iniFile = parcompmark.ini).
	 */
	void test_constructor_cstd__string_cstd__string();

	/** 
	 Tests destructor FileSystemManager::~FileSystemManager().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method FileSystemManager::getInstance().
	 */
	void test_getInstance();

	/** 
	 Tests method FileSystemManager::closeFile(FileSystemManager::CppFilePointer &fp).
	 */
	void test_closeFile_FileSystemManager__CppFilePointer();

	/** 
	 Tests method FileSystemManager::closeFile(FileSystemManager::CFilePointer &fp).
	 */
	void test_closeFile_FileSystemManager__CFilePointer();

	//
	// Method tests
	//

	/** 
	 Tests method FileSystemManager::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method FileSystemManager::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method FileSystemManager::existsDirectory(const std::string &name).
	 */
	void test_existsDirectory_cstd__string();

	/** 
	 Tests method FileSystemManager::createDirectory(const std::string &name).
	 */
	void test_createDirectory_cstd__string();

	/** 
	 Tests method FileSystemManager::findDataDirectory().
	 */
	void test_findDataDirectory();

	/** 
	 Tests method FileSystemManager::existsLibrary(const std::string &library, const std::vector < std::string > &additionalPaths = std::vector < std::string >()).
	 */
	void test_existsLibrary_cstd__string_cstd__vector___std__string__();

	/** 
	 Tests method FileSystemManager::findLibraryPath(const std::string &library, const std::vector < std::string > &additionalPaths = std::vector < std::string >()).
	 */
	void test_findLibraryPath_cstd__string_cstd__vector___std__string__();

	/** 
	 Tests method FileSystemManager::listDirectory(const std::string &directory).
	 */
	void test_listDirectory_cstd__string();

	/** 
	 Tests method FileSystemManager::listDataDirectory(const std::string &directory).
	 */
	void test_listDataDirectory_cstd__string();

	/** 
	 Tests method FileSystemManager::existsFile(const std::string &name).
	 */
	void test_existsFile_cstd__string();

	/** 
	 Tests method FileSystemManager::existsAppFile(const std::string &name).
	 */
	void test_existsAppFile_cstd__string();

	/** 
	 Tests method FileSystemManager::existsDataFile(const char * name).
	 */
	void test_existsDataFile_cchar_p();

	/** 
	 Tests method FileSystemManager::openFileC(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openFileC_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::openAppFileC(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openAppFileC_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::openDataFileC(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openDataFileC_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::openFileCpp(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openFileCpp_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::openAppFileCpp(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openAppFileCpp_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::openDataFileCpp(const std::string &name, const FileOperation &operation = FileSystemManager::READ).
	 */
	void test_openDataFileCpp_cstd__string_cFileOperation();

	/** 
	 Tests method FileSystemManager::getPathDataFile(const std::string &name).
	 */
	void test_getPathDataFile_cstd__string();

	/** 
	 Tests method FileSystemManager::readTextFile(const std::string &name).
	 */
	void test_readTextFile_cstd__string();

	/** 
	 Tests method FileSystemManager::readAppTextFile(const std::string &name).
	 */
	void test_readAppTextFile_cstd__string();

	/** 
	 Tests method FileSystemManager::readDataTextFile(const std::string &name).
	 */
	void test_readDataTextFile_cstd__string();

	/** 
	 Tests method FileSystemManager::_findHomeDirectory().
	 */
	void test__findHomeDirectory();

	/** 
	 Tests method FileSystemManager::_replaceHomeChar(const std::string &path).
	 */
	void test__replaceHomeChar_cstd__string();

	/** 
	 Tests method FileSystemManager::_translateToAbsolutePath(const std::string &path).
	 */
	void test__translateToAbsolutePath_cstd__string();

	/** 
	 Tests method FileSystemManager::_createAppDirectory().
	 */
	void test__createAppDirectory();

  };

}

#endif
