
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestGLXRenderWindow.h
/// Test case declaration for class GLXRenderWindow.

#ifndef __TEST_GLXRENDERWINDOW_H__
#  define __TEST_GLXRENDERWINDOW_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMGLXRenderWindow.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::GLXRenderWindow. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::GLXRenderWindow of project ParCompMark.
   */

  class TestGLXRenderWindow:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestGLXRenderWindow);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST
	 (test_constructor_XDisplay__Pointer_Process_p_cstd__string_cbool_cu32_cu32_cu32_cs32_cs32_cu32);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_reposition_cs32_cs32);
	CPPUNIT_TEST(test_resize_cu32_cu32);
	CPPUNIT_TEST(test_startFrame);
	CPPUNIT_TEST(test_finishFrame);
	CPPUNIT_TEST(test_setCurrent);
	CPPUNIT_TEST(test_releaseCurrent);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_createWindow);
	CPPUNIT_TEST(test_destroyWindow);
	CPPUNIT_TEST(test_resetStatistics);
	CPPUNIT_TEST(test_reportTriangles_cu32);
	CPPUNIT_TEST(test_updateStatistics);
	CPPUNIT_TEST(test__reposition);
	CPPUNIT_TEST(test__resize);
	CPPUNIT_TEST(test__setCaption);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor GLXRenderWindow::GLXRenderWindow(XDisplay::Pointer &display, Process * process, const std::string caption = PCM Framework, const bool &fullScreen = true, const u32 &colourDepth = 0, const u32 &width = GLXRenderWindow::MAXIMALSIZE, const u32 &height = GLXRenderWindow::MAXIMALSIZE, const s32 &left = GLXRenderWindow::CENTERED, const s32 &top = GLXRenderWindow::CENTERED, const u32 &fsaaSamples = 0).
	 */
	void test_constructor_XDisplay__Pointer_Process_p_cstd__string_cbool_cu32_cu32_cu32_cs32_cs32_cu32();

	/** 
	 Tests destructor GLXRenderWindow::~GLXRenderWindow().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method GLXRenderWindow::reposition(const s32 &left, const s32 &top).
	 */
	void test_reposition_cs32_cs32();

	/** 
	 Tests method GLXRenderWindow::resize(const u32 &width, const u32 &height).
	 */
	void test_resize_cu32_cu32();

	/** 
	 Tests method GLXRenderWindow::startFrame().
	 */
	void test_startFrame();

	/** 
	 Tests method GLXRenderWindow::finishFrame().
	 */
	void test_finishFrame();

	/** 
	 Tests method GLXRenderWindow::setCurrent().
	 */
	void test_setCurrent();

	/** 
	 Tests method GLXRenderWindow::releaseCurrent().
	 */
	void test_releaseCurrent();

	/** 
	 Tests method GLXRenderWindow::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method GLXRenderWindow::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method GLXRenderWindow::resetStatistics().
	 */
	void test_resetStatistics();

	/** 
	 Tests method GLXRenderWindow::reportTriangles(const u32 &triangleCount).
	 */
	void test_reportTriangles_cu32();

	/** 
	 Tests method GLXRenderWindow::createWindow().
	 */
	void test_createWindow();

	/** 
	 Tests method GLXRenderWindow::destroyWindow().
	 */
	void test_destroyWindow();

	/** 
	 Tests method GLXRenderWindow::updateStatistics().
	 */
	void test_updateStatistics();

	/** 
	 Tests method GLXRenderWindow::_reposition().
	 */
	void test__reposition();

	/** 
	 Tests method GLXRenderWindow::_resize().
	 */
	void test__resize();

	/** 
	 Tests method GLXRenderWindow::_setCaption().
	 */
	void test__setCaption();

  };

}

#endif
