
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestHost.h
/// Test case declaration for class Host.

#ifndef __TEST_HOST_H__
#  define __TEST_HOST_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMHost.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Host. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Host of project ParCompMark.
   */

  class TestHost:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestHost);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string_NetClient_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_getInstance);
	CPPUNIT_TEST(test_openXDisplay_cstd__string);
	CPPUNIT_TEST(test_getFirstXDisplay);
	CPPUNIT_TEST(test_openXDisplays);
	CPPUNIT_TEST(test_closeXDisplays);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_createNode_cchar_p);
	CPPUNIT_TEST(test_start);
	CPPUNIT_TEST(test_stop);
	CPPUNIT_TEST(test_collectData);
	CPPUNIT_TEST(test_setFrameID_cu32);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Host::Host(const std::string &name = (unknown host), NetClient * netClient = NULL).
	 */
	void test_constructor_cstd__string_NetClient_p();

	/** 
	 Tests destructor Host::~Host().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method Host::getInstance().
	 */
	void test_getInstance();

	//
	// Method tests
	//

	/** 
	 Tests method Host::openXDisplay(const std::string &displayName = ).
	 */
	void test_openXDisplay_cstd__string();

	/** 
	 Tests method Host::getFirstXDisplay().
	 */
	void test_getFirstXDisplay();

	/** 
	 Tests method Host::openXDisplays().
	 */
	void test_openXDisplays();

	/** 
	 Tests method Host::closeXDisplays().
	 */
	void test_closeXDisplays();

	/** 
	 Tests method Host::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Host::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Host::createNode(const char * nodeName).
	 */
	void test_createNode_cchar_p();

	/** 
	 Tests method Host::start().
	 */
	void test_start();

	/** 
	 Tests method Host::stop().
	 */
	void test_stop();

	/** 
	 Tests method Host::collectData().
	 */
	void test_collectData();

	/** 
	 Tests method Host::setFrameID(const u32 &frameID).
	 */
	void test_setFrameID_cu32();

  };

}

#endif
