
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestLogger.h
/// Test case declaration for class Logger.

#ifndef __TEST_LOGGER_H__
#  define __TEST_LOGGER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMLogger.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Logger. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Logger of project ParCompMark.
   */

  class TestLogger:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestLogger);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_log_cLogLevel_cstd__string);
	CPPUNIT_TEST(test_logMultiLine_cLogLevel_cstd__string);
	CPPUNIT_TEST(test_log_cException);
	CPPUNIT_TEST(test_translateLogLevel_cLogLevel);
	CPPUNIT_TEST(test_translateException_cException);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Logger::Logger(const std::string &logFileName = parcompmark.log).
	 */
	void test_constructor_cstd__string();

	/** 
	 Tests destructor Logger::~Logger().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method Logger::translateLogLevel(const LogLevel &loglevel).
	 */
	void test_translateLogLevel_cLogLevel();

	/** 
	 Tests method Logger::translateException(const Exception &exception).
	 */
	void test_translateException_cException();

	//
	// Method tests
	//

	/** 
	 Tests method Logger::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Logger::log(const LogLevel &loglevel, const std::string &message).
	 */
	void test_log_cLogLevel_cstd__string();

	/** 
	 Tests method Logger::logMultiLine(const LogLevel &loglevel, const std::string &message).
	 */
	void test_logMultiLine_cLogLevel_cstd__string();

	/** 
	 Tests method Logger::log(const Exception &exception).
	 */
	void test_log_cException();

  };

}

#endif
