
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestNode.h
/// Test case declaration for class Node.

#ifndef __TEST_NODE_H__
#  define __TEST_NODE_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMNode.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Node. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Node of project ParCompMark.
   */

  class TestNode:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestNode);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_cstd__string_Host_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_createProcess_cchar_p);
	CPPUNIT_TEST(test_createBuffer_cchar_p_cPCuint_cPCuint_cPCuint_cPCuint_cPCuint);
	CPPUNIT_TEST(test_getBuffer_cstd__string);
	CPPUNIT_TEST(test_start);
	CPPUNIT_TEST(test_stop);
	CPPUNIT_TEST(test_collectData);
	CPPUNIT_TEST(test_setFrameID_cu32);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Node::Node().
	 */
	void test_constructor();

	/** 
	 Tests constructor Node::Node(const std::string &name, Host * parent).
	 */
	void test_constructor_cstd__string_Host_p();

	/** 
	 Tests destructor Node::~Node().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method Node::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Node::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Node::createProcess(const char * processName).
	 */
	void test_createProcess_cchar_p();

	/** 
	 Tests method Node::createBuffer(const char * name, const PCuint left, const PCuint top, const PCuint width, const PCuint height, const PCuint depthFormat).
	 */
	void test_createBuffer_cchar_p_cPCuint_cPCuint_cPCuint_cPCuint_cPCuint();

	/** 
	 Tests method Node::getBuffer(const std::string name).
	 */
	void test_getBuffer_cstd__string();

	/** 
	 Tests method Node::start().
	 */
	void test_start();

	/** 
	 Tests method Node::stop().
	 */
	void test_stop();

	/** 
	 Tests method Node::collectData().
	 */
	void test_collectData();

	/** 
	 Tests method Node::setFrameID(const u32 &frameID).
	 */
	void test_setFrameID_cu32();

  };

}

#endif
