
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestOpenGLRenderingEngine.h
/// Test case declaration for class OpenGLRenderingEngine.

#ifndef __TEST_OPENGLRENDERINGENGINE_H__
#  define __TEST_OPENGLRENDERINGENGINE_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMOpenGLRenderingEngine.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::OpenGLRenderingEngine. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::OpenGLRenderingEngine of project ParCompMark.
   */

  class TestOpenGLRenderingEngine:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestOpenGLRenderingEngine);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_Process_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_create_cchar_p);
	CPPUNIT_TEST(test_createCustomRenderer_cchar_p);
	CPPUNIT_TEST(test__registerRenderer_Renderer_p);
	CPPUNIT_TEST(test_resizeRenderers_cu32_cu32);
	CPPUNIT_TEST(test_getSortOrderFromRenderers);
	CPPUNIT_TEST(test_setAutoRenderOrder_cRenderer_p_cs32);
	CPPUNIT_TEST(test_doAutoRendering);
	CPPUNIT_TEST(test_perspective_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_ortho2D_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_viewport_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_pushModelView);
	CPPUNIT_TEST(test_popModelView);
	CPPUNIT_TEST(test_translate_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_rotate_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_scale_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setCameraPosition_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setCameraTarget_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setCameraUpVector_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setColor_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setAmbientMaterial_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setDiffuseMaterial_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setSpecularMaterial_cdouble_cdouble_cdouble_cdouble_cint);
	CPPUNIT_TEST(test_setDrawStyle_cunsigned);
	CPPUNIT_TEST(test_setBackCulling_cbool);
	CPPUNIT_TEST(test_setBlending_cbool);
	CPPUNIT_TEST(test_drawTriangle);
	CPPUNIT_TEST(test_generateRandomTriangles_cint_cint);
	CPPUNIT_TEST(test__registerObject_ObjectData);
	CPPUNIT_TEST(test_renderObject_cu32_cbool);
	CPPUNIT_TEST(test_renderObjectTriangles_cu32_cbool_cu32);
	CPPUNIT_TEST(test__renderObject_cu32_cbool_cu32);
	CPPUNIT_TEST(test_setAmbientLight_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_removeLightSources);
	CPPUNIT_TEST(test_addLightSource);
	CPPUNIT_TEST(test_setLightSourcePosition_cint_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setLightSourceDiffuse_cint_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setLightSourceSpecular_cint_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_createDisplayList);
	CPPUNIT_TEST(test_finishDisplayList);
	CPPUNIT_TEST(test_executeDisplayList_cu32);
	CPPUNIT_TEST(test_drawSphere_cdouble_cint_cint);
	CPPUNIT_TEST(test_drawCylinder_cdouble_cdouble_cdouble_cint_cint);
	CPPUNIT_TEST(test_drawDisk_cdouble_cdouble_cint_cint);
	CPPUNIT_TEST(test_drawTeapot_cdouble);
	CPPUNIT_TEST(test_drawCube_cdouble);
	CPPUNIT_TEST(test_drawTorus_cdouble_cdouble_cint_cint);
	CPPUNIT_TEST(test_drawDodecahedron_cdouble);
	CPPUNIT_TEST(test_drawOctahedron_cdouble);
	CPPUNIT_TEST(test_drawTetrahedron_cdouble);
	CPPUNIT_TEST(test_drawIcosahedron_cdouble);
	CPPUNIT_TEST(test__refreshCamera);
	CPPUNIT_TEST(test__setGLUDrawStyle_cvoid_p);
	CPPUNIT_TEST(test__reportTriangles_cu32);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor OpenGLRenderingEngine::OpenGLRenderingEngine().
	 */
	void test_constructor();

	/** 
	 Tests constructor OpenGLRenderingEngine::OpenGLRenderingEngine(Process * parent).
	 */
	void test_constructor_Process_p();

	/** 
	 Tests destructor OpenGLRenderingEngine::~OpenGLRenderingEngine().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method OpenGLRenderingEngine::create(const char * stringPtr).
	 */
	void test_create_cchar_p();

	//
	// Method tests
	//

	/** 
	 Tests method OpenGLRenderingEngine::createCustomRenderer(const char * rendererName).
	 */
	void test_createCustomRenderer_cchar_p();

	/** 
	 Tests method OpenGLRenderingEngine::resizeRenderers(const u32 width, const u32 height).
	 */
	void test_resizeRenderers_cu32_cu32();

	/** 
	 Tests method OpenGLRenderingEngine::getSortOrderFromRenderers().
	 */
	void test_getSortOrderFromRenderers();

	/** 
	 Tests method OpenGLRenderingEngine::setAutoRenderOrder(const Renderer * renderer, const s32 &order).
	 */
	void test_setAutoRenderOrder_cRenderer_p_cs32();

	/** 
	 Tests method OpenGLRenderingEngine::doAutoRendering().
	 */
	void test_doAutoRendering();

	/** 
	 Tests method OpenGLRenderingEngine::perspective(const double fovy, const double zNear, const double zFar).
	 */
	void test_perspective_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::ortho2D(const double left, const double right, const double bottom, const double top).
	 */
	void test_ortho2D_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::viewport(const double left, const double top, const double width, const double height).
	 */
	void test_viewport_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::pushModelView().
	 */
	void test_pushModelView();

	/** 
	 Tests method OpenGLRenderingEngine::popModelView().
	 */
	void test_popModelView();

	/** 
	 Tests method OpenGLRenderingEngine::translate(const double x, const double y, const double z).
	 */
	void test_translate_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::rotate(const double angle, const double x, const double y, const double z).
	 */
	void test_rotate_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::scale(const double x, const double y, const double z).
	 */
	void test_scale_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setCameraPosition(const double eyeX, const double eyeY, const double eyeZ).
	 */
	void test_setCameraPosition_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setCameraTarget(const double centerX, const double centerY, const double centerZ).
	 */
	void test_setCameraTarget_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setCameraUpVector(const double upX, const double upY, const double upZ).
	 */
	void test_setCameraUpVector_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setColor(const double red, const double green, const double blue, const double alpha).
	 */
	void test_setColor_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setAmbientMaterial(const double red, const double green, const double blue, const double alpha).
	 */
	void test_setAmbientMaterial_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setDiffuseMaterial(const double red, const double green, const double blue, const double alpha).
	 */
	void test_setDiffuseMaterial_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setSpecularMaterial(const double red, const double green, const double blue, const double alpha, const int shininess).
	 */
	void test_setSpecularMaterial_cdouble_cdouble_cdouble_cdouble_cint();

	/** 
	 Tests method OpenGLRenderingEngine::setDrawStyle(const unsigned drawStyle).
	 */
	void test_setDrawStyle_cunsigned();

	/** 
	 Tests method OpenGLRenderingEngine::setBackCulling(const bool isBackCulling).
	 */
	void test_setBackCulling_cbool();

	/** 
	 Tests method OpenGLRenderingEngine::setBlending(const bool isOn).
	 */
	void test_setBlending_cbool();

	/** 
	 Tests method OpenGLRenderingEngine::drawTriangle().
	 */
	void test_drawTriangle();

	/** 
	 Tests method OpenGLRenderingEngine::generateRandomTriangles(const int dimension, const int count).
	 */
	void test_generateRandomTriangles_cint_cint();

	/** 
	 Tests method OpenGLRenderingEngine::renderObject(const u32 handle, const bool useVertexArrays).
	 */
	void test_renderObject_cu32_cbool();

	/** 
	 Tests method OpenGLRenderingEngine::renderObjectTriangles(const u32 handle, const bool useVertexArrays, const u32 triangleCount).
	 */
	void test_renderObjectTriangles_cu32_cbool_cu32();

	/** 
	 Tests method OpenGLRenderingEngine::setAmbientLight(const double red, const double green, const double blue, const double alpha).
	 */
	void test_setAmbientLight_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::removeLightSources().
	 */
	void test_removeLightSources();

	/** 
	 Tests method OpenGLRenderingEngine::addLightSource().
	 */
	void test_addLightSource();

	/** 
	 Tests method OpenGLRenderingEngine::setLightSourcePosition(const int light, const double x, const double y, const double z).
	 */
	void test_setLightSourcePosition_cint_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setLightSourceDiffuse(const int light, const double red, const double green, const double blue, const double alpha).
	 */
	void test_setLightSourceDiffuse_cint_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::setLightSourceSpecular(const int light, const double red, const double green, const double blue, const double alpha).
	 */
	void test_setLightSourceSpecular_cint_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::createDisplayList().
	 */
	void test_createDisplayList();

	/** 
	 Tests method OpenGLRenderingEngine::finishDisplayList().
	 */
	void test_finishDisplayList();

	/** 
	 Tests method OpenGLRenderingEngine::executeDisplayList(const u32 displayList).
	 */
	void test_executeDisplayList_cu32();

	/** 
	 Tests method OpenGLRenderingEngine::drawSphere(const double radius, const int slices, const int stacks).
	 */
	void test_drawSphere_cdouble_cint_cint();

	/** 
	 Tests method OpenGLRenderingEngine::drawCylinder(const double baseRadius, const double topRadius, const double height, const int slices, const int stacks).
	 */
	void test_drawCylinder_cdouble_cdouble_cdouble_cint_cint();

	/** 
	 Tests method OpenGLRenderingEngine::drawDisk(const double innerRadius, const double outerRadius, const int slices, const int loops).
	 */
	void test_drawDisk_cdouble_cdouble_cint_cint();

	/** 
	 Tests method OpenGLRenderingEngine::drawTeapot(const double size).
	 */
	void test_drawTeapot_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::drawCube(const double size).
	 */
	void test_drawCube_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::drawTorus(const double innerRadius, const double outerRadius, const int nsides, const int rings).
	 */
	void test_drawTorus_cdouble_cdouble_cint_cint();

	/** 
	 Tests method OpenGLRenderingEngine::drawDodecahedron(const double size).
	 */
	void test_drawDodecahedron_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::drawOctahedron(const double size).
	 */
	void test_drawOctahedron_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::drawTetrahedron(const double size).
	 */
	void test_drawTetrahedron_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::drawIcosahedron(const double size).
	 */
	void test_drawIcosahedron_cdouble();

	/** 
	 Tests method OpenGLRenderingEngine::_registerRenderer(Renderer * renderer).
	 */
	void test__registerRenderer_Renderer_p();

	/** 
	 Tests method OpenGLRenderingEngine::_registerObject(ObjectData objectData).
	 */
	void test__registerObject_ObjectData();

	/** 
	 Tests method OpenGLRenderingEngine::_renderObject(const u32 handle, const bool useVertexArrays, const u32 vertexCount).
	 */
	void test__renderObject_cu32_cbool_cu32();

	/** 
	 Tests method OpenGLRenderingEngine::_refreshCamera().
	 */
	void test__refreshCamera();

	/** 
	 Tests method OpenGLRenderingEngine::_setGLUDrawStyle(const void * quadratic).
	 */
	void test__setGLUDrawStyle_cvoid_p();

	/** 
	 Tests method OpenGLRenderingEngine::_reportTriangles(const u32 &triangleCount).
	 */
	void test__reportTriangles_cu32();

  };

}

#endif
