
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestOutputNode.h
/// Test case declaration for class OutputNode.

#ifndef __TEST_OUTPUTNODE_H__
#  define __TEST_OUTPUTNODE_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMOutputNode.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::OutputNode. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::OutputNode of project ParCompMark.
   */

  class TestOutputNode:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestOutputNode);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string_cNodeType);
	CPPUNIT_TEST(test_constructor_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_createChildNode_cstd__string_cNodeType);
	CPPUNIT_TEST(test_createChildNode_cstd__string);
	CPPUNIT_TEST(test_addChildNode_OutputNode__Pointer);
	CPPUNIT_TEST(test_getChildCount);
	CPPUNIT_TEST(test_getFirstChildNode);
	CPPUNIT_TEST(test_hasAttribute_cstd__string);
	CPPUNIT_TEST(test_getAttribute_cstd__string);
	CPPUNIT_TEST(test_setAttribute_cstd__string_cstd__string);
	CPPUNIT_TEST(test_serialize2XML_std__ostringstream);
	CPPUNIT_TEST(test_serialize2XML);
	CPPUNIT_TEST(test_parseFromXML_std__string);
	CPPUNIT_TEST(test__testXMLName_cstd__string);
	CPPUNIT_TEST(test__convertSpecialChars_cstd__string);
	CPPUNIT_TEST(test_refreshData);
	CPPUNIT_TEST(test_clean);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor OutputNode::OutputNode(const std::string &name, const NodeType &type).
	 */
	void test_constructor_cstd__string_cNodeType();

	/** 
	 Tests constructor OutputNode::OutputNode(const std::string &text).
	 */
	void test_constructor_cstd__string();

	/** 
	 Tests destructor OutputNode::~OutputNode().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method OutputNode::parseFromXML(std::string &xmlDocument).
	 */
	void test_parseFromXML_std__string();

	/** 
	 Tests method OutputNode::_testXMLName(const std::string &name).
	 */
	void test__testXMLName_cstd__string();

	/** 
	 Tests method OutputNode::_convertSpecialChars(const std::string &string).
	 */
	void test__convertSpecialChars_cstd__string();

	//
	// Method tests
	//

	/** 
	 Tests method OutputNode::createChildNode(const std::string &name, const NodeType &type).
	 */
	void test_createChildNode_cstd__string_cNodeType();

	/** 
	 Tests method OutputNode::createChildNode(const std::string &text).
	 */
	void test_createChildNode_cstd__string();

	/** 
	 Tests method OutputNode::addChildNode(OutputNode::Pointer child).
	 */
	void test_addChildNode_OutputNode__Pointer();

	/** 
	 Tests method OutputNode::getChildCount().
	 */
	void test_getChildCount();

	/** 
	 Tests method OutputNode::getFirstChildNode().
	 */
	void test_getFirstChildNode();

	/** 
	 Tests method OutputNode::hasAttribute(const std::string &attribute).
	 */
	void test_hasAttribute_cstd__string();

	/** 
	 Tests method OutputNode::getAttribute(const std::string &attribute).
	 */
	void test_getAttribute_cstd__string();

	/** 
	 Tests method OutputNode::setAttribute(const std::string &attribute, const std::string &value).
	 */
	void test_setAttribute_cstd__string_cstd__string();

	/** 
	 Tests method OutputNode::serialize2XML(std::ostringstream &osstr).
	 */
	void test_serialize2XML_std__ostringstream();

	/** 
	 Tests method OutputNode::serialize2XML().
	 */
	void test_serialize2XML();

	/** 
	 Tests method OutputNode::refreshData().
	 */
	void test_refreshData();

	/** 
	 Tests method OutputNode::clean().
	 */
	void test_clean();

  };

}

#endif
