
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestPCWrapper.h
/// Test case declaration for class PCWrapper.

#ifndef __TEST_PCWRAPPER_H__
#  define __TEST_PCWRAPPER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMPCWrapper.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::PCWrapper. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::PCWrapper of project ParCompMark.
   */

  class TestPCWrapper:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestPCWrapper);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_contextCreate_PCid_PCstring_PCcontextp);
	CPPUNIT_TEST(test_contextCreateMaster_cPCintp_PCstringp_PCstring_PCcontextp);
	CPPUNIT_TEST(test_contextDestroy_PCcontext);
	CPPUNIT_TEST(test_contextGetInteger_PCcontext_PCenum_PCint_PCintp);
	CPPUNIT_TEST(test_contextSetInteger_PCcontext_PCenum_PCint_PCint);
	CPPUNIT_TEST(test_contextSync_PCcontext);
	CPPUNIT_TEST(test_frameAddFramelet_PCcontext_PCid_PCvoidp_PCvoidp);
	CPPUNIT_TEST(test_frameAddGLFrameletEXT_PCcontext_PCid_PCint_PCint);
	CPPUNIT_TEST(test_frameBegin_PCcontext_PCidp_PCuint_PCuintp_PCuintp_PCuintp_PCuintp_PCuint);
	CPPUNIT_TEST(test_frameEnd_cPCcontext_cPCid);
	CPPUNIT_TEST(test_frameResultChannel_PCcontext_PCid_PCuint_PCuint_PCuint_PCuint_PCuint_PCchannel);
	CPPUNIT_TEST(test_getErrorString_PCerr);
	CPPUNIT_TEST(test_queryExtension_PCstring);
	CPPUNIT_TEST(test_sessionCreate_PCid);
	CPPUNIT_TEST(test_sessionDestroy);
	CPPUNIT_TEST(test_systemGetInteger_PCenum_PCint_PCintp);
	CPPUNIT_TEST(test_systemFinalize);
	CPPUNIT_TEST(test_systemGetString_PCenum_PCint_PCstringp);
	CPPUNIT_TEST(test_systemInitialize_cPCstring);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Class method tests
	//

	/** 
	 Tests method PCWrapper::contextCreate(PCid netId, PCstring thisHost, PCcontext* ctx).
	 */
	void test_contextCreate_PCid_PCstring_PCcontextp();

	/** 
	 Tests method PCWrapper::contextCreateMaster(const PCint* properties, PCstring* hosts, PCstring thisHost, PCcontext* ctx).
	 */
	void test_contextCreateMaster_cPCintp_PCstringp_PCstring_PCcontextp();

	/** 
	 Tests method PCWrapper::contextDestroy(PCcontext ctx).
	 */
	void test_contextDestroy_PCcontext();

	/** 
	 Tests method PCWrapper::contextGetInteger(PCcontext ctx, PCenum property, PCint hostIndex, PCint* value).
	 */
	void test_contextGetInteger_PCcontext_PCenum_PCint_PCintp();

	/** 
	 Tests method PCWrapper::contextSetInteger(PCcontext ctx, PCenum property, PCint hostIndex, PCint value).
	 */
	void test_contextSetInteger_PCcontext_PCenum_PCint_PCint();

	/** 
	 Tests method PCWrapper::contextSync(PCcontext ctx).
	 */
	void test_contextSync_PCcontext();

	/** 
	 Tests method PCWrapper::frameAddFramelet(PCcontext ctx, PCid frameId, PCvoidp color, PCvoidp depth).
	 */
	void test_frameAddFramelet_PCcontext_PCid_PCvoidp_PCvoidp();

	/** 
	 Tests method PCWrapper::frameAddGLFrameletEXT(PCcontext ctx, PCid frameId, PCint xOffset, PCint yOffset).
	 */
	void test_frameAddGLFrameletEXT_PCcontext_PCid_PCint_PCint();

	/** 
	 Tests method PCWrapper::frameBegin(PCcontext ctx, PCid* frameId, PCuint numFramelets, PCuint* xOffset, PCuint* yOffset, PCuint* width, PCuint* height, PCuint sortorder).
	 */
	void test_frameBegin_PCcontext_PCidp_PCuint_PCuintp_PCuintp_PCuintp_PCuintp_PCuint();

	/** 
	 Tests method PCWrapper::frameEnd(const PCcontext ctx, const PCid frameId).
	 */
	void test_frameEnd_cPCcontext_cPCid();

	/** 
	 Tests method PCWrapper::frameResultChannel(PCcontext ctx, PCid frameId, PCuint xOffset, PCuint yOffset, PCuint width, PCuint height, PCuint channelType, PCchannel channel).
	 */
	void test_frameResultChannel_PCcontext_PCid_PCuint_PCuint_PCuint_PCuint_PCuint_PCchannel();

	/** 
	 Tests method PCWrapper::getErrorString(PCerr error).
	 */
	void test_getErrorString_PCerr();

	/** 
	 Tests method PCWrapper::queryExtension(PCstring extensionName).
	 */
	void test_queryExtension_PCstring();

	/** 
	 Tests method PCWrapper::sessionCreate(PCid sessionId).
	 */
	void test_sessionCreate_PCid();

	/** 
	 Tests method PCWrapper::sessionDestroy().
	 */
	void test_sessionDestroy();

	/** 
	 Tests method PCWrapper::systemGetInteger(PCenum property, PCint qualifier, PCint* value).
	 */
	void test_systemGetInteger_PCenum_PCint_PCintp();

	/** 
	 Tests method PCWrapper::systemFinalize().
	 */
	void test_systemFinalize();

	/** 
	 Tests method PCWrapper::systemGetString(PCenum property, PCint qualifier, PCstring* value).
	 */
	void test_systemGetString_PCenum_PCint_PCstringp();

	/** 
	 Tests method PCWrapper::systemInitialize(const PCstring searchPath).
	 */
	void test_systemInitialize_cPCstring();

  };

}

#endif
