
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestPlugin.h
/// Test case declaration for class Plugin.

#ifndef __TEST_PLUGIN_H__
#  define __TEST_PLUGIN_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMPlugin.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Plugin. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Plugin of project ParCompMark.
   */

  class TestPlugin:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestPlugin);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cPluginType_cstd__string_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test__getNeededLibs);
	CPPUNIT_TEST(test__setPluginHandle);
	CPPUNIT_TEST(test__setLoggerFunction);
	CPPUNIT_TEST(test__loggerFunction_cvoid_p_cchar_p);
	CPPUNIT_TEST(test__onLoad);
	CPPUNIT_TEST(test__onUnload);
	CPPUNIT_TEST(test__getErrorMsg);
	CPPUNIT_TEST(test__checkError);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Plugin::Plugin(const PluginType &type, const std::string &name, const std::string &filename).
	 */
	void test_constructor_cPluginType_cstd__string_cstd__string();

	/** 
	 Tests destructor Plugin::~Plugin().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method Plugin::_loggerFunction(const void * handle, const char * message).
	 */
	void test__loggerFunction_cvoid_p_cchar_p();

	//
	// Method tests
	//

	/** 
	 Tests method Plugin::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Plugin::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Plugin::_getNeededLibs().
	 */
	void test__getNeededLibs();

	/** 
	 Tests method Plugin::_setPluginHandle().
	 */
	void test__setPluginHandle();

	/** 
	 Tests method Plugin::_setLoggerFunction().
	 */
	void test__setLoggerFunction();

	/** 
	 Tests method Plugin::_onLoad().
	 */
	void test__onLoad();

	/** 
	 Tests method Plugin::_onUnload().
	 */
	void test__onUnload();

	/** 
	 Tests method Plugin::_getErrorMsg().
	 */
	void test__getErrorMsg();

	/** 
	 Tests method Plugin::_checkError().
	 */
	void test__checkError();

  };

}

#endif
