
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestPluginManager.h
/// Test case declaration for class PluginManager.

#ifndef __TEST_PLUGINMANAGER_H__
#  define __TEST_PLUGINMANAGER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMPluginManager.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::PluginManager. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::PluginManager of project ParCompMark.
   */

  class TestPluginManager:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestPluginManager);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_loadPlugins);
	CPPUNIT_TEST(test_loadPlugins_old);
	CPPUNIT_TEST(test_unloadPlugins);
	CPPUNIT_TEST(test_getPlugin_cstd__string);
	CPPUNIT_TEST(test__loadPlugin_cPlugin__PluginType_cstd__string);
	CPPUNIT_TEST(test__checkNeededLibs_Plugin__Pointer);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor PluginManager::PluginManager(const std::string &pluginDirectory = plugins/, const std::string &iniFile = plugins.ini).
	 */
	void test_constructor_cstd__string_cstd__string();

	/** 
	 Tests destructor PluginManager::~PluginManager().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method PluginManager::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method PluginManager::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method PluginManager::loadPlugins().
	 */
	void test_loadPlugins();

	/** 
	 Tests method PluginManager::loadPlugins_old().
	 */
	void test_loadPlugins_old();

	/** 
	 Tests method PluginManager::unloadPlugins().
	 */
	void test_unloadPlugins();

	/** 
	 Tests method PluginManager::getPlugin(const std::string &name).
	 */
	void test_getPlugin_cstd__string();

	/** 
	 Tests method PluginManager::_loadPlugin(const Plugin::PluginType &type, const std::string &filename).
	 */
	void test__loadPlugin_cPlugin__PluginType_cstd__string();

	/** 
	 Tests method PluginManager::_checkNeededLibs(Plugin::Pointer &plugin).
	 */
	void test__checkNeededLibs_Plugin__Pointer();

  };

}

#endif
