
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestPointer.h
/// Test case declaration for class Pointer.

#ifndef __TEST_POINTER_H__
#  define __TEST_POINTER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMPointer.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Pointer. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Pointer of project ParCompMark.
   */

  class TestPointer:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestPointer);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_cT_p_cbool);
	CPPUNIT_TEST(test_constructor_Pointer___T__Lock__);
	CPPUNIT_TEST(test_constructor_cPointer___T__Lock__);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_operator_assignment_cT_p);
	CPPUNIT_TEST(test_operator_assignment_Pointer___T__Lock__);
	CPPUNIT_TEST(test_operator_assignment_cPointer___T__Lock__);
	CPPUNIT_TEST(test_operator_memberSelection);
	CPPUNIT_TEST(test_operator_equality_cT_p);
	CPPUNIT_TEST(test_operator_equality_Pointer___T__Lock__);
	CPPUNIT_TEST(test_operator_inequality_cT_p);
	CPPUNIT_TEST(test_operator_inequality_Pointer___T__Lock__);
	CPPUNIT_TEST(test_isNull);
	CPPUNIT_TEST(test_isNotNull);
	CPPUNIT_TEST(test_assignWithLock_Pointer___T__Lock__);
	CPPUNIT_TEST(test_reference_cT_p);
	CPPUNIT_TEST(test_getPtr);
	CPPUNIT_TEST(test_kill_cbool);
	CPPUNIT_TEST(test_setNull_cbool);
	CPPUNIT_TEST(test_lock);
	CPPUNIT_TEST(test_trylock);
	CPPUNIT_TEST(test_unlock);
	CPPUNIT_TEST(test_getLocked);
	CPPUNIT_TEST(test__deletePointer_cbool_cbool);
	CPPUNIT_TEST(test__assignCPointer_cT_p_cbool);
	CPPUNIT_TEST(test__assignPointer_Pointer___T__Lock___cbool);
	CPPUNIT_TEST(test__switchPointer_Pointer___T__Lock___cbool);
	CPPUNIT_TEST(test__equalsCPointer_cT_p);
	CPPUNIT_TEST(test__equalsPointer_Pointer___T__Lock__);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Pointer::Pointer().
	 */
	void test_constructor();

	/** 
	 Tests constructor Pointer::Pointer(const T * pointer, const bool &takeOwnership = true).
	 */
	void test_constructor_cT_p_cbool();

	/** 
	 Tests constructor Pointer::Pointer(Pointer < T, Lock > &pointer).
	 */
	void test_constructor_Pointer___T__Lock__();

	/** 
	 Tests constructor Pointer::Pointer(const Pointer < T, Lock > &pointer).
	 */
	void test_constructor_cPointer___T__Lock__();

	/** 
	 Tests destructor Pointer::~Pointer().
	 */
	void test_destructor();

	//
	// Operator tests
	//

	/** 
	 Tests operator Pointer::operator=(const T * pointer).
	 */
	void test_operator_assignment_cT_p();

	/** 
	 Tests operator Pointer::operator=(Pointer < T, Lock > &pointer).
	 */
	void test_operator_assignment_Pointer___T__Lock__();

	/** 
	 Tests operator Pointer::operator=(const Pointer < T, Lock > &pointer).
	 */
	void test_operator_assignment_cPointer___T__Lock__();

	/** 
	 Tests operator Pointer::operator->().
	 */
	void test_operator_memberSelection();

	/** 
	 Tests operator Pointer::operator==(const T * pointer).
	 */
	void test_operator_equality_cT_p();

	/** 
	 Tests operator Pointer::operator==(Pointer < T, Lock > &pointer).
	 */
	void test_operator_equality_Pointer___T__Lock__();

	/** 
	 Tests operator Pointer::operator!=(const T * pointer).
	 */
	void test_operator_inequality_cT_p();

	/** 
	 Tests operator Pointer::operator!=(Pointer < T, Lock > &pointer).
	 */
	void test_operator_inequality_Pointer___T__Lock__();

	//
	// Method tests
	//

	/** 
	 Tests method Pointer::isNull().
	 */
	void test_isNull();

	/** 
	 Tests method Pointer::isNotNull().
	 */
	void test_isNotNull();

	/** 
	 Tests method Pointer::assignWithLock(Pointer < T, Lock > &pointer).
	 */
	void test_assignWithLock_Pointer___T__Lock__();

	/** 
	 Tests method Pointer::reference(const T * pointer).
	 */
	void test_reference_cT_p();

	/** 
	 Tests method Pointer::getPtr().
	 */
	void test_getPtr();

	/** 
	 Tests method Pointer::kill(const bool &force = false).
	 */
	void test_kill_cbool();

	/** 
	 Tests method Pointer::setNull(const bool &force = false).
	 */
	void test_setNull_cbool();

	/** 
	 Tests method Pointer::lock().
	 */
	void test_lock();

	/** 
	 Tests method Pointer::trylock().
	 */
	void test_trylock();

	/** 
	 Tests method Pointer::unlock().
	 */
	void test_unlock();

	/** 
	 Tests method Pointer::getLocked().
	 */
	void test_getLocked();

	/** 
	 Tests method Pointer::_deletePointer(const bool &force = false, const bool &keepLock = false).
	 */
	void test__deletePointer_cbool_cbool();

	/** 
	 Tests method Pointer::_assignCPointer(const T * pointer, const bool &takeOwnership = true).
	 */
	void test__assignCPointer_cT_p_cbool();

	/** 
	 Tests method Pointer::_assignPointer(Pointer < T, Lock > &pointer, const bool &keepLock = false).
	 */
	void test__assignPointer_Pointer___T__Lock___cbool();

	/** 
	 Tests method Pointer::_switchPointer(Pointer < T, Lock > &pointer, const bool &keepLock = false).
	 */
	void test__switchPointer_Pointer___T__Lock___cbool();

	/** 
	 Tests method Pointer::_equalsCPointer(const T * pointer).
	 */
	void test__equalsCPointer_cT_p();

	/** 
	 Tests method Pointer::_equalsPointer(Pointer < T, Lock > &pointer).
	 */
	void test__equalsPointer_Pointer___T__Lock__();

  };

}

#endif
