
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestProcess.h
/// Test case declaration for class Process.

#ifndef __TEST_PROCESS_H__
#  define __TEST_PROCESS_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMProcess.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Process. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Process of project ParCompMark.
   */

  class TestProcess:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestProcess);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_cstd__string_Node_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_task);
	CPPUNIT_TEST(test_threadInitialize);
	CPPUNIT_TEST(test_threadFinalize);
	CPPUNIT_TEST(test_start);
	CPPUNIT_TEST(test_stop);
	CPPUNIT_TEST(test_addContext);
	CPPUNIT_TEST(test_initPC);
	CPPUNIT_TEST(test_openRenderWindow_cstd__string_cstd__string_cbool_cu32_cu32_cu32_cs32_cs32_cu32);
	CPPUNIT_TEST(test_actualizeRenderWindow);
	CPPUNIT_TEST(test_setViewportForRendering);
	CPPUNIT_TEST(test_displayFrameletIcon);
	CPPUNIT_TEST(test_refreshSortOrder);
	CPPUNIT_TEST(test_gatherStatistics);
	CPPUNIT_TEST(test_initProcess);
	CPPUNIT_TEST(test_runningProcess);
	CPPUNIT_TEST(test_setProcessTypeSq_cint);
	CPPUNIT_TEST(test_getProcessTypeSq);
	CPPUNIT_TEST(test_setBufferByName_cchar_p);
	CPPUNIT_TEST(test_setFinito_cu32);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Process::Process().
	 */
	void test_constructor();

	/** 
	 Tests constructor Process::Process(const std::string &name, Node * parent).
	 */
	void test_constructor_cstd__string_Node_p();

	/** 
	 Tests destructor Process::~Process().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method Process::threadInitialize().
	 */
	void test_threadInitialize();

	/** 
	 Tests method Process::threadFinalize().
	 */
	void test_threadFinalize();

	/** 
	 Tests method Process::start().
	 */
	void test_start();

	/** 
	 Tests method Process::stop().
	 */
	void test_stop();

	/** 
	 Tests method Process::addContext().
	 */
	void test_addContext();

	/** 
	 Tests method Process::initPC().
	 */
	void test_initPC();

	/** 
	 Tests method Process::openRenderWindow(const std::string caption = PCM Framework, const std::string &displayName = , const bool &fullScreen = true, const u32 &colourDepth = 0, const u32 &width = GLXRenderWindow::MAXIMALSIZE, const u32 &height = GLXRenderWindow::MAXIMALSIZE, const s32 &left = GLXRenderWindow::CENTERED, const s32 &top = GLXRenderWindow::CENTERED, const u32 &fsaaSamples = 0).
	 */
	void test_openRenderWindow_cstd__string_cstd__string_cbool_cu32_cu32_cu32_cs32_cs32_cu32();

	/** 
	 Tests method Process::actualizeRenderWindow().
	 */
	void test_actualizeRenderWindow();

	/** 
	 Tests method Process::setViewportForRendering().
	 */
	void test_setViewportForRendering();

	/** 
	 Tests method Process::displayFrameletIcon().
	 */
	void test_displayFrameletIcon();

	/** 
	 Tests method Process::initProcess().
	 */
	void test_initProcess();

	/** 
	 Tests method Process::runningProcess().
	 */
	void test_runningProcess();

	/** 
	 Tests method Process::setProcessTypeSq(const int processType).
	 */
	void test_setProcessTypeSq_cint();

	/** 
	 Tests method Process::getProcessTypeSq().
	 */
	void test_getProcessTypeSq();

	/** 
	 Tests method Process::setBufferByName(const char * name).
	 */
	void test_setBufferByName_cchar_p();

	/** 
	 Tests method Process::setFinito(const u32 &frameID).
	 */
	void test_setFinito_cu32();

	/** 
	 Tests method Process::task().
	 */
	void test_task();

	/** 
	 Tests method Process::refreshSortOrder().
	 */
	void test_refreshSortOrder();

	/** 
	 Tests method Process::gatherStatistics().
	 */
	void test_gatherStatistics();

  };

}

#endif
