
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestRenderer.h
/// Test case declaration for class Renderer.

#ifndef __TEST_RENDERER_H__
#  define __TEST_RENDERER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMRenderer.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Renderer. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Renderer of project ParCompMark.
   */

  class TestRenderer:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestRenderer);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor);
	CPPUNIT_TEST(test_constructor_RendererPlugin_p_void_p_OpenGLRenderingEngine_p);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_resize_cu32_cu32);
	CPPUNIT_TEST(test_getSortOrder);
	CPPUNIT_TEST(test_render);
	CPPUNIT_TEST(test_setMiscParam_cchar_p_cchar_p);
	CPPUNIT_TEST(test_setObjectSpaceBoundingBox_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setObjectId_cunsigned);
	CPPUNIT_TEST(test_setScreenSpaceFramelet_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_registerObjectSpaceBoundingBox_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Renderer::Renderer().
	 */
	void test_constructor();

	/** 
	 Tests constructor Renderer::Renderer(RendererPlugin * rendererPlugin, void * rendererHandle, OpenGLRenderingEngine * parent).
	 */
	void test_constructor_RendererPlugin_p_void_p_OpenGLRenderingEngine_p();

	/** 
	 Tests destructor Renderer::~Renderer().
	 */
	void test_destructor();

	//
	// Method tests
	//

	/** 
	 Tests method Renderer::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method Renderer::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method Renderer::resize(const u32 width, const u32 height).
	 */
	void test_resize_cu32_cu32();

	/** 
	 Tests method Renderer::getSortOrder().
	 */
	void test_getSortOrder();

	/** 
	 Tests method Renderer::render().
	 */
	void test_render();

	/** 
	 Tests method Renderer::setMiscParam(const char * name, const char * value).
	 */
	void test_setMiscParam_cchar_p_cchar_p();

	/** 
	 Tests method Renderer::setObjectSpaceBoundingBox(const double x0, const double y0, const double z0, const double x1, const double y1, const double z1).
	 */
	void test_setObjectSpaceBoundingBox_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method Renderer::setObjectId(const unsigned objectId).
	 */
	void test_setObjectId_cunsigned();

	/** 
	 Tests method Renderer::setScreenSpaceFramelet(const double u0, const double v0, const double u1, const double v1).
	 */
	void test_setScreenSpaceFramelet_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method Renderer::registerObjectSpaceBoundingBox(const double x0, const double y0, const double z0, const double x1, const double y1, const double z1).
	 */
	void test_registerObjectSpaceBoundingBox_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble();

  };

}

#endif
