
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestRendererPlugin.h
/// Test case declaration for class RendererPlugin.

#ifndef __TEST_RENDERERPLUGIN_H__
#  define __TEST_RENDERERPLUGIN_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMRendererPlugin.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::RendererPlugin. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::RendererPlugin of project ParCompMark.
   */

  class TestRendererPlugin:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestRendererPlugin);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cPlugin__PluginType_cstd__string_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test__initializeSpecific);
	CPPUNIT_TEST(test__finalizeSpecific);
	CPPUNIT_TEST(test_createRenderer_GLXRenderWindow__Pointer_OpenGLRenderingEngine_p);
	CPPUNIT_TEST(test_destroyRenderer_Renderer_p);
	CPPUNIT_TEST(test_resize_void_p_cu32_cu32);
	CPPUNIT_TEST(test_getSortOrder_void_p);
	CPPUNIT_TEST(test_render_void_p_cdouble_cunsigned);
	CPPUNIT_TEST(test_setMiscParam_void_p_cchar_p_cchar_p);
	CPPUNIT_TEST(test_setObjectSpaceBoundingBox_void_p_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_setObjectId_void_p_cunsigned);
	CPPUNIT_TEST(test_setScreenSpaceFramelet_void_p_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test_registerObjectSpaceBoundingBox_void_p_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble);
	CPPUNIT_TEST(test__bufferGetter_cvoid_p_cchar_p_void_p);
	CPPUNIT_TEST(test__setBufferGetter);
	CPPUNIT_TEST(test__getNeededOpenGLExts);
	CPPUNIT_TEST(test__initFastFunctions);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor RendererPlugin::RendererPlugin(const Plugin::PluginType &type, const std::string &name, const std::string &filename).
	 */
	void test_constructor_cPlugin__PluginType_cstd__string_cstd__string();

	/** 
	 Tests destructor RendererPlugin::~RendererPlugin().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method RendererPlugin::_bufferGetter(const void * pRenderer, const char * bufferName, void * pBufferData).
	 */
	void test__bufferGetter_cvoid_p_cchar_p_void_p();

	//
	// Method tests
	//

	/** 
	 Tests method RendererPlugin::createRenderer(GLXRenderWindow::Pointer window, OpenGLRenderingEngine * parent).
	 */
	void test_createRenderer_GLXRenderWindow__Pointer_OpenGLRenderingEngine_p();

	/** 
	 Tests method RendererPlugin::destroyRenderer(Renderer * renderer).
	 */
	void test_destroyRenderer_Renderer_p();

	/** 
	 Tests method RendererPlugin::resize(void * rendererHandle, const u32 &width, const u32 &height).
	 */
	void test_resize_void_p_cu32_cu32();

	/** 
	 Tests method RendererPlugin::getSortOrder(void * rendererHandle).
	 */
	void test_getSortOrder_void_p();

	/** 
	 Tests method RendererPlugin::render(void * rendererHandle, const double &time, const unsigned &frame).
	 */
	void test_render_void_p_cdouble_cunsigned();

	/** 
	 Tests method RendererPlugin::setMiscParam(void * rendererHandle, const char * &name, const char * &value).
	 */
	void test_setMiscParam_void_p_cchar_p_cchar_p();

	/** 
	 Tests method RendererPlugin::setObjectSpaceBoundingBox(void * rendererHandle, const double &x0, const double &y0, const double &z0, const double &x1, const double &y1, const double &z1).
	 */
	void test_setObjectSpaceBoundingBox_void_p_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method RendererPlugin::setObjectId(void * rendererHandle, const unsigned &objectId).
	 */
	void test_setObjectId_void_p_cunsigned();

	/** 
	 Tests method RendererPlugin::setScreenSpaceFramelet(void * rendererHandle, const double &u0, const double &v0, const double &u1, const double &v1).
	 */
	void test_setScreenSpaceFramelet_void_p_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method RendererPlugin::registerObjectSpaceBoundingBox(void * rendererHandle, const double &x0, const double &y0, const double &z0, const double &x1, const double &y1, const double &z1).
	 */
	void test_registerObjectSpaceBoundingBox_void_p_cdouble_cdouble_cdouble_cdouble_cdouble_cdouble();

	/** 
	 Tests method RendererPlugin::_initializeSpecific().
	 */
	void test__initializeSpecific();

	/** 
	 Tests method RendererPlugin::_finalizeSpecific().
	 */
	void test__finalizeSpecific();

	/** 
	 Tests method RendererPlugin::_setBufferGetter().
	 */
	void test__setBufferGetter();

	/** 
	 Tests method RendererPlugin::_getNeededOpenGLExts().
	 */
	void test__getNeededOpenGLExts();

	/** 
	 Tests method RendererPlugin::_initFastFunctions().
	 */
	void test__initFastFunctions();

  };

}

#endif
