
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestSqVM.h
/// Test case declaration for class SqVM.

#ifndef __TEST_SQVM_H__
#  define __TEST_SQVM_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMSqVM.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::SqVM. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::SqVM of project ParCompMark.
   */

  class TestSqVM:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestSqVM);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_printFunction___HSQUIRRELVM_cSQChar_p_);
	CPPUNIT_TEST(test_runScriptFromFile_cstd__string_cstd__string_cstd__list___std__string___cbool);
	CPPUNIT_TEST(test_runScriptFromString_cstd__string_cstd__string_cstd__string_cstd__list___std__string___cbool);
	CPPUNIT_TEST(test_runScriptByName_cstd__string_cstd__list___std__string__);
	CPPUNIT_TEST(test_activate);
	CPPUNIT_TEST(test_deactivate);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_createScript_cstd__string_cbool_cstd__string_cbool);
	CPPUNIT_TEST(test_findScript_cstd__string);
	CPPUNIT_TEST(test_findOrAddScript_cstd__string_cbool_cstd__string_cbool);
	CPPUNIT_TEST(test_compileAndExecuteScript_SqVM__Script__Pointer);
	CPPUNIT_TEST(test_setParameters_SqVM__Script__Pointer_cstd__list___std__string__);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor SqVM::SqVM(const std::string &name).
	 */
	void test_constructor_cstd__string();

	/** 
	 Tests destructor SqVM::~SqVM().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method SqVM::printFunction(::HSQUIRRELVM sqVM, const SQChar * s, ...).
	 */
	void test_printFunction___HSQUIRRELVM_cSQChar_p_();

	//
	// Method tests
	//

	/** 
	 Tests method SqVM::runScriptFromFile(const std::string &filename, const std::string &mainMethod = main, const std::list < std::string > &parameters = std::list < std::string >(), const bool &hasReturn = false).
	 */
	void test_runScriptFromFile_cstd__string_cstd__string_cstd__list___std__string___cbool();

	/** 
	 Tests method SqVM::runScriptFromString(const std::string &scriptString, const std::string &scriptName = , const std::string &mainMethod = main, const std::list < std::string > &parameters = std::list < std::string >(), const bool &hasReturn = false).
	 */
	void test_runScriptFromString_cstd__string_cstd__string_cstd__string_cstd__list___std__string___cbool();

	/** 
	 Tests method SqVM::runScriptByName(const std::string &scriptName, const std::list < std::string > &parameters = std::list < std::string >()).
	 */
	void test_runScriptByName_cstd__string_cstd__list___std__string__();

	/** 
	 Tests method SqVM::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method SqVM::activate().
	 */
	void test_activate();

	/** 
	 Tests method SqVM::deactivate().
	 */
	void test_deactivate();

	/** 
	 Tests method SqVM::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method SqVM::createScript(const std::string &scriptName = , const bool &dynamic = false, const std::string &mainMethod = main, const bool &hasReturn = false).
	 */
	void test_createScript_cstd__string_cbool_cstd__string_cbool();

	/** 
	 Tests method SqVM::findScript(const std::string &scriptName).
	 */
	void test_findScript_cstd__string();

	/** 
	 Tests method SqVM::findOrAddScript(const std::string &scriptName, const bool &dynamic = false, const std::string &mainMethod = main, const bool &hasReturn = false).
	 */
	void test_findOrAddScript_cstd__string_cbool_cstd__string_cbool();

	/** 
	 Tests method SqVM::compileAndExecuteScript(SqVM::Script::Pointer &script).
	 */
	void test_compileAndExecuteScript_SqVM__Script__Pointer();

	/** 
	 Tests method SqVM::setParameters(SqVM::Script::Pointer &script, const std::list < std::string > &parameters).
	 */
	void test_setParameters_SqVM__Script__Pointer_cstd__list___std__string__();

  };

}

#endif
