
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestStringConverter.h
/// Test case declaration for class StringConverter.

#ifndef __TEST_STRINGCONVERTER_H__
#  define __TEST_STRINGCONVERTER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMStringConverter.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::StringConverter. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::StringConverter of project ParCompMark.
   */

  class TestStringConverter:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestStringConverter);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_toString_cu8);
	CPPUNIT_TEST(test_toString_cu16);
	CPPUNIT_TEST(test_toString_cu32);
	CPPUNIT_TEST(test_toString_cs8);
	CPPUNIT_TEST(test_toString_cs16);
	CPPUNIT_TEST(test_toString_cs32);
	CPPUNIT_TEST(test_toString_cReal_cs32_cs32);
	CPPUNIT_TEST(test_toString_cbool);
	CPPUNIT_TEST(test_toString_cvoid_p);
	CPPUNIT_TEST(test_toU32_cstd__string);
	CPPUNIT_TEST(test_toReal_cstd__string);
	CPPUNIT_TEST(test_parsePointer_cstd__string);
	CPPUNIT_TEST(test_trim_std__string);
	CPPUNIT_TEST(test_tokenize_cstd__string_cchar_p);
	CPPUNIT_TEST(test__fitFieldWidth_cstd__string_cs32);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Class method tests
	//

	/** 
	 Tests method StringConverter::toString(const u8 &value).
	 */
	void test_toString_cu8();

	/** 
	 Tests method StringConverter::toString(const u16 &value).
	 */
	void test_toString_cu16();

	/** 
	 Tests method StringConverter::toString(const u32 &value).
	 */
	void test_toString_cu32();

	/** 
	 Tests method StringConverter::toString(const s8 &value).
	 */
	void test_toString_cs8();

	/** 
	 Tests method StringConverter::toString(const s16 &value).
	 */
	void test_toString_cs16();

	/** 
	 Tests method StringConverter::toString(const s32 &value).
	 */
	void test_toString_cs32();

	/** 
	 Tests method StringConverter::toString(const Real &value, const s32 &fieldWidth = StringConverter::DEFAULTFIELDWIDTH, const s32 &precision = StringConverter::DEFAULTPRECISION).
	 */
	void test_toString_cReal_cs32_cs32();

	/** 
	 Tests method StringConverter::toString(const bool value).
	 */
	void test_toString_cbool();

	/** 
	 Tests method StringConverter::toString(const void * value).
	 */
	void test_toString_cvoid_p();

	/** 
	 Tests method StringConverter::toU32(const std::string value).
	 */
	void test_toU32_cstd__string();

	/** 
	 Tests method StringConverter::toReal(const std::string value).
	 */
	void test_toReal_cstd__string();

	/** 
	 Tests method StringConverter::parsePointer(const std::string value).
	 */
	void test_parsePointer_cstd__string();

	/** 
	 Tests method StringConverter::trim(std::string &str).
	 */
	void test_trim_std__string();

	/** 
	 Tests method StringConverter::tokenize(const std::string &str, const char * delimiters).
	 */
	void test_tokenize_cstd__string_cchar_p();

	/** 
	 Tests method StringConverter::_fitFieldWidth(const std::string &str, const s32 &fieldWidth).
	 */
	void test__fitFieldWidth_cstd__string_cs32();

  };

}

#endif
