
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestThread.h
/// Test case declaration for class Thread.

#ifndef __TEST_THREAD_H__
#  define __TEST_THREAD_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMThread.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Thread. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Thread of project ParCompMark.
   */

  class TestThread:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestThread);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_initThread_cu32_cu32_cbool_cbool);
	CPPUNIT_TEST(test_threadInitialize);
	CPPUNIT_TEST(test_threadFinalize);
	CPPUNIT_TEST(test_startThread);
	CPPUNIT_TEST(test_joinThread);
	CPPUNIT_TEST(test_shutDownThread);
	CPPUNIT_TEST(test_stopThread);
	CPPUNIT_TEST(test_yield);
	CPPUNIT_TEST(test_getUSTime);
	CPPUNIT_TEST(test_entryPoint_void_p);
	CPPUNIT_TEST(test_thread);
	CPPUNIT_TEST(test_iteration);
	CPPUNIT_TEST(test_task);
	CPPUNIT_TEST(test_wait);
	CPPUNIT_TEST(test_go);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor Thread::Thread(const std::string &name).
	 */
	void test_constructor_cstd__string();

	/** 
	 Tests destructor Thread::~Thread().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method Thread::yield().
	 */
	void test_yield();

	/** 
	 Tests method Thread::getUSTime().
	 */
	void test_getUSTime();

	/** 
	 Tests method Thread::entryPoint(void * arg).
	 */
	void test_entryPoint_void_p();

	//
	// Method tests
	//

	/** 
	 Tests method Thread::initThread(const u32 &iterationNumber = 0, const u32 &expectedFPS = 0, const bool &joinable = false, const bool &waitThread = false).
	 */
	void test_initThread_cu32_cu32_cbool_cbool();

	/** 
	 Tests method Thread::threadInitialize().
	 */
	void test_threadInitialize();

	/** 
	 Tests method Thread::threadFinalize().
	 */
	void test_threadFinalize();

	/** 
	 Tests method Thread::startThread().
	 */
	void test_startThread();

	/** 
	 Tests method Thread::joinThread().
	 */
	void test_joinThread();

	/** 
	 Tests method Thread::shutDownThread().
	 */
	void test_shutDownThread();

	/** 
	 Tests method Thread::stopThread().
	 */
	void test_stopThread();

	/** 
	 Tests method Thread::go().
	 */
	void test_go();

	/** 
	 Tests method Thread::thread().
	 */
	void test_thread();

	/** 
	 Tests method Thread::iteration().
	 */
	void test_iteration();

	/** 
	 Tests method Thread::task().
	 */
	void test_task();

	/** 
	 Tests method Thread::wait().
	 */
	void test_wait();

  };

}

#endif
