
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestTimer.h
/// Test case declaration for class Timer.

#ifndef __TEST_TIMER_H__
#  define __TEST_TIMER_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMTimer.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::Timer. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::Timer of project ParCompMark.
   */

  class TestTimer:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestTimer);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_getSystemTime);
	CPPUNIT_TEST(test_getApplicationTime);
	CPPUNIT_TEST(test_getUncorrectedSystemTime);
	CPPUNIT_TEST(test_getTimeString_cbool_cbool_cbool_cbool_cReal);
	CPPUNIT_TEST(test_getDateString_cbool_cbool_cbool_cReal);
	CPPUNIT_TEST(test_getTimeDateString_cbool_cbool_cbool_cbool_cbool_cbool_cbool_cReal);
	CPPUNIT_TEST(test_sleep_cReal);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Class method tests
	//

	/** 
	 Tests method Timer::getSystemTime().
	 */
	void test_getSystemTime();

	/** 
	 Tests method Timer::getApplicationTime().
	 */
	void test_getApplicationTime();

	/** 
	 Tests method Timer::getUncorrectedSystemTime().
	 */
	void test_getUncorrectedSystemTime();

	/** 
	 Tests method Timer::getTimeString(const bool &microseconds = true, const bool &seconds = true, const bool &minutes = true, const bool &hours = true, const Real &correction = 0.0).
	 */
	void test_getTimeString_cbool_cbool_cbool_cbool_cReal();

	/** 
	 Tests method Timer::getDateString(const bool &days = true, const bool &months = true, const bool &years = true, const Real &correction = 0.0).
	 */
	void test_getDateString_cbool_cbool_cbool_cReal();

	/** 
	 Tests method Timer::getTimeDateString(const bool &microseconds = true, const bool &seconds = true, const bool &minutes = true, const bool &hours = true, const bool &days = true, const bool &months = true, const bool &years = true, const Real &correction = 0.0).
	 */
	void test_getTimeDateString_cbool_cbool_cbool_cbool_cbool_cbool_cbool_cReal();

	/** 
	 Tests method Timer::sleep(const Real time).
	 */
	void test_sleep_cReal();

  };

}

#endif
