
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestXDisplay.h
/// Test case declaration for class XDisplay.

#ifndef __TEST_XDISPLAY_H__
#  define __TEST_XDISPLAY_H__

//
// CppUnit include
//
#  include <cppunit/extensions/HelperMacros.h>

//
// Tested class include
//
#  define PARCOMPMARK_TEST
#  include "../../include/PCMXDisplay.h"
#  undef PARCOMPMARK_TEST

//
// Opening namespace of the tested project
//
using namespace ParCompMark;

namespace ParCompMarkTest
{

  /**
	Test suite for ParCompMark::XDisplay. This is a <a href="http://cppunit.sourceforge.net/">CppUnit</a> 
	test suite for class ParCompMark::XDisplay of project ParCompMark.
   */

  class TestXDisplay:public CppUnit::TestFixture
  {

	// First, we declare the suite, passing the class name to the macro:
	/// \skip CPPUNIT_TEST_SUITE
	CPPUNIT_TEST_SUITE(TestXDisplay);

	// Then, we declare each test case of the fixture:
	/// \skip CPPUNIT_TEST
	CPPUNIT_TEST(test_constructor_cstd__string);
	CPPUNIT_TEST(test_destructor);
	CPPUNIT_TEST(test_errorHandler_Display_p_XErrorEvent_p);
	CPPUNIT_TEST(test_initialize);
	CPPUNIT_TEST(test_finalize);
	CPPUNIT_TEST(test_synchronize_cbool);
	CPPUNIT_TEST(test_findBestVisual_cs32_cs32);
	CPPUNIT_TEST(test_getVisualAttribs_XVisualInfo_p_VisualAttribs);
	CPPUNIT_TEST(test_initializeMT);

	// Finally, we end the suite declaration:
	/// \skip CPPUNIT_TEST_SUITE_END
	CPPUNIT_TEST_SUITE_END();

   public:

  /** Sets up context before running a test. */
	void setUp();

  /** Cleans up after the test run. */
	void tearDown();

   protected:

	//
	// Constructor & destructor tests
	//

	/** 
	 Tests constructor XDisplay::XDisplay(const std::string &displayName = ).
	 */
	void test_constructor_cstd__string();

	/** 
	 Tests destructor XDisplay::~XDisplay().
	 */
	void test_destructor();

	//
	// Class method tests
	//

	/** 
	 Tests method XDisplay::errorHandler(Display * display, XErrorEvent * errorEvent).
	 */
	void test_errorHandler_Display_p_XErrorEvent_p();

	//
	// Method tests
	//

	/** 
	 Tests method XDisplay::initialize().
	 */
	void test_initialize();

	/** 
	 Tests method XDisplay::finalize().
	 */
	void test_finalize();

	/** 
	 Tests method XDisplay::synchronize(const bool &discard = false).
	 */
	void test_synchronize_cbool();

	/** 
	 Tests method XDisplay::findBestVisual(const s32 &screenNumber, const s32 &multiSample = XDisplay::IGNOREMULTISAMPLE).
	 */
	void test_findBestVisual_cs32_cs32();

	/** 
	 Tests method XDisplay::getVisualAttribs(XVisualInfo * vInfo, VisualAttribs &attribs).
	 */
	void test_getVisualAttribs_XVisualInfo_p_VisualAttribs();

	/** 
	 Tests method XDisplay::initializeMT().
	 */
	void test_initializeMT();

  };

}

#endif
