
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestConfigOptions.cpp
/// Test case implementation for class ConfigOptions.

#include "../include/TestConfigOptions.h"
#include "../../include/PCMFileSystemManager.h"

namespace ParCompMarkTest
{

  void TestConfigOptions::setUp()
  {
	std::string currentLogFile = "logs/TestConfigOptions.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestConfigOptions::test_constructor()
  {
	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_destructor()
  {
	// Tested at constructor
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestConfigOptions::test_initialize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize option container.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_finalize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Finalize option container.
	 */

	// Tested a initialize
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_hasOption_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Check for option.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());

	// False negative
	CPPUNIT_ASSERT(!options->hasOption("something"));

	// True positive
	s32 val = 1234;

	CPPUNIT_ASSERT_NO_THROW(options->setValue("something", &val, ConfigOptions::INTEGER, ""));
	CPPUNIT_ASSERT(options->hasOption("something"));

	// False negative
	CPPUNIT_ASSERT_NO_THROW(options->removeOption("something"));
	CPPUNIT_ASSERT(!options->hasOption("something"));
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getString_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get option.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());

	// False negative
	CPPUNIT_ASSERT_THROW(options->getString("something"), ParCompMark::Exception);

	// True positive
	CPPUNIT_ASSERT_NO_THROW(options->setString("something", "stuff"));
	CPPUNIT_ASSERT(options->getString("something") == "stuff");

	// False negative
	CPPUNIT_ASSERT_NO_THROW(options->removeOption("something"));
	CPPUNIT_ASSERT_THROW(options->getString("something"), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getBool_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get option.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());

	// False negative
	CPPUNIT_ASSERT_THROW(options->getBool("something"), ParCompMark::Exception);

	// True positive
	CPPUNIT_ASSERT_NO_THROW(options->setBool("something", true));
	CPPUNIT_ASSERT(options->getBool("something") == true);

	// False negative
	CPPUNIT_ASSERT_NO_THROW(options->removeOption("something"));
	CPPUNIT_ASSERT_THROW(options->getBool("something"), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getInteger_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get option.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());

	// False negative
	CPPUNIT_ASSERT_THROW(options->getInteger("something"), ParCompMark::Exception);

	// True positive
	CPPUNIT_ASSERT_NO_THROW(options->setInteger("something", 12345));
	CPPUNIT_ASSERT(options->getInteger("something") == 12345);

	// False negative
	CPPUNIT_ASSERT_NO_THROW(options->removeOption("something"));
	CPPUNIT_ASSERT_THROW(options->getInteger("something"), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getReal_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get option.
	 */

	ConfigOptions::Pointer options;
	CPPUNIT_ASSERT_NO_THROW(options = new ConfigOptions());
	CPPUNIT_ASSERT_NO_THROW(options->initialize());

	// False negative
	CPPUNIT_ASSERT_THROW(options->getReal("something"), ParCompMark::Exception);

	// True positive
	CPPUNIT_ASSERT_NO_THROW(options->setReal("something", 1.2345));
	CPPUNIT_ASSERT(options->getReal("something") == 1.2345);

	// False negative
	CPPUNIT_ASSERT_NO_THROW(options->removeOption("something"));
	CPPUNIT_ASSERT_THROW(options->getReal("something"), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_testString_cstd__string_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Return true if the option exists with the given name and equals to the given value.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_setString_cstd__string_cstd__string_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set option.
	 */

	// Tested at getter
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_setBool_cstd__string_cbool_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set option.
	 */

	// Tested at getter
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_setInteger_cstd__string_cs32_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set option.
	 */

	// Tested at getter
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_setReal_cstd__string_cReal_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set option.
	 */

	// Tested at getter
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_removeOption_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Remove option.
	 */

	// Tested above
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_saveToIniFile_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Create user-level ini file in the application directory (existing file will be overwritten).
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_loadFromIniFile_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Load user-level ini file.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getValue_cstd__string_cConfigOptions__OptionType()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get option value for the specified type (internal method).
	 */

	// Tested above
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_setValue_cstd__string_cvoid_p_cConfigOptions__OptionType_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set option value for the specified type (internal method).
	 */

	// Tested above
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_getOptionTypeSize_cConfigOptions__OptionType()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get size of option type
	 */

	// Tested above
  }

 /*----------------------------------------------------------------------*/

  void TestConfigOptions::test_optionToString_ConfigOptions__Option__Pointer()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Give string representation of the option value
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestConfigOptions);
}
