
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestDynLoad.cpp
/// Test case implementation for class DynLoad.

#include "../include/TestDynLoad.h"
#include "../../include/PCMFileSystemManager.h"

namespace ParCompMarkTest
{

  void TestDynLoad::setUp()
  {
	std::string currentLogFile = "logs/TestDynLoad.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestDynLoad::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestDynLoad::test_constructor_cstd__string()
  {
	//tested in destructor
  }

 /*----------------------------------------------------------------------*/

  void TestDynLoad::test_destructor()
  {
	DynLoad *d = 0;

	CPPUNIT_ASSERT_THROW(d = new DynLoad("cc"), ParCompMark::Exception);

	CPPUNIT_ASSERT_NO_THROW(d = new DynLoad("libm.so.6"));
	CPPUNIT_ASSERT_NO_THROW(delete d);
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestDynLoad::test_hasFunction_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Return true when the dynamic library has a callable function with the specified name.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestDynLoad::test_getFunction_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get a function pointer from dynamic library by name (symbol).
	 */

	DynLoad *d = 0;

	CPPUNIT_ASSERT_NO_THROW(d = new DynLoad("libm.so.6"));

	CPPUNIT_ASSERT_THROW(d->getFunction("cc"), ParCompMark::Exception);

	CPPUNIT_ASSERT_NO_THROW(d->getFunction("sin"));

	CPPUNIT_ASSERT_NO_THROW(delete d);
  }

 /*----------------------------------------------------------------------*/

  void TestDynLoad::test_load()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Load a library. Called by constructor.
	 */

	//tested in destructor
  }

 /*----------------------------------------------------------------------*/

  void TestDynLoad::test_unload()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Load a library. Called by constructor.
	 */

	//tested in destructor
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestDynLoad);
}
