
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestHostInfo.cpp
/// Test case implementation for class HostInfo.

#include "../include/TestHostInfo.h"
#include "../../include/PCMFileSystemManager.h"
//#include "tinyxml/tinyxml.h"

namespace ParCompMarkTest
{

  void TestHostInfo::setUp()
  {
	std::string currentLogFile = "logs/TestHostInfo.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestHostInfo::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestHostInfo::test_constructor_cstd__string_cOutputNode__NodeType()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Creates a host with a specified name.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestHostInfo::test_destructor()
  {

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  //
  // Class method tests
  //

  void TestHostInfo::test_parseXML_TiXmlElement_p()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Creates a HostInfo class instance based on xml
	 */

	//TODO: test with other HostInfo that contains more parameters

	std::
	 string
	 xml
	 ("<info:iltomee-desktop name=\"iltomee-desktop\"><info:param name=\"pc-number-networks\" value=\"1\"/><info:network-ids-names><info:network-id-name id=\"14002838\" name=\"TCP/IP\"/></info:network-ids-names><info:param name=\"pc-vendor\" value=\"CEI Inc\"/><info:param name=\"pc-extensions\" value=\"PC_EXT_IO_COUNT\"/><info:param name=\"pc-volatile-framelet-limit\" value=\"0\"/><info:param name=\"pc-retain-ouput-limit\" value=\"1\"/></info:iltomee-desktop>");

	TiXmlDocument tiDocument;

	tiDocument.Parse(xml.c_str());

	TiXmlElement *tiHostInfo = tiDocument.RootElement();

	HostInfo::Pointer myHostInfo = HostInfo::parseXML(tiHostInfo);

	CPPUNIT_ASSERT(myHostInfo->mPCNumNetworks == 1);
	CPPUNIT_ASSERT(myHostInfo->mNetIDNames->get("TCP/IP")->ID == 14002838);
	CPPUNIT_ASSERT(myHostInfo->mPCVendor.compare("CEI Inc") == 0);
	CPPUNIT_ASSERT(myHostInfo->mPCExtensions.compare("PC_EXT_IO_COUNT") == 0);
	CPPUNIT_ASSERT(myHostInfo->mVolatileFrameletLimit == 0);
	CPPUNIT_ASSERT(myHostInfo->mRetainOutputLimit == 1);

  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestHostInfo::test_refreshData()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Refresh output node data.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestHostInfo::test_refreshCPUs_OutputNode__Pointer()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Refresh CPUs data on the hosts.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestHostInfo::test_refreshGPUs_OutputNode__Pointer()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Refresh GPUs data on the hosts.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestHostInfo);
}
