
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestLogger.cpp
/// Test case implementation for class Logger.

#include "../include/TestLogger.h"

namespace ParCompMarkTest
{

  void TestLogger::setUp()
  {
	// TestLogger does not need a logger :)
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestLogger::test_constructor_cstd__string()
  {
	/*
	 * You have to verify the following:
	 * 
	 * Create logger.
	 */
	Logger *logger = 0;

	CPPUNIT_ASSERT_NO_THROW(logger = new Logger("logs/TestLogger1.log"));
	CPPUNIT_ASSERT_NO_THROW(delete logger);
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::test_destructor()
  {
	// Tested above
  }

 /*----------------------------------------------------------------------*/

  //
  // Class method tests
  //

  void TestLogger::test_translateLogLevel_cLogLevel()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Translate log level to human readable.
	 */

	CPPUNIT_ASSERT_NO_THROW(Logger::translateLogLevel(Logger::FATAL));
	CPPUNIT_ASSERT_NO_THROW(Logger::translateLogLevel(Logger::ERROR));
	CPPUNIT_ASSERT_NO_THROW(Logger::translateLogLevel(Logger::WARNING));
	CPPUNIT_ASSERT_NO_THROW(Logger::translateLogLevel(Logger::NOTICE));
	CPPUNIT_ASSERT_NO_THROW(Logger::translateLogLevel(Logger::DEBUG));
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::test_translateException_cException()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Translate an exception to human readable.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestLogger::test_initialize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initializes the logger.
	 */

	Logger *logger = 0;

	CPPUNIT_ASSERT_NO_THROW(logger = new Logger("logs/TestLogger2.log"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->initialize());
	CPPUNIT_ASSERT_NO_THROW(delete logger);
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::test_log_cLogLevel_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Logs a message. (Does not care about multiline messages, because of performance).
	 */

	Logger *logger = 0;

	CPPUNIT_ASSERT_NO_THROW(logger = new Logger("logs/TestLogger3.log"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setFileLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->initialize());
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Logger::DEBUG, "This is a debug message"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Logger::NOTICE, "This is a notice message"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Logger::WARNING, "This is a warning message"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Logger::ERROR, "This is a error message"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Logger::FATAL, "This is a fatal message"));
	CPPUNIT_ASSERT_NO_THROW(delete logger);
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::test_logMultiLine_cLogLevel_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Logs a message. Handles multiline messages too. 
	 */

	Logger *logger = 0;

	CPPUNIT_ASSERT_NO_THROW(logger = new Logger("logs/TestLogger4.log"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setFileLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->initialize());
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->
				logMultiLine(Logger::DEBUG,
					  "[1] This is the first line\n[2] This is the second line\n[3] This is the third line. That's all."));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->logMultiLine(Logger::DEBUG, "\n\nThere are empty lines.\n\n\n"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->logMultiLine(Logger::DEBUG, "Line without backspash-n char."));
	CPPUNIT_ASSERT_NO_THROW(delete logger);
  }

 /*----------------------------------------------------------------------*/

  void TestLogger::test_log_cException()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Logs an exception.
	 */

	Logger *logger = 0;

	CPPUNIT_ASSERT_NO_THROW(logger = new Logger("logs/TestLogger5.log"));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->setFileLogLevel(Logger::DEBUG));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->initialize());
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->log(Exception()));
	CPPUNIT_ASSERT_NO_THROW(Logger::getInstance()->
				log(Exception
				 (Exception::INVALID_OPERATION_ERROR, "It\'s a pity, is an exception", "sample.cpp",
				  "sample()", 1234)));
	CPPUNIT_ASSERT_NO_THROW(delete logger);
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestLogger);
}
