
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestMutex.cpp
/// Test case implementation for class Mutex.

#include "../include/TestMutex.h"

namespace ParCompMarkTest
{

  void TestMutex::setUp()
  {
	// DummyLock cannot log, beacuse Lock cannot
  }

 /*----------------------------------------------------------------------*/

  void TestMutex::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestMutex::test_constructor()
  {
	Mutex *m = 0;

	CPPUNIT_ASSERT_NO_THROW(m = new Mutex());
	CPPUNIT_ASSERT_MESSAGE("Mutex is locked by default", !m->getLocked());
	CPPUNIT_ASSERT_NO_THROW(delete m);
  }

 /*----------------------------------------------------------------------*/

  void TestMutex::test_destructor()
  {
	// Upper tested
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestMutex::test_lock()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Lock the mutex.
	 */

	Mutex m;

	CPPUNIT_ASSERT_NO_THROW(m.lock());
	CPPUNIT_ASSERT_MESSAGE("Mutex cannot be locked", m.getLocked());
  }

 /*----------------------------------------------------------------------*/

  void TestMutex::test_trylock()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Try locking the mutex.
	 */

	Mutex m;

	while(!m.getLocked())
	 CPPUNIT_ASSERT_NO_THROW(m.lock());
	CPPUNIT_ASSERT_MESSAGE("Mutex cannot be locked", m.getLocked());
  }

 /*----------------------------------------------------------------------*/

  void TestMutex::test_unlock()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Unlock the mutex.
	 */

	Mutex m;

	CPPUNIT_ASSERT_NO_THROW(m.lock());
	CPPUNIT_ASSERT_NO_THROW(m.unlock());
	CPPUNIT_ASSERT_MESSAGE("Mutex cannot be unlocked", !m.getLocked());
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestMutex);
}
