
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestNetServer.cpp
/// Test case implementation for class NetServer.

#include "../include/TestNetServer.h"

namespace ParCompMarkTest
{

  void TestNetServer::setUp()
  {

	std::string currentLogFile = "logs/TestNetServer.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}

  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestNetServer::test_constructor_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Create network class.
	 */

	NetServer::Pointer server;

	CPPUNIT_ASSERT_NO_THROW(server = new NetServer("Server"));
	Timer::sleep(0.2);
	CPPUNIT_ASSERT_NO_THROW(server.kill());

	CPPUNIT_ASSERT_NO_THROW(server = new NetServer("Server"));
	CPPUNIT_ASSERT_NO_THROW(server.kill());
  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::test_destructor()
  {

	//Tested in constructor
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestNetServer::test_initialize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize the class (server thread, broadcast send).
	 */

	//Tested in constructor
  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::test_finalize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize the class (broadcast recieve thread).
	 */

	//Tested in constructor
  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::test_sendBroadcastMessage_cstd__string_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Send a broadcast message.
	 */

	NetServer::Pointer server(new NetServer("Server"));
	NetClient::Pointer client(new NetClient("Client"));

	server->startThread();
	client->startThread();

	Timer::sleep(0.2);

	CPPUNIT_ASSERT_NO_THROW(server->sendBroadcastMessage("LOGIN", "152.66.243.165"));

	Timer::sleep(1.0);

  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::test_buildCluster()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Build cluster information.
	 */

/**
	NetServer::Pointer server(new NetServer("Server"));
	NetClient::Pointer client(new NetClient("Client"));

	Cluster::createInstance();

	server->startThread();
	client->startThread();

	Timer::sleep(0.2);

	CPPUNIT_ASSERT_NO_THROW(server->buildCluster());

	Timer::sleep(0.2);
	*/
  }

 /*----------------------------------------------------------------------*/

  void TestNetServer::test_task()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Thread for accept TCP/IP connection.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestNetServer);
}
