
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestSqVM.cpp
/// Test case implementation for class SqVM.

#include "../include/TestSqVM.h"

namespace ParCompMarkTest
{

  void TestSqVM::setUp()
  {
	std::string currentLogFile = "logs/TestSqVM.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestSqVM::test_constructor_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Create Squirrel virtual machine.
	 */

	SqVM::Pointer sqvm;
	CPPUNIT_ASSERT_NO_THROW(sqvm = new SqVM("TestVM"));
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_destructor()
  {
	// Tested at constructor
  }

 /*----------------------------------------------------------------------*/

  //
  // Class method tests
  //

  void TestSqVM::test_printFunction___HSQUIRRELVM_cSQChar_p_()
  {

	/*
	 * You have to verify the following:
	 * 
	 * The print function of the virtual machine.
	 * This function is used by the built-in function '::print()' to output text.
	 * 
	 */

	// No need to test
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestSqVM::test_runScriptFromFile_cstd__string_cstd__string_cstd__list___std__string___cbool()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Execute script loaded from file. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 */

	// Create VM
	SqVM::Pointer sqvm(new SqVM("TestVM"));
	Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG);

	for(int i = 0; i < 10; i++)
	{

	 // Test with a proper script with main method
	 CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptFromFile("scripts/helloWorld.nut"));

	 // Test with a proper script without main method
	 /*
	  * CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptFromFile("scripts/hello.nut", SqVM::NOMAINMETHOD));
	  * 
	  * // Test with a script containing syntax error
	  * CPPUNIT_ASSERT_THROW(sqvm->runScriptFromFile("scripts/syntaxError.nut"), ParCompMark::Exception);
	  */

	}
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_runScriptFromString_cstd__string_cstd__string_cstd__string_cstd__list___std__string___cbool()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Execute script from the given string. 
	 * If the scriptName is not empty the VM store this script into the class level store for better performance.
	 * Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 */

	// Create VM
	SqVM::Pointer sqvm(new SqVM("TestVM"));
	Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG);

	// Test with a proper script without main method and name
	CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptFromString("print(\"Hello World!\\n\");", "", SqVM::NOMAINMETHOD));

	// Test with a proper script with main method and without name
	CPPUNIT_ASSERT_NO_THROW(sqvm->
				runScriptFromString("function main() { print(\"Hello World!\\n\"); }", "", "main"));

	// Test with a proper script with main method and name
	CPPUNIT_ASSERT_NO_THROW(sqvm->
				runScriptFromString
				("function main() { print(\"Hello World! This is my dynamic script. I will call it later, too.\\n\"); }",
				 "myScript", "main"));

	// Test with a script has errors
	CPPUNIT_ASSERT_THROW(sqvm->runScriptFromString("sdhfusid huisfh usifh suf", "", SqVM::NOMAINMETHOD),
			  ParCompMark::Exception);

	// Recall the script
	CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptFromString("", "myScript", "main"));
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_runScriptByName_cstd__string_cstd__list___std__string__()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Execute a previously stored script.
	 */

	// Create VM
	SqVM::Pointer sqvm(new SqVM("TestVM"));
	Logger::getInstance()->setConsoleLogLevel(Logger::DEBUG);

	// Test with a proper script with main method
	Logger::getInstance()->log(Logger::DEBUG, "Calling `scripts/helloWorld.nut\' for 1st.");
	CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptFromFile("scripts/helloWorld.nut"));

	// Test with a proper script with main method and name
	Logger::getInstance()->log(Logger::DEBUG, "Calling `myScript\' for 1st.");
	CPPUNIT_ASSERT_NO_THROW(sqvm->
				runScriptFromString
				("function dynamicMain() { print(\"Hello World! This is my dynamic script. I will call it later, too.\\n\"); }",
				 "myScript", "dynamicMain"));

	Logger::getInstance()->log(Logger::DEBUG, "Calling `scripts/helloWorld.nut\' again");
	CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptByName("scripts/helloWorld.nut"));
	Logger::getInstance()->log(Logger::DEBUG, "Calling `myScript\' again");
	CPPUNIT_ASSERT_NO_THROW(sqvm->runScriptByName("myScript"));
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_finalize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Finalize virtual machine.
	 */

	SqVM::Pointer sqvm1(new SqVM("TestVM1"));
	SqVM::Pointer sqvm2(new SqVM("TestVM2"));

	CPPUNIT_ASSERT_NO_THROW(sqvm1->activate());
	CPPUNIT_ASSERT_NO_THROW(sqvm1->deactivate());
	CPPUNIT_ASSERT_NO_THROW(sqvm2->activate());
	CPPUNIT_ASSERT_NO_THROW(sqvm2->deactivate());
	CPPUNIT_ASSERT_NO_THROW(sqvm1->activate());
	CPPUNIT_ASSERT_NO_THROW(sqvm1->deactivate());
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_activate()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Activate this VM. This virtual machine will be selected to operate.
	 */

	// Tested at activate method
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_deactivate()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Deactivate this VM. This virtual machine will go to sleep and let other VMs to be activated.
	 */

	// Tested at activate method
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_initialize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize virtual machine.
	 */

	// Tested at activate method
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_createScript_cstd__string_cbool_cstd__string_cbool()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Create script object with the specified name and entry point.
	 * The dynamic flag is also set, and the returned script object is locked by default.
	 * 
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_findScript_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Finds a prevously added script.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_findOrAddScript_cstd__string_cbool_cstd__string_cbool()
  {

	/*
	 * You have to verify the following:
	 * 
	 * If the script exists with the given name then returns, if not then adds it to the internal class level container and return it.
	 */

	SqVM::Pointer sqvm(new SqVM("TestVM"));
	SqVM::Script::Pointer script0, script1;

	// Test attributes
	CPPUNIT_ASSERT_NO_THROW(script0 = sqvm->findOrAddScript("test0.nut"));
	CPPUNIT_ASSERT(script0->compiled == false);
	CPPUNIT_ASSERT(script0->name == "test0.nut");
	// TODO: not locked, why?
	// CPPUNIT_ASSERT(script0.getLocked());

	// Test caching
	CPPUNIT_ASSERT_NO_THROW(script1 = sqvm->findOrAddScript("test0.nut"));
	CPPUNIT_ASSERT(script0 == script1);

	// Test another script
	CPPUNIT_ASSERT_NO_THROW(script1 = sqvm->findOrAddScript("test1.nut"));
	CPPUNIT_ASSERT(script0 != script1);

	// Unlock scripts
	CPPUNIT_ASSERT_NO_THROW(script0.unlock());
	CPPUNIT_ASSERT_NO_THROW(script1.unlock());
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_compileAndExecuteScript_SqVM__Script__Pointer()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Compile and execute the script.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestSqVM::test_setParameters_SqVM__Script__Pointer_cstd__list___std__string__()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Set parameters of a script.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestSqVM);
}
